/*
 * pettanR system.js
 *   version 0.5.12
 * 
 * 
 * Type
 * 
 * Util
 * 
 * UA
 * 
 * gadgetOS
 *   author:
 *     itozyun
 *   licence:
 *     3-clause BSD
 */


/*
 * http://pettanr.sourceforge.jp/test/type.html
 */
	var Type = {
		isObject : function(v) {
			return v !== null && typeof v === 'object';
		},
		isFunction : function(v) {
			return typeof v === 'function';
		},
		isArray : function(v) {
			return Object.prototype.toString.call(v) === "[object Array]";
		},
		isBoolean : function(v) {
			return typeof v === 'boolean';
		},
		isString : function(v) {
			return typeof v === 'string';
		},
		isNumber : function(v) {
			return typeof v === 'number';
		},
		isFinite : function(v){
			return Type.isNumber(v) === true && isFinite(v);
		},
		isHTMLElement : function(v){
			if( 'HTMLElement' in window ){
				Type.isHTMLElement = function(v){
					return v instanceof HTMLElement;
				}
			} else {
				Type.isHTMLElement = function(v) {
					if( Type.isObject(v) === false ){
						return false;
					}
					var r;
					if(v && v.nodeType === 1){
						try{
							r = v.cloneNode(false);
						} catch(n) {
							return false;
						}
						if(r === v) return false;
						try{
							r.nodeType = 9;
							return r.nodeType === 1;
						} catch(n) {}
						return true;
					}
					return false;
				}
			}
			return Type.isHTMLElement(v);
		},
		/*
		isElementCollection : function(v) {
			return (Object.prototype.toString.call(v) === "[object HTMLCollection]");
		},
		*/
		isNull : function(v) {
			return v === null;
		},
		isUndefined : function(v) {
			return typeof v === 'undefined';
		}
	};

/*
 * UA
 * 
 *   detect userAgent
 *   detect plug in
 */
	var UA = ( function(){
		var ua = (function(){
				var acme = {};
				
				var n    = navigator;
				var dua  = n.userAgent;
				var dav  = n.appVersion;
				var tv   = parseFloat(dav);
				acme.isOpera  = (dua.indexOf("Opera") >= 0) ? tv: undefined;
				acme.isKhtml  = (dav.indexOf("Konqueror") >= 0) ? tv : undefined;
				acme.isWebKit = parseFloat(dua.split("WebKit\/")[1]) || undefined;
				acme.isChrome = parseFloat(dua.split("Chrome\/")[1]) || undefined;
				acme.isGecko  = (dua.indexOf("Gecko\/") >= 0) ? parseFloat(dua.split("rv:")[1].replace( /^(\d*\.\d*)\.(\d*)/, '$1$2' )) : undefined;
				var index = Math.max(dav.indexOf("WebKit"), dav.indexOf("Safari"), 0);
				if(index && !acme.isChrome){
					acme.isSafari = parseFloat(dav.split("Version/")[1]);
					if(!acme.isSafari || parseFloat(dav.substr(index + 7)) <= 419.3){
						acme.isSafari = 2;
					}
				}
				if(document.all && !acme.isOpera){
					acme.isIE = parseFloat(dav.split("MSIE ")[1]) || undefined;
				}
				
				return acme;
			})(),
			isIE = navigator.userAgent.toLowerCase().indexOf( 'msie') !== -1,
			ieVersion = isIE === true ? parseInt( navigator.appVersion.toLowerCase().replace( /.*msie[ ]/, '').match( /^[0-9]+/)) : 0,
			ieRenderingVersion = ieVersion === 8 ? document.documentMode : ieVersion,
			isStanderdMode = document.compatMode === 'CSS1Compat',
			ActiveX = ( function(){
				if( isIE === false || ieVersion > 8 ) return false;
				var	b = document.body,
					c = b.className || '',
					x,
					ret = undefined, //pettanr.URL_PARAMS.ActiveX,
					ns = 'pettanr-ActiveX-',
					enabled = 'enabled',
					disabled = 'disabled';
				if( ret !== true && ret !== false){
					if( b.className.indexOf( ns + enabled ) !== -1 )  return true;
					if( b.className.indexOf( ns + disabled ) !== -1 ) return false;
					x = document.createElement( 'div' );
					b.appendChild(x);
					x.style.cssText = 'width:1px;height:1px;line-height:1px;filter:progid:DXImageTransform.Microsoft.Shadow()';
					ret = x.offsetHeight > 1;
					b.removeChild(x);
				}
				b.className += [ c !== '' ? ' ' : c, ns, ret === true ? enabled : disabled ].join( '');
				return ret;
			})(),
			VML = ( function(){
				if( ActiveX === false || isIE === false || ieVersion > 8) return false;
				var globalObjectName = 'detect_activex',//Util.createGlobalUniqueName(),
					script,
					id = 'detectVML';
				document.write( [ '<!--[if gte vml 1]><script id="', id, '">window', '.', globalObjectName, '=1;<\/script><![endif]-->'].join( ''));
				if( window[ globalObjectName ] === 1 ){
					script = document.getElementById( id );
					script.parentNode.removeChild( script );
					window[ globalObjectName ] = null;
					return true;
				}
				return false;
			})(),
			isStandAloneMode = ( function(){
				if( isIE === false) return false;
				if( VML === true) return false;
				var globalObjectName = 'detect_standalone', //Util.createGlobalUniqueName(),
					script,
					id = 'detectStandAlone';
				document.write( [ '<!--[if IE ', Math.floor( ieVersion ), ']><script id="', id, '">window', '.', globalObjectName, '=1;<\/script><![endif]-->'].join( ''));
				if( window[globalObjectName] === 1){
					script = document.getElementById( id);
					script.parentNode.removeChild( script);
					window[globalObjectName] = null;
					return false;
				}
				return true;
			})();
		return {
			IE:					ua.isIE,
			GECKO:				ua.isGecko,
			OPERA:				ua.isOpera,
			KHTML:				ua.isKhtml,
			WEBKIT:				ua.isWebKit,
			CHROME:				ua.isChrome,
			isIE:				isIE,
			ieVersion:			ieVersion,
			ieRenderingVersion:	ieRenderingVersion,
			isStanderdMode:		isStanderdMode,
			ACTIVEX:            ActiveX,
			VML:                VML,
			STANDALONE:         isStandAloneMode,
			VENDER_PREFIX: ( function() {
				var ua = navigator.userAgent.toLowerCase();
				if ( ua.indexOf('opera') !== -1 ){
					return 'O';
				} else if ( ua.indexOf('msie') !== -1 ){
					return 'ms';
				} else if ( ua.indexOf('webkit') !== -1 ){
					return 'webkit';
				} else if ( navigator.product === 'Gecko' ){
					return 'Moz';
				}
				return '';
			})(),
			startVML: function(){
				delete UA.startVML;
				if( UA.VML !== true) return false;
				if (!document.namespaces["v"]) {
			        document.namespaces.add("v", "urn:schemas-microsoft-com:vml", "#default#VML");
			    }
			    document.createStyleSheet().cssText = "v\:shape,v\:image{behavior:url(#default#VML);display:block;};";
			}
		}
	})();


/* ----------------------------------------------------
 * Util
 * 
 *   extend( baseInstance, extend)
 *   cleanCommnetNode()
 *   cleanElement()
 *   getElementSize( _elm)
 *   getImageSize()
 *   getAbsolutePath()
 *   getGrobalObjectName()
 * 
 */
	var Util = ( function(){
		var doc  = window.document,
			body = doc.getElementsByTagName( 'body' )[ 0 ];
			
		var ELM_SIZE_GETTER = ( function(){
				var ret = doc.createElement( 'DIV' );
				body.appendChild( ret );
				
				ret.id            = 'elmSizeGetter';
				ret.style.cssText = 'position:absolute;left:0;top:-9999px;visibility:hidden';
				
				body.removeChild( ret );
				return ret;
			})(),
			CLEAN_TARGET_ELEMENT = 'script,style,object,applet,embed,iframe,frame,base,bgsound,frameset,listing'.split( ',' );
		
		var UNIT_RATIO = ( function( elm ){
				var ret = {},
					unit,
					units = 'px,cm,mm,in,pt,pc'.split( ',' );
				
				body.appendChild( elm );
				
				for( var i=units.length; i; ){
					unit = units[ --i ];
					elm.style.width = 1000 + unit;
					ret[ unit ] = elm.offsetWidth / 1000;
				};
				body.removeChild( elm );
				elm.style.cssText = '';
				
				return ret;
			})( ELM_SIZE_GETTER );
			
		var FONT_SIZE_RATIO = ( function( elm ){
				var ret = {},
					size,
					list = 'xx-large,x-large,large,larger,medium,small,smaller,x-small,xx-small'.split( ',' );
				
				body.appendChild( elm );
				elm.style.cssText = 'font-size:100px;line-height:1em;';
				elm.appendChild( document.createTextNode( 'X' ) );
				
				base = elm.offsetHeight;
				
				for( var i = list.length; i; ){
					size = list[ --i ];
					elm.style.fontSize = size;
					ret[ size ] = elm.offsetHeight;// / base;
				};
				body.removeChild( elm );
				elm.style.cssText = '';
				elm.removeChild( elm.firstChild );
				
				return ret;
			})( ELM_SIZE_GETTER );
		
		var SPECIAL = ( function(){
			var ret = {};
			if( UA.isIE === true && UA.ieVersion < 9 ){
				if( UA.ACTIVEX === true ){
					// ret.opacity    = 'ActiveXOpacity';
					ret.getFilters = function( style ){
						var filters = ( style.filter || '' ).split( ') ' ),
							ret     = {},
							i       = filters.length,
							filter, name, props, prop, j, l, key, v;
						for( ; i; ){
							filter   = filters[ --i ].split( ' ' ).join( '' ).split( '(' );
							if( filter.length !== 2 ) continue;
							name     = filter[ 0 ].split( '.' ); // progid:DXImageTransform.Microsoft.Shadow(color=#666666,direction=120,strength=9)
							props    = filter[ 1 ].split( ',' ); // 
							filter   = {};
							for( j = 0, l = props.length; j < l; ++j ){
								prop = props[ j ].split( '=' );
								key  = prop[ 0 ].toLowerCase();
								v    = prop[ 1 ];
								filter[ key ] = v; //v.charAt( 0 ) === '#' ? v : parseInt( v );
							};
							ret[ name[ name.length - 1 ] ] = filter;
						};
						return ret;
					};
				} else {
					ret.opacity = null;
				};
			} else {
				var style = document.documentElement.style;
				ret.opacity =	style.opacity             !== undefined ? 'opacity' : 
								style.MozOpacity          !== undefined ? 'MozOpacity' :
								style.KhtmlOpacity        !== undefined ? 'KhtmlOpacity' :
								style[ '-khtml-opacity' ] !== undefined ? 'KhtmlOpacity' : null;
			};
					
			return ret;
		})();
		
		function cssToObject( css ){
			var ret = {}, i, nv, n, v, parse = Util.parse, isNumber = Type.isNumber;
			if( Type.isString( css ) === true ){
				css = css.split( ';' );
				for( i = css.length; i; ){
					nv    = css[ --i ].split( ':' ); // filter の場合, progid: がくる
					n     = nv.shift();
					if( isNumber( parse( n ) ) === true ) continue;
					v     = nv.join( '' );
					while( v.charAt( 0 ) === ' ' ) v = v.substr( 1 );
					ret[ Util.camelize( n ) ] = parse( v );
				};
			} else {
				for( n in css ){
					if( Type.isNumber( parse( n ) ) === false ) ret[ n ] = parse( css[ n ] );
				};
			};
			
			if( SPECIAL.getFilters ){
				var f = SPECIAL.getFilters( ret );
				ret.filter  = f;
				ret.opacity = f.alpha && f.alpha.opacity ? f.alpha.opacity / 100 : 1;
			} else {
				ret.opacity = SPECIAL.opacity !== null ? ret[ SPECIAL.opacity ] : 1;
			}
			
			return ret;
		};	
		
		/* clean comment node */
		window.setTimeout( function(){
			Util.cleanCommnetNode();
			
			/* clean noscript */

			var noscripts = doc.getElementsByTagName( 'noscript' ),
				ns;
			for( var i = noscripts.length; i; ){
				ns = noscripts[ --i ];
				ns.parentNode && ns.parentNode.removeChild( ns );
			};
		}, 0 );
		
		return {
			parse: function( v ){
				if( Type.isString( v ) === true ){
					if( v === 'true' )      return true;
					if( v === 'false' )     return false;
					if( v === 'null' )      return null;
					if( v === 'undefined' ) return undefined;
					if( v === 'NaN' )       return NaN;
					if( v === '' )          return v;
					var _v = v.split( ' ' ).join( '' ),
						n = _v - 0;
					if( '' + n === _v || '' + n === '0' + _v  ) return n - 0;
				};
				return v;
			},
			extend: function( baseInstance, extend ){
				for( var key in extend ){
					if( Type.isUndefined( baseInstance[ key ] ) === true ){
						baseInstance[ key ] = extend[ key ];
					} else
					if( typeof baseInstance[ key ] === typeof extend[ key ] ){
						baseInstance[ key ] = extend[ key ];
					} else {
						alert( 'extend error' );
					};
				};
				return baseInstance;
			},	
			cleanCommnetNode: function ( _targetElm ){
				search( _targetElm || body );
				
				function search( _elm ){
					if( !_elm ) return;
					if( _elm.nodeType === 8 ){
						_elm.parentNode.removeChild( _elm );
						return;
					};
					if( _elm.nodeType === 1 ){
						var _array = Util.copyArray( _elm.childNodes ); // liveNode > array
						while( _array.length > 0 ) search( _array.shift() );
					};
				};
			},
			cleanElement: function( _targetElm ){
				var _array, _elm, j, _css;
				for( var i = CLEAN_TARGET_ELEMENT.length; i; ){
					_array = Util.copyArray( _targetElm.getElementsByTagName( CLEAN_TARGET_ELEMENT[ --i ] ) );
					for( j = _array.length; j; ){
						_elm = _nodes[ --j ];
						_elm.parentNode && _elm.parentNode.removeChild( _elm );
					};
				};
				Util.cleanCommnetNode( _targetElm );
				
				if( UA.isIE === false ) return;
				_nodes = _targetElm.all || _targetElm.getElementsByTagName( '*' );
				for( i = _nodes.length; i; ){
					_elm = _nodes[ --i ];
					_css = Util.getInlineStyle( _elm );
					_css.filter   = null;
					_css.behavior = null;
					_elm.style.cssText = Util.toCssText( _css );
				};
			},
			getInlineStyle: function( _elm ){
				return cssToObject( _elm.style.cssText );
			},
			getComputedStyle: function( elm ){
				/* if( window.getComputedStyle ){
			        Util.getComputedStyle = function( elm ){
			        	return cssToObject( window.getComputedStyle( elm, '' ) );
			        };
			  	} else */
			    if( document.defaultView && document.defaultView.getComputedStyle ){
			        Util.getComputedStyle = function( elm ){
			        	var obj = document.defaultView.getComputedStyle( elm, '' );
			        	return cssToObject( obj.cssText || obj );
			        };
			    } else {
			        Util.getComputedStyle = function( elm ){
			        	return cssToObject( elm.currentStyle );
			        };
			    };
			    return Util.getComputedStyle( elm );
			},
			camelize: function( cssProp ){
			    var parts = cssProp.split( ' ' ).join( '' ).split( '-' ),
			    	l     = parts.length;
			    if( l === 1 ) return parts[ 0 ];
			
			    var camelized = cssProp.charAt(0) === '-'
			      ? parts[ 0 ].charAt( 0 ).toUpperCase() + parts[ 0 ].substring( 1 )
			      : parts[ 0 ];
			
			    for( var i = 1; i < l; ++i ){
			    	camelized += parts[ i ].charAt( 0 ).toUpperCase() + parts[ i ].substring( 1 );
			    };
			    return camelized;
			},
			uncamelize: function( str ){
				return str.split( ' ' ).join( '' ).replace( /[A-Z]/g, '-$&' ).toLowerCase();
			},
			toCssText: function( _css ){
				var ret = [], v;
				for( var p in _css ){
					v = _css[ p ];
					v && ret.push( [ p, v ].join( ':' ) );
				};
				return ret.join( ';' );
			},
			pxTo: function( _px, _unit ){
				return _px / ( UNIT_RATIO[ _unit ] || 1 );
			},
			toPx: function( x, _unit ){
				return x * ( UNIT_RATIO[ _unit ] || 1 );
			},
			absoluteFontSizeToPx: function( fontsize ){
				return FONT_SIZE_RATIO[ fontsize ] || 0;
			},
			getElementSize: function( _elm ){
				if( Type.isHTMLElement( _elm ) === false ){
					return {
						width:	0,
						height:	0
					};
				};
				var	parentElm   = _elm.parentNode,
					prevElm     = _elm.previousSibling,
					nextElm     = _elm.nextSibling,
					displayNone = _elm.style.display === 'none';
				if( displayNone === true ) _elm.style.display = 'block';
				
				body.appendChild( ELM_SIZE_GETTER );
				ELM_SIZE_GETTER.appendChild( _elm );
				
				var ret = {
					width:		_elm.offsetWidth,
					height:		_elm.offsetHeight
				};
				if( displayNone === true ) _elm.style.display = 'none';
				if( nextElm ){
					parentElm.insertBefore( _elm, nextElm );
				} else		
				if( prevElm && prevElm.nextSibling ){
					parentElm.insertBefore( _elm, prevElm.nextSibling );
				} else {
					parentElm && parentElm.appendChild( _elm );
				};
				body.removeChild( ELM_SIZE_GETTER );
				
				return ret;
			},
			getImageSize: function( img ){
				var	parentElm   = img.parentNode,
					prevElm     = img.previousSibling,
					nextElm     = img.nextSibling,
					displayNone = img.style.display === 'none';
				if( displayNone === true ) img.style.display = '';
				
				body.appendChild( ELM_SIZE_GETTER );
				ELM_SIZE_GETTER.appendChild( img );
				
				var size = getActualDimension( img );
				
				if( displayNone === true ) img.style.display = 'none';
				if( nextElm ){
					parentElm.insertBefore( img, nextElm );
				} else		
				if( prevElm && prevElm.nextSibling ){
					parentElm.insertBefore( img, prevElm.nextSibling );
				} else {
					parentElm && parentElm.appendChild( img );
				}
				body.removeChild( ELM_SIZE_GETTER );
				
			/* LICENSE: MIT
			 * AUTHOR: uupaa.js@gmail.com
			 */
				function getActualDimension(image) {
					var run, mem, w, h, key = "actual";
				
				// for Firefox, Safari, Google Chrome
					if( "naturalWidth" in image ){
						return {
							width:	image.naturalWidth,
							height:	image.naturalHeight
						};
					};
				
					if( "src" in image ){ // HTMLImageElement
						if (image[key] && image[key].src === image.src) {
							return image[key];
						};
						if( doc.uniqueID ){ // for IE
							run = image.runtimeStyle;
							mem = { w: run.width, h: run.height }; // keep runtimeStyle
							run.width  = "auto"; // override
							run.height = "auto";
							w = image.width;
							h = image.height;
							run.width  = mem.w; // restore
							run.height = mem.h;
						} else { // for Opera and Other
							mem = { w: image.width, h: image.height }; // keep current style
							image.removeAttribute("width");
							image.removeAttribute("height");
							w = image.width;
							h = image.height;
							image.width  = mem.w; // restore
							image.height = mem.h;
						};
						return image[key] = { width: w, height: h, src: image.src }; // bond
					};
					// HTMLCanvasElement
					return { width: image.width, height: image.height };
				};
				
				return size;
			},
			getAbsolutePath: function( path ){
				var elm = document.createElement('div');
				body.appendChild( elm );
				elm.innerHTML = '<a href=\"' + path + '\" />';
				body.removeChild( elm );
				return elm.firstChild.href;
			},
			getAbsolutePosition: function( _elm ){
				// Find the destination's position
				// need html, body { fontfamily: "MS PGothic", "MS Gothic" } for ie6.
				var	destx    = _elm.offsetLeft,
					desty    = _elm.offsetTop,
					thisNode = _elm,
					body     = document.body;
				while( thisNode.offsetParent && thisNode !== body ){
					thisNode = thisNode.offsetParent;
					destx += thisNode.offsetLeft;
					desty += thisNode.offsetTop;
				};
				return {
					x:	destx,
					y:	desty
				};
			},
			pullHtmlAsTemplete: function( html ){
				var elm = document.createElement( 'div' );
				body.appendChild( elm );
				elm.innerHTML = html;
				body.removeChild( elm );
				return elm.firstChild;
			},
			getElementsByClassName: function( _elm, _className, opt_tagName ){
				var _all       = !opt_tagName || opt_tagName === '*',
					_livenodes = _all === true ? ( _elm.all || _elm.getElementsByTagName( '*' )) : _elm.getElementsByTagName( opt_tagName ),
					sp         = ' ',
					_cnames    = _className.replace( /\s+/g, sp ).split( sp ),
					_cname,
					_nodes     = Util.copyArray( _livenodes );
				
				for( var i = _cnames.length; i; ){
					_cname = _cnames[ --i ];
					for( var j = 0; j < _nodes.length; ){
						_node = _nodes[ j ];
						if( _node.nodeType !== 1 || Util.hasClassName( _node, _cname ) === false ){
							_nodes.splice( j, 1 );
						} else {
							++j;
						};
					};
				};
				return _nodes;
			},
			getChildIndex: function( _parent, _child ){
				var _children = _parent.getElementsByTagName( _child.tagName );
				for( var i = _children.length; i; ){
					if( _children[ --i ] === _child ) return i;
				};
				return -1;
			},
			hasClassName: function( _elm, _className ){
				return Util.getIndex( ( _elm.className || '' ).split( ' ' ), _className ) !== -1;
			},
			addClass: function( _elm, _className ){
				if( Util.hasClassName( _elm, _className ) === false ){
					var sp = ' ';
					var _array = ( _elm.className || '' ).replace( /\s+/g, sp ).split( sp );
					_array.push( _className );
					_elm.className = _array.join( sp );
				};
			},
			removeClass: function( _elm, _className ){
				var sp     = ' ',
					_array = _elm.className.replace( /\s+/g, sp ).split( sp ),
					i      = Util.getIndex( _array, _className );
				if( i !== -1 ){
					_array.splice( i, 1 );
					_elm.className = _array.join( sp );
				};
			},
			toggleClass: function( _elm, _className ){
				Util.hasClassName( _elm, _className ) === false ? Util.addClass( _elm, _className ) : Util.removeClass( _elm, _className );
			},
			removeAllChildren: function ( _elm){
				while( _elm.firstChild){
					remove( _elm.firstChild);
				}
				function remove( _node){
					while( _node.firstChild){
						remove( _node.firstChild);
					}
					_node.parentNode && _node.parentNode.removeChild( _node);
				}
			},
			getIndex: function( _array, _element ){
				if( Array.prototype.indexof ){
					Util.getIndex = function( _array, _element){
						return _array.indexof( _element);
					};
				} else {
					Util.getIndex = function( _array, _element){
						for( var i = _array.length; i; ){
							if( _array[ --i ] === _element ) return i;
						};
						return -1;
					};
				};
				return Util.getIndex( _array, _element );
			},
			copyArray: function( _array ){
				var l = _array.length,
					ret = new Array( l );
				for( var i=0; i<l; ++i ){
					ret[ i ] = _array[ i ];
				}
				return ret;
			},
			/*
			 * 
			 */
			createGlobalUniqueName: function(){
				var randomKey = null;
				while( true ){
					randomKey = '_uniqueName'+(''+Math.random()).replace(/\./,'');
					if( typeof window[randomKey] === 'undefined'){
						break;
					}
				}
				return randomKey;
			},
			createIframe: function( id, callback){
				var ua = UA;
	            var el = document.createElement( ua.isIE ? '<iframe name="' + id + '" frameborder="0" scrolling="no">' : 'iframe');
	
	            if( ua.isIE){
					el.onreadystatechange = detect;
	            } else {
	            	// iron(chrome) の場合、append の前に onload を指定しないと onload が呼ばれない
	            	el.onload = onLoad;
	            	//setTimeout( asynkCallback, 0 );
				}
	
				document.body.appendChild( el);
	            el.id = el.name = id;
	            el.setAttribute( 'name', id);
	            el.style.cssText = 'width:1px;height:1px;visibility:hidden;position:absolute;top:1px;left:1px;';
				// http://d.hatena.ne.jp/onozaty/20070830/p1
				// [JavaScript]IE6ではJavaScriptで動的に作成したiframeに対してsubmitできない(IE7は未確認) ->解決
				el.contentWindow.name = id;					
	            
	            window[id] = el;
	
	            function detect(){
	                if ( this.readyState === "complete" ){
	                    this.onreadystatechange = new Function();
	                    this.onreadystatechange = null;
	                    setTimeout( asynkCallback, 0 );
	                };
	            };
				function onLoad(){
					el.onload = null;
					setTimeout( asynkCallback, 0 );
				};
				function asynkCallback(){
					callback( el );
				};
			}
		};
	})();



/* ----------------------------------------------------
 * gadgetOS
 * 
 */	

( function( window, document, undefined ){
	
	var doc            = window.document;
	var body           = doc.getElementsByTagName( 'body' )[ 0 ]; //( doc.compatMode || '' ) !== 'CSS1Compat' ? doc.body : doc.documentElement;// 
	
	var SERVICE_LIST   = [];
	var SUPER_USER_KEY = { getUID: function(){ return 0; }};
	var API_USER_LIST  = [ SUPER_USER_KEY ];
	var numApiUser     = 1;
	
	function isApiUser( _user ){
		if( _user === SUPER_USER_KEY ) return true;
		if( File.isDriver( _user ) === true ) return true;
		if( Application.isApplicationInstance( _user ) === true ) return true;
		return false;
	}
	
	var Const = {
		FILE: {
			TYPE: {
				UNKNOWN:	0,
				FOLDER:		1,
				IMAGE:		2,
				TEXT:		3,
				HTML:		4,
				CSV:		5,
				JSON:		6,
				XML:		7
			},
			STATE: {
				UNKNOWN:	0,
				OK:			1,
				LOADING:	2,
				ERROR:		3,
				BROKEN:		4
			},
			UPDATE_POLICY: {
				_____:		parseInt( '00000', 2 ),
				____C:		parseInt( '00001', 2 ), // hasCreateMenu
				___W_:		parseInt( '00010', 2 ), // isWritable
				___WC:		parseInt( '00011', 2 ), // isWritable
				__R__:		parseInt( '00100', 2 ), // isRenamable
				__R_C:		parseInt( '00101', 2 ), // hasCreateMenu
				__RW_:		parseInt( '00110', 2 ), // isWritable
				__RWC:		parseInt( '00111', 2 ), // isWritable
				_S___:		parseInt( '01000', 2 ), // childrenIsSortable
				_S__C:		parseInt( '01001', 2 ),
				_S_W_:		parseInt( '01010', 2 ),
				_S_WC:		parseInt( '01011', 2 ),
				_SR__:		parseInt( '01100', 2 ),
				_SR_C:		parseInt( '01101', 2 ),
				_SRW_:		parseInt( '01110', 2 ),
				_SRWC:		parseInt( '01111', 2 ),
				D____:		parseInt( '10000', 2 ),
				D___C:		parseInt( '10001', 2 ), // hasCreateMenu
				D__W_:		parseInt( '10010', 2 ), // isWritable
				D__WC:		parseInt( '10011', 2 ), // isWritable
				D_R__:		parseInt( '10100', 2 ), // isRenamable
				D_R_C:		parseInt( '10101', 2 ), // hasCreateMenu
				D_RW_:		parseInt( '10110', 2 ), // isWritable
				D_RWC:		parseInt( '10111', 2 ), // isWritable
				DS___:		parseInt( '11000', 2 ), // childrenIsSortable
				DS__C:		parseInt( '11001', 2 ),
				DS_W_:		parseInt( '11010', 2 ),
				DS_WC:		parseInt( '11011', 2 ),
				DSR__:		parseInt( '11100', 2 ),
				DSR_C:		parseInt( '11101', 2 ),
				DSRW_:		parseInt( '11110', 2 ),
				DSRWC:		parseInt( '11111', 2 ),
				CREATE:		1,
				WRAITE:		2,
				RENAME:		4,
				SORT:		8,
				DELETE:		16
			},
			EVENT: {
				UPDATE_ATTRIVUTE:	'onFileUpdate',
				GET_SEQENTIAL_FILES:'gotSeqentilFiles'
			},
			DATA_PROPERTY_RESERVED: [
				'children', 'driver', 'state', 'type'
			]			
		},
		TREE: {
			EVENT: {
				UPDATE:				'onTreeUpdate'
			}
		},
		KEY: {
			EVENT: {
				KEY_DOWN:			'keydown',
				KEY_UP:				'keyup',
				KEY_CHANGE:			'keychange',
				CURSOL:				'cursol'
			}
		},
		APP: {
			TYPE: {
				GENERAL: 0,
				OVERLAY: 1,
				PAGE:    2
			}
		}
	};

var EX = ( function(){
	var F = new Function();
	
	function clone( src ) {
		var ret;
		if( Type.isArray(src) === true ){
			ret = [];
		} else
		if( Type.isObject(src) === true ){
			ret = {};
		} else
		if( Type.isNumber(src) === true || Type.isString(src) === true || Type.isBoolean( src ) === true ){
			return src;
		} else {
			return null;
		}
		for( var key in src ){
			ret[ key ] = clone( src[ key ]);
		}
		return ret;
	};
	
	return {
		extend: function( base, extend ){
			F.prototype = base;
			var ret = new F;
			for( var p in extend ){
				ret[ p ] = extend[ p ];
			};
			return ret;
		},
		clone: function( obj ){
			return clone( obj );
		},
		kill: function(){
			var self = this, v;
			for( var p in s ){
				if( self.hasOwnProperty && !self.hasOwnProperty( p ) ) continue;
				v = self[ p ];
				v && v instanceof TicketBase && self.kill();
				delete self[ p ];
			}
		}
	}
})();

var TicketBase = function(){
	this.kill = function(){
		var t = this, v;
		for( var p in t ){
			if( t.hasOwnProperty && !t.hasOwnProperty( p ) ) continue;
			v = t[ p ];
			v && v instanceof TicketBase && v.kill();
			delete t[ p ];
		}
	};
};



/* --------------------------------------------------------------
 * System Timer
 * 
 */

var SystemTimer = ( function(){
	var setTimeout    = window.setTimeout;
	var clearTimeout  = window.clearTimeout;
	var INTERVAL_TIME = 16;
	var TICKET_LIST   = [];
	var timerId       = undefined;
	var next          = 0;
	
	function loop(){
	    for( var i = 0; i < TICKET_LIST.length; ) {
	        if( TICKET_LIST[ i ].call( next ) !== false ) ++i;
	    };
	    timerId = undefined;
	    update();
	};
	function update(){
		var l = TICKET_LIST.length,
			n = 99999999,
			c;
		if( l === 0 ){
			timerId !== undefined && clearTimeout( timerId );
			timerId = undefined;
			return;
		};
	    for( var i = 0; i<l; i++ ){
	    	c = TICKET_LIST[ i ].count;
	        if( n > c ) n = c;
	    };
	    if( next > n || timerId === undefined ){
	    	timerId !== undefined && clearTimeout( timerId );
	    	timerId = setTimeout( loop, INTERVAL_TIME * n );
	    	next = n;
	    };
	};
	
	var TimerTicketClass = function( _apiuser, _callback, _time, _once ){
		this.apiuser  = _apiuser;
		this.callback = _callback;
		this.time     = _time;
		this.count    = _time;
		this.once     = _once;
		_apiuser = _callback = null;
	};
	TimerTicketClass.prototype = new TicketBase();
	TimerTicketClass.prototype.call = function( c ){
		this.count -= c;
		if( this.count <= 0 ){
			this.callback();
			if( this.once === true ){
				this.destroy();
				TICKET_LIST.splice( Util.getIndex( TICKET_LIST, this ), 1 );
				return false;
			} else {
				this.count = this.time;
			};
		};
	};
	TimerTicketClass.prototype.destroy = function( _apiuser, _callback ){
		if( _apiuser  && _apiuser  !== this.apiuser )  return false;
		if( _callback && _callback !== this.callback ) return false;
		
		this.kill();
		return true;
	};
	
	return {
		add: function( _apiuser, _handler, _time, _once ){
			if( Type.isNumber( _time ) === false || _time < INTERVAL_TIME ) _time = INTERVAL_TIME;
			
		    var _ticket = new TimerTicketClass( _apiuser, _handler, Math.ceil( _time / INTERVAL_TIME ), _once );
		    TICKET_LIST.push( _ticket );
		    
		    update();
		},
		remove: function( _apiuser, _handler ) {
			var _ticket,
				i = 0;
			while( _ticket = TICKET_LIST[ i ] ){
				if( _ticket.destroy( _apiuser, _handler ) === true ){
					TICKET_LIST.splice( i, 1 );
				} else {
					++i;
				};
			};
		    update();
		}
	}
})();

/* --------------------------------------------------------------
 * Async Callback
 * 
 */
var AsyncCall = ( function(){
	var CALLBACK_LIST = [];

	var CallbackTicketClass = function( _apiuser, _callback, _argments ){
		this.apiuser  = _apiuser;
		this.callback = _callback;
		this.argments = _argments;
		_apiuser = _callback = _argments = null;
	}
	CallbackTicketClass.prototype = new TicketBase();
	CallbackTicketClass.prototype.call = function(){
		var f = this.callback,
			a = this.argments;
		if( Type.isArray( a ) === true ){
			f.apply( this.apiuser, a );
		} else {
			f( a );
		}
	};
	CallbackTicketClass.prototype.destroy = function( _apiuser, _callback ){
		if( _apiuser  && _apiuser  !== this.apiuser ) return false;
		if( _callback && _callback !== this.callback ) return false;
		
		this.kill();
		return true;
	};

	function dispatch(){
		var _ticket = CALLBACK_LIST.shift();
		if( _ticket ){
			_ticket.call();
			_ticket.destroy();
			CALLBACK_LIST.length !== 0 && SystemTimer.add( SUPER_USER_KEY, dispatch, 1, true );
		}
	}

	return {
		add: function( _apiuser, _callback, _argments ){
			CALLBACK_LIST.length === 0 && SystemTimer.add( SUPER_USER_KEY, dispatch, 1, true );
			CALLBACK_LIST.push( new CallbackTicketClass( _apiuser, _callback, _argments ) );
		},
		remove: function( _apiuser, _callback ){
			var _ticket,
				i = 0;
			while( _ticket = CALLBACK_LIST[ i ] ){
				if( _ticket.destroy( _apiuser, _callback ) === true ){
					CALLBACK_LIST.splice( i, 1 );
				} else {
					++i;
				}
			}
		}
	}
})();

/* -----------------------------------------------------------
 * 画像一覧は
 * 	お気に入り画像一覧 > tag:ペン次郎 > ペン次郎：笑う
 *  最近アップロードされた画像 > images
 *  最近使われた画像 > images
 *  キャラクター画像庫 > アニマル系 > tag:ペン次郎 > ペン次郎：笑う
 *  風景画像庫 >
 *  効果画像庫 >
 *  アイテム画像庫 >
 *  
 * 画像一覧を読み込むタイミング
 */
var File = ( function(){
	var DRIVER_LIST             = [];
	
	var FILE_TYPE_IS_FOLDER     = Const.FILE.TYPE.FOLDER,
		numFileType             = Const.FILE.TYPE.XML,
		FILEDATA_RESITER        = [],			// store all of fileData( json object )
		FILEDATA_ACCESS         = [],			// file operations for Kernel only ! hide from Out of File
		FILE_OBJECT_POOL        = [],
		EVENT_LISTENER_REGISTER = [],
		TREE_ARRAY              = [],
		TREE_ACCESS_ARRAY       = [];
	
	var REQUEST_CONTROLER = ( function(){
		var REQUEST_TICKET_RESISTER = [],
			currentTicket           = null,
			currentData             = null,
			DATA_TYPE_ARRAY         = 'json,xml,html,text'.split( ','),
			DATA_IS_JSON            = 0,
			DATA_IS_XML             = 1,
			DATA_IS_HTML            = 2,
			DATA_IS_TEXT            = 3,
			numError                = 0;
		
		var RequestTicketClass = function( _apiuser, _type, _data, _url, _onLoad, _onError ){
			this.apiuser = _apiuser;
			this.type    = _type;
			this.data    = _data;
			this.url     = _url;
			this.onLoad  = _onLoad;
			this.onError = _onError;
			this.state   = 0;
			_apiuser = _type = _data = _onLoad = _onError = null;
		};
		RequestTicketClass.prototype = new TicketBase();
		RequestTicketClass.prototype.load = function( _data ){
			AsyncCall.add( this.apiuser, this.onLoad, [ this.data, _data ] );
		};
		RequestTicketClass.prototype.error = function(){
			AsyncCall.add( this.apiuser, this.onError, this.data );
		};
		
		function request(){
			if( currentTicket !== null || REQUEST_TICKET_RESISTER.length === 0 ) return;
			currentTicket = REQUEST_TICKET_RESISTER.shift();
			$.ajax({
				url:		currentTicket.url,
				dataType:	DATA_TYPE_ARRAY[ currentTicket.type ],
				success:	onSuccess,
				error: 		onError
			});
		}
		function onSuccess( _data ){
			currentTicket.load( _data );
			currentTicket.kill();
			currentTicket = null;
			request();
		}
		function onError(){
			++numError;
			currentTicket.error();
			currentTicket.kill(); // retry
			currentTicket = null;
			request();
		}

		return {
			getNumTask: function(){
				return REQUEST_TICKET_RESISTER.length;
			},
			getNumError: function(){
				return numError;
			},
			getJson: function( _apiuser, _data, _url, _onLoad, _onError ){
				REQUEST_TICKET_RESISTER.push( new RequestTicketClass( _apiuser, DATA_IS_JSON, _data, _url, _onLoad, _onError ));
				currentTicket === null && request();
			}
		}
	})();

	var FILE_CONTROLER = {
		createTree: function( _apiuser, _rootFileData ){
			var _tree = new TreeClass( _apiuser, _rootFileData );
			TREE_ARRAY.push( _tree );
			return _tree;
		},
		getFileUID: function( FILEDATAorFILE ){
			if( FILEDATAorFILE instanceof FileClass ){
				return FILEDATAorFILE.getUID();
			};
			
			var uid = Util.getIndex( FILEDATA_RESITER, FILEDATAorFILE );
			if( uid === -1 ){
				uid = FILEDATA_RESITER.length;
				FILEDATA_RESITER.push( FILEDATAorFILE );
			};
			return uid;
		},
		getFileDataAccess: function( UIDorFILEorFILEDATA ){
			var _uid, _data = FILE_CONTROLER.getFileData( UIDorFILEorFILEDATA ), _access;

			if( _data === null || typeof _data !== 'object' ) return null;
			for( var i=0, l = FILEDATA_ACCESS.length; i<l; ++i ){
				_access = FILEDATA_ACCESS[ i ];
				if( _access.DATA === _data ) return _access;
			};
			return null;
		},	
		getFileData: function( UIDorFILEorFILEDATA ){
			if( typeof UIDorFILEorFILEDATA === 'number' ){
				return FILEDATA_RESITER[ UIDorFILEorFILEDATA ] || null;
			} else
			if( UIDorFILEorFILEDATA instanceof FileClass ){
				return FILEDATA_RESITER[ UIDorFILEorFILEDATA.getUID() ] || null;
			} else
			if( Util.getIndex( FILEDATA_RESITER, UIDorFILEorFILEDATA ) !== -1 ){
				return UIDorFILEorFILEDATA;
			};
			return null;
		},
		getChildren: function( UIDorFILEorFILEDATA ){
			var _data = FILE_CONTROLER.getFileData( UIDorFILEorFILEDATA );
			return _data !== null ? _data.children || null : null;
		},
		getDriver: function( _file ){
			var _data = FILE_CONTROLER.getFileData( _file );
			return _data !== null && _data.driver ? _data.driver : BASE_DRIVER;
		},
		getUpdateFlag: function( _file, _bit ){
			var _driver = FILE_CONTROLER.getDriver( _file ),
				_policy;
			if( typeof _driver.getUpdatePolicy === 'function' ){
				_policy = _driver.getUpdatePolicy( _file );
				
			}
			if( typeof _policy !== 'number' ) {
				_policy = BASE_DRIVER.getUpdatePolicy( _file )
			}
			return _policy % ( _bit * 2 ) >= _bit;
		},
		move: function( _prentUID, _targetfile, _newFolder, _newIndex, _opt_callback ){
			var _parentData = FILE_CONTROLER.getFileDataAccess( _prentUID ),
				_parentType = _parentData.TYPE,
				_targetData = FILE_CONTROLER.getFileDataAccess( _targetfile ),
				_targetType = _targetData.TYPE;
		},
		replace: function( _uid, _file, _newIndex ){
			
		},
		addEventListener: function( FILEorNULL, _eventType, _callback ){
			var _uid = FILEorNULL instanceof FileClass ? FILEorNULL.getUID() : FILEorNULL;
			EVENT_LISTENER_REGISTER.push( new FileEventTicketClass( _uid, _eventType, _callback ));
		},
		removeEventListener: function( FILEorNULL, _eventType, _callback ){
			var _uid = FILEorNULL instanceof FileClass ? FILEorNULL.getUID() : FILEorNULL,
				_ticket;
			for(var i=0, l = EVENT_LISTENER_REGISTER.length; i<l; ++i ){
				_ticket = EVENT_LISTENER_REGISTER[ i ];
				if( _ticket.fileUID === _uid && _ticket.eventType === _eventType && _ticket.callBack === _callback ){
					EVENT_LISTENER_REGISTER.splice( i, 1 );
					_ticket.kill();
				};
			};
		},
		getTreeAccess: function(){
			
		},
		fileEventRellay: function( _uid, _event ){
			var _fileAccess = FILE_CONTROLER.getFileDataAccess( _uid );
			if( _fileAccess === null ) return;
			var _treeUID    =  _fileAccess.TREE.getUID(),
				_treeAccess = TREE_ACCESS_ARRAY[ _treeUID ],
				_data       = _fileAccess.DATA,
				_tree;
			if( !_treeAccess ) return;
			_treeAccess.dispatchFileEvent( _event );
			for( var i=0, l = TREE_ARRAY.length; i<l; ++i ){
				if( i !== _treeUID ){
					_tree = TREE_ARRAY[ i ];
					if( FILE_CONTROLER.getFileData( _tree.getCurrentFile() ) === _data ){
						_treeAccess = TREE_ACCESS_ARRAY[ _tree.getUID() ];
						_treeAccess && _treeAccess.dispatchFileEvent( _event );
					};
				};
			};
		}
	};
	
	var TreeClass = function( apiuser, rootFileData ){
		var PARENT_FILE_RESITER = [],
			ACCESS = {
				apiuser          : apiuser,
				dispatchFileEvent: dispatchFileEvent
			},
			EVENT_LISTENER_ARRAY = [],
			instance             = this,
			rootFile             = new FileClass( instance, null, rootFileData ),
			currentFile          = rootFile;
		
		currentFile.getSeqentialFiles();
		TREE_ACCESS_ARRAY.push( ACCESS );
		
		function dispatchFileEvent( e ){
			var _eventType  = e.eventType,
				_targetFile = e.targetFile,
				_uid        = _targetFile.getUID(),
				_ticket, _type, _callback;
			for( var i=0, l = EVENT_LISTENER_REGISTER.length; i<l; ++i ){
				_ticket   = EVENT_LISTENER_REGISTER[ i ];
				_type     = _ticket.eventType;
				_callback = _ticket.callBack;
				if( _eventType === _type && _uid === _ticket.fileUID ){
					AsyncCall.add( apiuser, _callback, [ _eventType, _targetFile, e.key, e.value ] );
				} else
				if( _type === Const.TREE.EVENT.UPDATE && _eventType === Const.FILE.EVENT.GET_SEQENTIAL_FILES ){
					//_callback( _eventType, _targetFile );
					AsyncCall.add( apiuser, _callback, [ _eventType, _targetFile ] );
				};
			};
		};
		
		this.getUID = function(){
			return Util.getIndex( TREE_ACCESS_ARRAY, ACCESS );
		};
		this.getRootFile = function(){
			return rootFile;
		};
		this.getCurrentFile = function(){
			return currentFile;
		};
		this.hierarchy = function(){
			return PARENT_FILE_RESITER.length;
		};
		this.getParentFileAt = function( _index ){
			var l = PARENT_FILE_RESITER.length;
			if( typeof _index !== 'number' || _index < 0 || _index >= l ) return null;
			return PARENT_FILE_RESITER[ l -1 -_index ];
		};
		this.down = function( _index ){
			if( typeof _index !== 'number' || _index < 0 || _index >= currentFile.getChildFileLength()) return;
			PARENT_FILE_RESITER.unshift( currentFile );
			currentFile = currentFile.getChildFileByIndex( _index );
			currentFile.getSeqentialFiles();
			return currentFile;
		};
		this.up = function( _index ){
			var l = PARENT_FILE_RESITER.length;
			if( l === 0) return null;
			
			if( currentFile ){
				var _currentFile = currentFile;
				currentFile = null;
				_currentFile.destroy();
			};
			if( typeof _index === 'number'){
				if( _index >= l) return null;
				currentFile = this.getParentFileAt( _index );
				PARENT_FILE_RESITER.splice( 0, l -_index);
			} else {
				currentFile = PARENT_FILE_RESITER.shift();
			};
			currentFile.getSeqentialFiles();
			return currentFile;	
		};
		this.addTreeEventListener = function( _eventType, _callback ){
			FILE_CONTROLER.addEventListener( null, _eventType, _callback );
		};
		this.removeTreeEventListener = function( _eventType, _callback ){
			FILE_CONTROLER.removeEventListener( null, _eventType, _callback );
		};
		this.destroy = function( _apiuser ){
			if( _apiuser && apiuser !== _apiuser ) return false;
			// removeEvent
			var _currentFile = currentFile;
			currentFile = rootFile = rootFileData = null;
			// currentFile, rootFile を null にしないと .File.destroy() ができない.
			_currentFile.destroy();
			while( PARENT_FILE_RESITER.length > 0 ){
				_currentFile = PARENT_FILE_RESITER.shift();
				_currentFile.destroy();
			};
			
			AsyncCall.remove( apiuser );
			instance = apiuser = null;
			return true;
		};
	};

	var FileEventTicketClass = function( _uid, _eventType, _callback ){
		this.fileUID   = _uid;
		this.eventType = _eventType;
		this.callBack  = _callback;
		_uid = _eventType = _callback = undefined;
	};
	FileEventTicketClass.prototype = new TicketBase();
	
	var FileEventClass = function( eventType, file, key, value ){
		this.eventType = eventType;
		this.targetFile = file;
		this.updatedAttribute = key;
		this.updatedValue = value;
	};

/*
 * file の data は object で保持している。
 * File の外からファイルをみるときは、FileClassを通して操作する。
 * fileの変更、それに付随して追加されたイベントは、TreeClassで管理される。
 * treeがdestryされると、fileのイベントリスナーも全て削除される。
 * 他の tree も data の共通する currentFile に対してのみは、file の変更イベントを受け取って流す．
 * 
 */
	
	var FileClass = function( tree, parentData, data ){
		var uid = FILE_CONTROLER.getFileUID( data );
		
		FILEDATA_ACCESS.push( {
			TREE:				tree,
			parentData:			parentData,
			DATA:				data
		} );
		
		tree = parentData = data = null;

		this.getUID = function(){
			return uid;
		}
	};
	
	FileClass.prototype = {
		isChildFile: function( _FILEorFILEDATA ){
			return this.getChildFileIndex( _FILEorFILEDATA) !== -1;
		},
		getSeqentialFiles: function(){
			var _driver = FILE_CONTROLER.getDriver( this );
			if( _driver !== null && typeof _driver.getSeqentialFiles === 'function' ){
				_driver.getSeqentialFiles( this );
			}
		},
		addEventListener: function( _eventType, _callback ){
			FILE_CONTROLER.addEventListener( this, _eventType, _callback );
		},
		removeEventListener: function( _eventType, _callback ){
			FILE_CONTROLER.removeEventListener( this, _eventType, _callback );
		},
		dispatchEvent: function( e ){
			e instanceof FileEventClass && FILE_CONTROLER.fileEventRellay( this.getUID(), e );
		},
		getChildFileLength: function(){
			var children = FILE_CONTROLER.getChildren( this );
			return Type.isArray( children ) === true ? children.length : -1;
		},
		getChildFileIndex: function( _FILEorFILEDATA ){
			var children = FILE_CONTROLER.getChildren( this );
			if( Type.isArray( children ) === false ) return -1;
			var l = children.length,
				_fileData = FILE_CONTROLER.getFileData( _FILEorFILEDATA );
			if( _fileData === null ) return -1;
			for( var i=0; i<l; ++i ){
				if( children[ i ] === _fileData ) return i;
			}
			return -1;
		},
		getChildFileByIndex: function( _index ){
			var _access = FILE_CONTROLER.getFileDataAccess( this ),
				_children = FILE_CONTROLER.getChildren( this );
			if( typeof _index !== 'number' || _index < 0 || Type.isArray( _children ) === false || _index >= _children.length) return null;
			var _file = new FileClass( _access.TREE, _access.DATA, _children[ _index ]);
			// _file.init();
			return _file;
		},
		getName: function(){
			var driver = FILE_CONTROLER.getDriver( this );
			if( typeof driver.getName === 'function'){
				return driver.getName( this );
			}
			return BASE_DRIVER.getName( this);
		},
		getThumbnail: function(){
			var driver = FILE_CONTROLER.getDriver( this );
			if( typeof driver.getThumbnail === 'function'){
				return driver.getThumbnail( this );
			}
			return BASE_DRIVER.getThumbnail( this );
		},
		getType: function(){
			var _data = FILE_CONTROLER.getFileData( this );
			return typeof _data.type === 'number' ? _data.type : Const.FILE.TYPE.UNKNOWN;
		},
		getState: function(){
			var _data = FILE_CONTROLER.getFileData( this );
			return typeof _data.state === 'number' ? _data.state : Const.FILE.STATE.OK;
		},
		getSummary: function(){
			var driver = FILE_CONTROLER.getDriver( this );
			if( typeof driver.getSummary === 'function'){
				return driver.getSummary( this );
			}
			return BASE_DRIVER.getSummary( this );
		},
		isWritable: function(){
			return FILE_CONTROLER.getUpdateFlag( this, Const.FILE.UPDATE_POLICY.WRITE );
		},
		isSortable: function(){
			return FILE_CONTROLER.getUpdateFlag( this, Const.FILE.UPDATE_POLICY.SORT );
		},		
		isCreatable: function(){
			return FILE_CONTROLER.getUpdateFlag( this, Const.FILE.UPDATE_POLICY.CREATE );
		},
		isRenamable: function(){
			return FILE_CONTROLER.getUpdateFlag( this, Const.FILE.UPDATE_POLICY.RENAME );
		},
		isDeletable: function(){
			return FILE_CONTROLER.getUpdateFlag( this, Const.FILE.UPDATE_POLICY.DELETE );
		},
		read: function(){
			// simpleDeepCopy
			var driver = FILE_CONTROLER.getDriver( this ),
				data;
			if( typeof driver.read === 'function'){
				 data = driver.read( this );
			}
			return BASE_DRIVER.read( data || this );
		},
		write: function( _newData, _onUpdateFunction ){
			var driver = FILE_CONTROLER.getDriver( this );
			if( typeof driver.write === 'function'){
				return driver.write( this, _newData, _onUpdateFunction );
			}
			return BASE_DRIVER.write( this, _newData, _onUpdateFunction );
		},
		viewerApplicationList: function(){
			var driver = FILE_CONTROLER.getDriver( this );
			if( typeof driver.viewerApplicationList === 'function'){
				return driver.viewerApplicationList( this );
			}
			return BASE_DRIVER.viewerApplicationList( this );
		},
		editorApplicationList: function(){
			var driver = FILE_CONTROLER.getDriver( this );
			if( typeof driver.editorApplicationList === 'function'){
				return driver.editorApplicationList( this );
			}
			return BASE_DRIVER.viwerApps( this );
		},
		create: function(){
			
		},
		sort: function(){
			
		},
		onCopy: function(){
			
		},
		onDelete: function(){
			
		},
		move: function( _newFolder, _newIndex, opt_callback ){
			var _access = FILE_CONTROLER.getFileDataAccess( this );
			_access.TREE.move( _access.parentData, this.getUID(), _newFolder, _newIndex, opt_callback );
		},
		replace: function( _newIndex, opt_callback ){
			var _access = FILE_CONTROLER.getFileDataAccess( this );
			_access.TREE.replace( _access.parentData, this.getUID(), _newIndex, opt_callback);
		},
		/**
		 * サーチ
		 * 探しているファイルの属性と値を指定．一致する child の index を配列で返す．
		 */
		search: function( obj, rule ){
			var _children = FILE_CONTROLER.getChildren( this ),
				_child,
				ret = [], k, c;
			for( var i=0, l=_children.length; i<l; ++i ){
				_child = _children[ i ];
				c = true;
				for( k in obj ){
					if( obj[ k ] !== _child[ k ] ){
						c = false;
						break;
					}
				}
				c === true && ret.push( i );
			}
			return ret;
		},
		destroy: function(){
			var _access = FILE_CONTROLER.getFileDataAccess( this );
			var _tree = _access.TREE;
			if( _tree.getCurrentFile() === this ) return;
			if( _tree.getRootFile() === this ) return;
			for( var i=0, l = _tree.hierarchy(); i<l; ++i ){
				if( _tree.getParentFileAt( i ) === this ){
					return;
				}
			}
			var _index = Util.getIndex( FILEDATA_ACCESS, _access );
			if( _index === -1 ) return;
			// event の 削除
			FILEDATA_ACCESS.splice( _index, 1 );
			delete _access.DATA;
			delete _access.TREE;
			delete _access.parentData;
		}
	}

	/*
	 * FileDriverBase
	 */
	var FileDriverBase = function( driverClass ){
		this.getUID = function(){
			return Util.getIndex( API_USER_LIST, driverClass );
		};
		this.getSeqentialFiles = function( _file ){
		};
		this.getName = function( _file ){
			var _data = FILE_CONTROLER.getFileData( _file );
			return _data.name || 'No Name';
		};
		this.getThumbnail = function( _file ){
			var _data = FILE_CONTROLER.getFileData( _file ),
				_type = _data.type,
				_className = '';
			if( _type === Const.FILE.TYPE.FOLDER){
				_className = 'folder';
			} else
			if( _type === Const.FILE.TYPE.IMAGE){
				
			} else
			if( _type === Const.FILE.TYPE.TEXT){
				
			} else
			if( _type === Const.FILE.TYPE.HTML){
				
			} else
			if( _type === Const.FILE.TYPE.CSV){
				
			} else
			if( _type === Const.FILE.TYPE.JSON){
				
			} else
			if( _type === Const.FILE.TYPE.XML){
				
			}
			return {
				image:		null,
				className:	' file-type-' + _className
			}
		};
		this.getSummary = function( _file ){
			var _data = FILE_CONTROLER.getFileData( _file ),
				_type = _data.type;
			if( _type === Const.FILE.TYPE.FOLDER ){
				return 'folder';
			} else
			if( _type === Const.FILE.TYPE.IMAGE ){
				return 'image file';
			} else
			if( _type === Const.FILE.TYPE.TEXT ){
				return 'text file';
			} else
			if( _type === Const.FILE.TYPE.HTML ){
				return 'html document file';
			} else
			if( _type === Const.FILE.TYPE.CSV ){
				return 'csv daat file';
			} else
			if( _type === Const.FILE.TYPE.JSON ){
				return 'json data file';
			} else
			if( _type === Const.FILE.TYPE.XML ){
				return 'xml data file';
			}
			return '';
		};
		this.getUpdatePolicy = function( _file ){
			// debug用 全てのメニューを許可
			return Const.FILE.UPDATE_POLICY.DSRWC;
		};
		this.read = function( _FILEorDATA ){
			var data,
				protects = Const.FILE.DATA_PROPERTY_RESERVED;			
			if( _FILEorDATA instanceof FileClass ){
				data = FILE_CONTROLER.getFileData( _FILEorDATA )
			} else {
				data = _FILEorDATA;
			}

			function clone( src ) {
				var ret;
				if( Type.isArray(src) === true ){
					ret = [];
				} else
				if( Type.isObject(src) === true ){
					ret = {};
				} else
				if( Type.isNumber(src) === true || Type.isString(src) === true || Type.isBoolean( src ) === true ){
					return src;
				} else {
					return null;
				}
				for( var key in src ){
					if( Util.getIndex( protects, key ) === -1 ){
						//alert( key )
						ret[ key ] = clone( src[ key ]);
					}
				}
				return ret;
			};
				
			return clone( data );
		};
		this.write = function( _file, _newData, _onUpdateFunction ){
			var _data = FILE_CONTROLER.getFileData( _file ),
				_type = _data.type;
			return false;
		};
		this.viewerApplicationList = function(){
			return [];
		};
		this.editorApplicationList = function(){
			return [];
		};
		this.onCreate = function(){
			
		};
		this.onSort = function(){
			
		};
		this.onCopy = function(){
			
		};
		this.onDelete = function(){
			
		};
	}
	
	var BASE_DRIVER   = new FileDriverBase();
	
	var ROOT_FILEDATA = {
			name: 		'system root',
			type:		FILE_TYPE_IS_FOLDER,
			children:	[]
		},
		SYSTEM_TREE = FILE_CONTROLER.createTree( SUPER_USER_KEY, ROOT_FILEDATA ),
		ROOT_FILE   = SYSTEM_TREE.getRootFile();

	function createFileTypeID(){
		return ++numFileType;
	}
	
	var FileAPIClass = function( driver ){
		var constObject;
		this.createFolderUnderRoot = function( _fileData ){
			if( _fileData !== null && Type.isObject( _fileData ) === true ){
				ROOT_FILEDATA.children.push( _fileData );
				ROOT_FILE.dispatchEvent( new FileEventClass( Const.FILE.EVENT.GET_SEQENTIAL_FILES, ROOT_FILE, 'children', null ));
			}
		};
		this.createFileEvent   = function( _eventType, _file, _key, _value ){
			return new FileEventClass( _eventType, _file, _key, _value );
		};
		this.createFileTypeID  = createFileTypeID;
		this.getFileDataAccess = FILE_CONTROLER.getFileDataAccess;
		this.getFileData       = FILE_CONTROLER.getFileData;
		this.getJson           = function( _data, _url, _onLoad, _onError ){
			REQUEST_CONTROLER.getJson( driver, _data, _url, _onLoad, _onError );
		};
		this.createTree        = function( _rootFile ){
			return FILE_CONTROLER.createTree( driver, _rootFile );
		};
		this.isTreeInstance    = function( _tree ){
			return _tree instanceof TreeClass;
		};
		this.isFileInstance    = function( _file ){
			return _file instanceof FileClass;
		};
		this.isFileEvent       = function( _event ){
			return _event instanceof FileEventClass;
		};
		this.getConst          = function(){
			return Const; // constObject = constObject || clone( Const )
		};
	}
	
	return {
		registerDriver: function( _class ){
			_class.prototype = new FileDriverBase( _class );
			var _driver = new _class();
			
			DRIVER_LIST.push( _driver );
			API_USER_LIST.push( _class );

			return new FileAPIClass( _driver );
		},
		isDriver: function( _driver ){
			return _driver instanceof FileDriverBase;
		},
		isTreeInstance: function( _tree ){
			return _tree instanceof TreeClass;
		},
		isFileInstance: function( _file ){
			return _file instanceof FileClass;
		}
	}
})();


/* ----------------------------------------------------
 * ApplicationManager
 * window resize event, overlayApplication currentAplication に流す
 */	

var APPLICATION_LIST         = [];

var AbstractBasicPane = function(){
	var instance    = null;
	this.MIN_WIDTH  = 240;
	this.MIN_HEIGHT = 240;
	this.init = function(){
		instance = this;
		instance.onInit();
	};
	this.onInit = function(){};
	this.resize = function( _w, _h ){
		if( instance.MIN_WIDTH > _w || instance.MIN_HEIGHT > _h ){
			if( Type.isHTMLElement( instance.rootElement ) === true ){
				// 小さすぎる！、と表示
			};
			return;
		};
		instance.onPaneResize( _w, _h );
	};
	this.onPaneResize = function( _w, _h ){};
	this.close = function(){
		instance.onClose();
		instance = this;
	};
}

var AbstractApplication = function( displayName, appClass, isOverlay ){
	var self          = null, // init で設定
		uiList        = [],
		finderList    = [],
		fetchResource = 0,
		bootParams    = null,
		phase         = 0;
	this.rootElement = document.createElement( 'div' );
	this.bgColor     = '#C1CACF';
	this.getUID = function(){
		return Util.getIndex( API_USER_LIST, appClass );
	}
	this.init = function(){
		phase = 1;
		self  = this;
		self.onInit();
		phase = 2;
	};
	this.open = function( _w, _h /*, _option */ ){
		phase = 3;
		bootParams = Util.copyArray( arguments );
		
		if( this.rootElement.innerHTML && this.rootElement.innerHTML.length > 0 ){
			SystemTimer.add( self, detect, 16 );
		} else {
			onOpen();
		}
		
		function detect(){
			if( self.rootElement.firstChild && fetchResource === 0 ){
				SystemTimer.remove( self, detect );
				onOpen();
			};
		};
		
		function onOpen(){
			self.rootElement.style.display = '';
			
			if( self.MIN_WIDTH > _w || self.MIN_HEIGHT > _h ){
				if( Type.isHTMLElement( self.rootElement ) === true ){
					// 小さすぎる！、と表示
				}
			}
			if( bootParams.length > 2 ){
				self.onOpen.apply( self, bootParams );
			} else {
				self.onOpen( _w, _h );
			}
			phase = 4;			
		};
	};
	this.resize = function( _w, _h ){
		if( phase !== 4 ) return;
		if( self.MIN_WIDTH > _w || self.MIN_HEIGHT > _h ){
			if( Type.isHTMLElement( self.rootElement ) === true ){
				// 小さすぎる！、と表示
			};
			return;
		};
		self.onPaneResize( _w, _h );
	};
	this.close = function(){
		phase = 5;
		if( self.onClose() === false ){
			return false;
		};
		MouseEvent.remove( self );
		KeyEvent.remove( self );
		SystemTimer.remove( self );
		AsyncCall.remove( self );
		Css.unload( self );
		
		while( uiList.length > 0 ){
			uiList.shift().destroy();
		};
		while( finderList.length > 0 ){
			finderList.shift().destroy();
		};
		var elm = self.rootElement;
		Util.removeAllChildren( elm );
		elm.parentNode.removeChild( elm );
		self.rootElement = null;
		
		Application.shutdown( self, isOverlay );
		self = appClass = uiList = null;
		
		phase = 6;
	};
	this.createUIGroup = function(){
		var _ui = UI.createUIGroup( self );
		uiList.push( _ui );
		return _ui;
	};
	this.createFinder = function( _elmTarget, _tree, _header, _footer, _onSelect, _viewerOption, _editorOption ){
		var _finder = Finder.create( self, _elmTarget, _tree, _header, _footer, _onSelect, _viewerOption, _editorOption );
		finderList.push( _finder );
		return _finder;
	};
	this.createBasicPane = function( _class, _options ){
		if( Type.isFunction( _class ) === false ) return null;
		_class.prototype = new AbstractBasicPane();
		return new _class( _options );
	};
	this.fetchCSS = function( _url, opt_onload, opt_onerror ){
		if( phase === 1 ){
			++fetchResource;
			Css.load( self, _url, fetchResourceComplete, fetchResourceComplete );
		};
	};
	
	function fetchResourceComplete(){
		--fetchResource;
	};
};

AbstractApplication.prototype = new AbstractBasicPane();
AbstractApplication.prototype.onInit = function(){
	// overrride
};	
AbstractApplication.prototype.onOpen = function( _w, _h /*, _option */ ){
	// overrride
};
AbstractApplication.prototype.onClose = function(){
	// overrride
	return true;
}; // false の場合、close の拒否 
AbstractApplication.prototype.addMouseEventListener = function( _element, _eventType, _handler ){
	MouseEvent.add( this, _element, _eventType, _handler );
};
AbstractApplication.prototype.removeMouseEventListener = function( _element, _eventType, _handler ){
	MouseEvent.remove( this, _element, _eventType, _handler );
};
AbstractApplication.prototype.addKeyEventListener = function( _eventType, _handler, _keyCode, _shift, _ctrl ){
	KeyEvent.add( this, _eventType, _handler, _keyCode, _shift, _ctrl );
};
AbstractApplication.prototype.removeKeyEventListener = function( _eventType, _handler, _keyCode, _shift, _ctrl ){
	KeyEvent.remove( this, _eventType, _handler, _keyCode, _shift, _ctrl );
};
AbstractApplication.prototype.shiftEnabled = function(){
	return KeyEvent.shiftEnabled;
};
AbstractApplication.prototype.ctrlEnabled = function(){
	return KeyEvent.ctrlEnabled;
};
AbstractApplication.prototype.addTimer = function( handler, time, once ){
	SystemTimer.add( this, handler, time, !!once );
};
AbstractApplication.prototype.removeTimer = function( handler ){
	SystemTimer.remove( this, handler );
};
AbstractApplication.prototype.addAsyncCall = function( _callback, _argments ){
	AsyncCall.add( this, _callback, _argments );
};
AbstractApplication.prototype.removeAsyncCall = function( _callback ){
	AsyncCall.remove( this, _callback );
};
AbstractApplication.prototype.fetchHTMLElement = function( id ){
	var elm = doc.getElementById( id );
	if( elm ){
		elm.removeAttribute( 'id' );
		elm.parentNode.removeChild( elm );
		return elm;
	};
};

var Application = ( function(){
	
	var LIVE_APPLICATION_LIST = [];
	
	var currentApplication    = null,
		coveredApplication    = null,
		winW                  = 0,
		winH                  = 0;
	
	var ApplicationReference = function( appClass, isOverlay, displayName, id, thumbnailUrl, tailColor ){
		var self          = this;
		var application   = null;
		this.id           = id;
		this.displayName  = displayName;
		this.thumbnailUrl = thumbnailUrl;
		this.tailColor    = tailColor;
		this.getUID = function(){
			return Util.getIndex( API_USER_LIST, appClass );
		};
		this.boot = function( /* _option */ ){
			application = Application.boot( displayName, self.getUID(), appClass, isOverlay, Util.copyArray( arguments ) );
		};
		this.shutdown = function(){
			if( !application ) return false;
			
			if( ( isOverlay === true ? Overlay.hide() : application.close() ) === false ) return false;
			application = null;
		};
	};
	
	function asyncBootHome(){
		currentApplication === null && Home.boot();
	};
	function asyncOpen( /* arguments */ ){
		var _arg = Util.copyArray( arguments );
		_arg.unshift( winW, winH );
		currentApplication.open.apply( currentApplication, _arg );
	};
	return {
		register: function( _class, _overlay, _tail, _displayName, _id, _thumbnailUrl, _tailColor ){
			APPLICATION_LIST.push( _class );
			API_USER_LIST.push( _class );
			var _ref = new ApplicationReference( _class, _overlay, _displayName, _id, _thumbnailUrl, _tailColor );
			_tail === true && Home.add( _ref );
			return _ref;
		},
		isBasicPaneInstance: function( _basicPane ){
			return  _basicPane instanceof AbstractBasicPane;
		},
		isApplicationInstance: function( _application ){
			return _application instanceof AbstractApplication;
		},
		isApplicationReference: function( _reference ){
			return _reference instanceof ApplicationReference;
		},
		isCurrentAppplication: function( _application ){
			return true
		},
		boot: function( displayName, uid, appClass, isOverlay, arg ){
			if( currentApplication ){
				if( currentApplication.getUID() === uid ) return null;
				if( isOverlay === false && currentApplication.close() === false ) return null;
			};

			appClass.prototype = new AbstractApplication( displayName, appClass, isOverlay );
			var application = new appClass(); // new は boot で
			
			coveredApplication = isOverlay === true ? currentApplication : null;
			
			Application.onCurrentApplicationChange( application );
			
			if( isOverlay === false ){
				body.style.backgroundColor = application.bgColor;
				
				body.appendChild( application.rootElement );
				application.rootElement.style.display = 'none';
				application.init();

				application.addAsyncCall( asyncOpen, arg );
			} else {
				Overlay.show( application, arg );
			}
			
			return application;
		},
		shutdown: function( _application, isOverlay ){
			if( isOverlay === false ){
				currentApplication = null;
				AsyncCall.add( SUPER_USER_KEY, asyncBootHome );
			} else {
				Application.onCurrentApplicationChange( coveredApplication );
				coveredApplication = null;
			}
		},
		onCurrentApplicationChange: function( _application ){
			if( Application.isApplicationInstance( _application ) === false ) return;
			if( currentApplication === _application ) return;
			currentApplication = _application;
			MouseEvent.onCurrentApplicationChange( _application );
			KeyEvent.updateCurrentListener( _application );
		},
		onApplicationShutdown: function( _application ){
			LIVE_APPLICATION_LIST.splice( Util.getIndex(  LIVE_APPLICATION_LIST, _application ) );
		},
		onWindowResize: function( w, h ){
			winW = w;
			winH = h;
			currentApplication && currentApplication.resize( w, h );
			Overlay.onWindowResize( w, h );
			UI.onWindowResize( w, h );
		},
		onSystemShutdown: function(){
			
		}
	}
})();

/* --------------------------------------------------------------
 * Home
 * 
 */
	var Home = ( function(){
		var APP_REF_LIST    = [];
		var ELM_TAIL_ORIGIN = ( function(){
			var ret = document.createElement( 'div' ),
				h2  = document.createElement( 'h2' );
			ret.className = 'tail-wrapper';
			ret.appendChild( h2 );
			h2.appendChild( document.createTextNode( 'appName' ) );
			return ret;
		})();
		
		var TailClass = function( appRef ){
			this.elm = ELM_TAIL_ORIGIN.cloneNode( true );
			this.destroy = function(){
				appRef = self = elmName = null;
			};			
			
			var self    = this,
				elmName = this.elm.getElementsByTagName( 'h2' )[ 0 ].firstChild;
			
			this.elm.style.backgroundColor = appRef.tailColor;
			elmName.data = appRef.displayName;
		};
		
		var ref = Application.register( function(){
			var self     = this,
				winW     = 0,
				winH     = 0,
				tailList = [],
				elmContainer, elmHeader;
			
			function draw(){
				var tail, elm;
				for( var i=0, l=APP_REF_LIST.length; i<l; ++i ){
					tail = new TailClass( APP_REF_LIST[ i ] );
					tailList.push( tail );
					elm  = tail.elm;
					elmContainer.appendChild( elm );
					self.addMouseEventListener( elm, 'click', onTailClick );
				};
			};
			
			function onTailClick( e ){
				var _children = elmContainer.getElementsByTagName( 'div' );
				for( var i=0, l=_children.length; i<l; ++i ){
					if( this === _children[ i ] ){
						APP_REF_LIST[ i ].boot();
						break;
					};
				};
			};
			
			this.bgColor     = '#0F6D39';
			this.MIN_WIDTH   = 320;
			this.MIN_HEIGHT  = 320;
			this.onInit = function(){
				self.rootElement.id = 'home-root';
				
				elmContainer        = document.createElement( 'div' );
				self.rootElement.appendChild( elmContainer );
				elmContainer.id     = 'home-tail-container';
				
				elmHeader           = document.createElement( 'div' );
				self.rootElement.appendChild( elmHeader );
				elmHeader.id        = 'home-header';
			};
			this.onOpen = function( _w, _h ){
				winW = _w;
				winH = _h;
				draw();
			};
			this.onPaneResize = function( _w, _h ){
				
			};
			this.onClose = function(){
				self.removeMouseEventListener();
				while( tailList.length > 0 ){
					tailList.shift().destroy();
				}
				self = tailList = elmContainer = null;
			};
		}, false, false, 'home', 'home', null );
		
		return {
			add: function( _appRef ){
				if( Application.isApplicationReference( _appRef ) === false ) return;
				Util.getIndex( APP_REF_LIST, _appRef ) === -1 && APP_REF_LIST.push( _appRef );
			},
			boot: function(){
				ref.boot();
			}
		}
	})();

	var Page = ( function(){
		var pageNodes = [];

		var ref;
		
		var ignoreTagList = [ 'script', 'noscript' ];
		
		var MemoryClass = function( node ){
			var _nodeType = node.nodeType;
			if( _nodeType === 1 && Util.getIndex( ignoreTagList, node.tagName.toLowerCase() ) === -1 ){
				this.node    = node;
				this.type    = _nodeType;
				this.display = node.style.display;
			} else
			if( _nodeType === 3 ){
				if( node.data.replace( /\s/g, '' ).length !== 0 ){
					this.node    = node;
					this.type    = _nodeType;
					this.before  = pageNodes.length === 0 ? null : pageNodes[ pageNodes.length - 1 ].node;
				} else {
					body.removeChild( node );
				}
			} else {
				// body.removeChild( node );
				return false;
			};
		};
		MemoryClass.prototype = {
			show: function(){
				if( this.type === 1 ){
					if( this.display ){
						this.node.style.display = this.display;
					} else {
						this.node.style.display = '';
					}
				} else {
					if( this.before ){
						body.insertBefore( this.node, this.before );
					} else {
						body.appendChild( this.node );
					};
				};
			},
			hide: function(){
				if( this.type === 1 ){
					this.node.style.display = 'none';
				} else {
					body.removeChild( this.node );
				};
			}
		};
		
		return {
			onReady: function(){
				var _children = Util.copyArray( body.childNodes ),
					_mem;
				for( var i = 0, l = _children.length; i<l; ++i ){
					_mem = new MemoryClass( _children[ i ] );
					_mem.node && pageNodes.push( _mem );
				};
				if( pageNodes.length !== 0 ){
					ref = Application.register( function(){
						var self     = this,
							winW     = 0,
							winH     = 0,
							tailList = [],
							elmContainer, elmHeader;
						
						this.bgColor     = '#ffffff';
						this.MIN_WIDTH   = 320;
						this.MIN_HEIGHT  = 320;
						this.onInit = function(){
							for( var i=0, l = pageNodes.length; i < l; ++i ){
								pageNodes[ i ].show();
							};
							KeyEvent.add( self, Const.KEY.EVENT.KEY_DOWN, ref.shutdown, 27 ); // 27.esc
						};
						this.onOpen = function( _w, _h ){
			
						};
						this.onPaneResize = function( _w, _h ){
							
						};
						this.onClose = function(){
							for( var i=0, l = pageNodes.length; i < l; ++i ){
								pageNodes[ i ].hide();
							};
						};
					}, false, true, doc.title, 'page', null, '#999999' );
				};
				delete Page.onReady;
			},
			boot: function(){
				ref && ref.boot();
			},
			registered: function(){
				return !!ref;
			}
		}
	})();

/* --------------------------------------------------------------
 * Event
 * 
 *  screenX
 *  スクリーン座標は、コンピュータのディスプレイの左上を原点とする座標系である。screenX, screenY属性で取得できる。Javascritpでは、同名のプロパティとして実装されている。
 *  しかし、これは、現実的には、何の役に立たない。ブラウザ自体がディスプレイのどの位置にいるのかがわからないので、画面上の位置を知ったところで、何にもならないからだ。 
 * 
 *  clientX
 *  ウインドウ座標とは、現在のブラウザのウインドウの、ドキュメントを表示している部分の左上原点とした座標である。
 *  問題は、ウインドウは、必ずしもドキュメント全体を表示するとは限らない。スクロールと呼ばれるUIによって、ドキュメントの一部だけを表示しているかもしれない。
 */
	var XBrowserEvent = ( function(){
		var wrappedHandlerClass,
			wrappedEventClass,
			tmp;
		
		if( window.addEventListener ){
			wrappedHandlerClass = function( element, handler ){
				this.handler = function( e ){
					if( handler.call( element, e ) !== false ) return;
					e.preventDefault();
					e.stopPropagation();
					return false;
				};
				this.destroy = function(){
					element = handler = null;
					delete this.handler;
					delete this.destroy;
				};
			};
		} else {
			wrappedEventClass = function( e, element ){
				this._event        = e;
				this.type          = e.type;
				this.target        = e.srcElement;
				this.currentTarget = element;
				this.relatedTarget = e.formElement ? e.formElement : e.toElement;
				this.eventPhase    = e.srcElement === element ? 2: 3;
				
				this.clientX       = e.clientX;
				this.clientY       = e.clientY;
				this.screenX       = e.screenX;
				this.screenY       = e.screenY;
				
				this.keyCode       = e.keyCode;
				this.altKey        = e.altKey;
				this.ctrlKey       = e.ctrlKey;
				this.shiftKey      = e.shiftKey;
				
				this.wheelDelta    = e.wheelDelta;
				
				e = element = null;
			};
			wrappedEventClass.prototype.stopPropagation = function(){
				this._event.cancelBubble = true;
			};
			wrappedEventClass.prototype.preventDefault  = function(){
				this._event.returnValue = false;
			};

			if( doc.attachEvent ){
				wrappedHandlerClass = function( element, handler ){
					this.handler = function(){
						var e = new wrappedEventClass( window.event, element );
						if( handler.call( element, e ) !== false ) return;
						e.preventDefault();
						e.stopPropagation();
						return false;
					};
					this.destroy = function(){
						element = handler = null;
						delete this.handler;
						delete this.destroy;
					};
				};
			} else {
				tmp = {
					list: [],
					find: function( _ticket ){
						for( var i=0, l= tmp.list.length, _item; i<l; ++i ){
							_item = tmp.list[ i ];
							if( _item.match( _ticket.element, _ticket.eventType ) === true ){
								return _item;
							};
						};
						return null;
					}
				};
				tmp.ticketClass = function( _ticket ){
					var self = this;
					this.element   = _ticket.element;
					this.eventType = _ticket.eventType;
					this.handlers  = [ _ticket.handler ];
					this.element[ 'on' + this.eventType ] = function( e ){
						return self.fire( self, e );
					};
					this.clean = function(){
						self = null;
					};
					_ticket = null;
				};
				tmp.ticketClass.prototype = {
					add: function( _ticket ){
						this.match( _ticket.element, _ticket.eventType ) === true && 
							this.match( null, null, _ticket.handler ) === false &&
								this.handlers.push( _ticket.handler );
					},
					remove: function( _ticket ){
						if( this.match( _ticket.element, _ticket.eventType, _ticket.handler ) === true ){
							var i = Util.getIndex( this.handlers, handler );
							i !== 0 && this.handlers.splice( i, 1 );
							this.handlers.length === 0 && this.destroy();
						}
					},
					fire: function( self, e ){
						e = e || new wrappedEventClass( window.event, self.element );
						for( var i=self.handlers.length, cancel; i; ){
							self.element._currentHandler = self.handlers[ --i ];
							if( self.element._currentHandler( e ) === false ) cancel = false;
							delete self.element._currentHandler;
						};
						return cancel;
					},
					match: function( element, eventType, handler ){
						if( handler && Util.getIndex( this.handlers, handler ) === -1 ) return false;
						if( eventType && this.eventType !== eventType ) return false;
						if( element && this.element !== element ) return false;
						return true;
					},
					destroy: function(){
						this.clean();
						this.element[ 'on' + this.eventType ] = '';
						tmp.list.splice( Util.getIndex( tmp.list, this ), 1 );
						delete this.element;
						delete this.eventType;
						delete this.handlers;
						delete this.clean;
					}
				};
			};
		};

		return {
			add: function( _ticket ){
				if( doc.addEventListener ){
					XBrowserEvent.add = function( _ticket ){
						_ticket.wrappedHandler = new wrappedHandlerClass( _ticket.element, _ticket.handler );
						_ticket.element.addEventListener( _ticket.eventType, _ticket.wrappedHandler.handler, false );
					}
				} else
				if( doc.attachEvent ){
					XBrowserEvent.add = function( _ticket ){
						_ticket.wrappedHandler = new wrappedHandlerClass( _ticket.element, _ticket.handler );
						_ticket.element.attachEvent( 'on' + _ticket.eventType, _ticket.wrappedHandler.handler );
					};
				} else {
					XBrowserEvent.add = function( _ticket ){
						var t = tmp.find( _ticket );
						if( t !== null ){
							t.add( _ticket );
						} else {
							tmp.list.push( new tmp.ticketClass( _ticket ) );
						};
					};
				}
				
				XBrowserEvent.add( _ticket );
			},
			remove: function( _ticket ){
				if( doc.removeEventListener ){
					XBrowserEvent.remove = function( _ticket ){
						_ticket.element.removeEventListener( _ticket.eventType, _ticket.wrappedHandler.handler, false );
						_ticket.wrappedHandler.destroy();
					};
				} else
				if( doc.detachEvent ){
					XBrowserEvent.remove = function( _ticket ){
						_ticket.element.detachEvent( 'on' + _ticket.eventType, _ticket.wrappedHandler.handler );
						_ticket.wrappedHandler.destroy();
					};
				} else {
					XBrowserEvent.remove = function( _ticket ){
						var t = tmp.find( _ticket );
						if( t !== null ){
							t.remove( _ticket );
						};
					};
				};
				
				XBrowserEvent.remove( _ticket );
			}
		}
	})();

/*
 * EventTicketClass
 */
	var EventTicketClass = function( _element, _eventType, _handler ){
		this.element        = _element;
		this.eventType      = _eventType;
		this.handler        = _handler;
		this.wrappedHandler = null; // for ie
		
		XBrowserEvent.add( this );
		
		_element = _eventType = _handler = null;
	};
	EventTicketClass.prototype = {
		match: function( _element, _eventType, _handler ){
			if( _handler   && _handler   !== this.handler )   return false;
			if( _eventType && _eventType !== this.eventType ) return false;
			if( _element   && _element   !== this.element )   return false;
			
			return true;
		},
		destroy: function( _element, _eventType, _handler ){
			if( this.match( _element, _eventType, _handler ) === false ) return false;
			
			XBrowserEvent.remove( this );
			
			delete this.element;
			delete this.eventType;
			delete this.handler;
			delete this.wrappedHandler;
			
			return true;
		}
	};

var ReadyEvent = ( function(){
	var ticketReady,
		ticketLoad;

	function webkitDetect(){
		var state = document.readyState;
		if( state === 'loaded' || state === 'complete' ){
			SystemTimer.remove( SUPER_USER_KEY, webkitDetect );
			timer = null;
			onReady();
		};
	};
	function ieDetect(){
		if( this.readyState === 'complete' ){ // this.readyState === 'loaded' || 
			this.onreadystatechange = new Function();
			this.onreadystatechange = null;
			this.parentNode.removeChild( this );
			onReady();
		};
	};
		
	function onReady(){
		ticketReady && ticketReady.destroy();
		ticketLoad  && ticketLoad.destroy();
		ticketReady = ticketLoad = null;
		Page.onReady();
		if( Page.registered() === true ){
			Page.boot();
		} else {
			Home.boot();
		};
	};
	
	// Apple WebKit (Safari, OmniWeb, ...)
	if( doc.readyState && !!UA.WEBKIT ){
		SystemTimer.add( SUPER_USER_KEY, webkitDetect, 50 );
	} else
	/*
	if( document.readyState && UA.isIE && UA.ieVersion < 9 ){
		document.write('<script type="text\/javascript" defer="defer" id="ieDOMContentLoaded" src="' +
			( ( window.location.protocol === 'https:' ) ? ':\/\/0' : 'javascript:void(0)' ) +
			'"><\/script>');
		document.getElementById( 'ieDOMContentLoaded' ).onreadystatechange = ieDetect;
	} else */ {
		ticketReady = new EventTicketClass( document, 'DOMContentLoaded', onReady );
		ticketLoad  = new EventTicketClass( window, 'load', onReady );
	};
})();




/* =====================================================
 *  ResizeEvent
 * 
 */

var ResizeEvent = ( function(){
	var _globalLock = 0;
	var _resize;
	var root = window;
	var w = 0, h = 0;
	
	function getInnerSize(){
		return {
			w : root.innerWidth || root.clientWidth,
			h : root.innerHeight || root.clientHeight
		};
	}
	function unlock(){
		_globalLock = 0;
	}
	
	if( document.uniqueID ){
		_resize = function(){
			root = (doc.compatMode || "") !== "CSS1Compat" ? doc.body : doc.documentElement;

			// resize agent
			function loop(){
				if( !_globalLock++ ){
					var size = getInnerSize();
					if( w !== size.w || h !== size.h ){// resized
						w = size.w;
						h = size.h;
						// update
						Application.onWindowResize( w, h );
					}
					window.setTimeout( unlock, 0 );
					// delay unlock
				}
				window.setTimeout( loop, 100 );
			}
			loop();
		};
	} else {
		_resize = function(){
			new EventTicketClass( window, 'resize', onResize );
			
			function onResize(){
				if( !_globalLock++ ) {
					var size = getInnerSize();
					if( w !== size.w || h !== size.h ){// resized
						w = size.w;
						h = size.h;
						// update
						Application.onWindowResize( w, h );
					}
					window.setTimeout( unlock, 0 );
				}
			}
			onResize();
		};
	}
	AsyncCall.add( SUPER_USER_KEY, _resize );
	
	return {
		getSize: getInnerSize,
		onSystemShutdown: function(){
			
		}
	}
})();


/* =====================================================
 *  MouseEvent
 * 
 */
	var MouseEvent = ( function(){
		var CLICK_OFFSET   = 2 * 2;		
		
		var EVENT_LIST_MAP = [],
			TMP = {},
			currentEventList;

		/* ClickHelper
		 * mousedown, mouseup, の移動距離を調べて clickハンドラ を呼ぶ
		 */
		var ClickEventTicketClass = function( element, clickhandler ){
			var startX, startY,
				mousedownTicket = new EventTicketClass( element, 'mousedown', mousedownHandler ),
				mouseupTicket, mouseoutTicket;
			
			function mousedownHandler( e ){
				startX = e.clientX;
				startY = e.clientY;
				
				mouseupTicket  = new EventTicketClass( element, 'mouseup', mouseupHandler );
				mouseoutTicket = new EventTicketClass( element, 'mouseout', mouseoutHandler );				
			}
			
			function mouseupHandler( e ){
				var offsetX = e.clientX - startX,
					offsetY = e.clientY - startY;
				mouseoutHandler();
				if( offsetX * offsetX + offsetY * offsetY < CLICK_OFFSET ){
					if( Function.prototype.call ){
						return clickhandler.call( element, e );
					}
					element._currentHandler = clickhandler;
					var ret = element._currentHandler( e );
					delete element._currentHandler;
					return ret;
				};
			};
			function mouseoutHandler( e ){
				mouseupTicket && mouseupTicket.destroy();
				mouseoutTicket && mouseoutTicket.destroy();
				mouseupTicket = mouseoutTicket = null;
			};
			
			this.element = element;
			this.handler = clickhandler;		
			this.destroy = function( _element, _eventType, _handler ){
				if( this.match( _element, _eventType, _handler ) === false ) return false;
				
				mouseoutHandler();
				mousedownTicket.destroy();
				element = clickhandler = mousedownTicket = null;
				
				delete this.element;
				delete this.eventType;
				delete this.handler;
				
				return true;
			};
		};
		ClickEventTicketClass.prototype = {
			eventType: 'click',
			match    : EventTicketClass.prototype.match
		};
		
		/* WheelHelper
		 */
		var WheelEventTicketClass = ( function(){
			if( UA.GECKO ){
				return function( element, wheelhandler ){
					this.wheelTicket = new EventTicketClass( element, 'DOMMouseScroll', onWheel );
					this.element     = element;
					this.handler     = wheelhandler;
					this.destroy     = function( _element, _eventType, _handler ){
						if( this.match( _element, _eventType, _handler ) === false ) return false;
						element = wheelhandler = null;
						
						this.wheelTicket && this.wheelTicket.destroy();
						
						delete this.wheelTicket;
						delete this.element;
						delete this.handler;
						
						return true;
					};
					
					function onWheel( e ){
						e.wheelDelta = e.detail * -40;
						return wheelhandler.call( element, e );
					}
				};
			} else
			if( true || UA.isIE ){
				return function( element, wheelhandler ){
					this.wheelTicket    = new EventTicketClass( element, 'mousewheel', wheelhandler );
					this.element        = element;
					this.handler        = wheelhandler;
					element = wheelhandler = null;
				};
			} else {
				TMP.wheelList   = [];
				//TMP.wheelLegacy = undefined;
				TMP.onWheel   = function( e ){
					e = e || window.event;
					var cancel = true,
						f = TMP.wheelLegacy;
					if( f ) cancel = f.call( this, e );
					
					for( i = TMP.wheelList.length; i; ){
						if( TMP.wheelList[ --i ].call( this, e ) === false ) cancel = false;
					}
					return cancel;
				};
				return function( element, wheelhandler ){
					this.wheelTicket    = null;
					this.element        = element;
					this.handler        = wheelhandler;
					this.destroy        = function( _element, _eventType, _handler ){
						if( this.match( _element, _eventType, _handler ) === false ) return false;
						
						TMP.wheelList.splice( Util.getIndex( TMP.wheelList, this.handler ) );
						if( TMP.wheelList.length === 0 ) this.element.onmousewheel = '';
						
						delete this.element;
						delete this.handler;
						
						return true;
					};
					
					if( TMP.wheelList.length === 0 ){
						//TMP.wheelLegacy     = Type.isFunction( window.onmousewheel ) === true ? window.onmousewheel : undefined;
						element.onmousewheel = TMP.onWheel;
					};
					TMP.wheelList.push( wheelhandler );
					element = wheelhandler = null;
				};
			}
		})();
		WheelEventTicketClass.prototype = {
			eventType : 'mousewheel',
			match     : EventTicketClass.prototype.match,
			destroy   : function( _element, _eventType, _handler ){
				if( this.match( _element, _eventType, _handler ) === false ) return false;
				
				this.wheelTicket && this.wheelTicket.destroy();
				
				delete this.wheelTicket;
				delete this.element;
				delete this.handler;
				
				return true;
			}
		};
		
		
		return {
			add: function( _apiuser, _element, _eventType, _handler ){
				if( isApiUser( _apiuser ) === true &&
					( Type.isHTMLElement( _element ) === true || _element === window || _element === doc ) &&
					Type.isString( _eventType ) === true &&
					Type.isFunction( _handler ) === true
				){
					var _uid    = _apiuser.getUID(),
						_events = EVENT_LIST_MAP[ _uid ];
					if( Type.isArray( _events ) === false ){
						_events = EVENT_LIST_MAP[ _uid ] = [];
					} else {
						// 2重登録の禁止
						for( var i=0, l=_events.length; i<l; ++i ){
							if( _events[ i ].match( _element, _eventType, _handler ) === true ) return;
						}
					};
					if( _eventType === 'click' ){
						_events.push( new ClickEventTicketClass( _element, _handler ) );
					} else
					if( _eventType === 'mousewheel' ){
						_events.push( new WheelEventTicketClass( _element, _handler ) );
					} else {
						_events.push( new EventTicketClass( _element, _eventType, _handler ) );
					};
				};
			},
			remove: function( _apiuser, _element, _eventType, _handler ){
				if( isApiUser( _apiuser ) === true ){
					var _uid    = _apiuser.getUID(),
						_events = EVENT_LIST_MAP[ _uid ],
						_removed,
						i = 0;
					if( Type.isArray( _events ) === false ) return;
					for( ;i < _events.length; ){
						if( _events[ i ].destroy( _element, _eventType, _handler ) === true ){
							_events.splice( i, 1 );
						} else {
							++i;
						}
					}
					if( _events.length === 0 ){
						_events = EVENT_LIST_MAP[ _uid ] = null;
					}
				}
			},
			onCurrentApplicationChange: function(){
				
			},
			onApplicationShutdown: function(){
				
			},
			onSystemShutdown: function(){
				
			}
		}
	})();

/* ----------------------------------------
 * KEY
 * 
 *  - EDITABLE_TEXT_CONTROL
 * 
 *    .SHIFT_DOWN_EVENT:	'shiftDown',
 *    .SHIFT_UP_EVENT:		'shiftUp',
 *    .CTRL_DOWN_EVENT:		'ctrlDown',
 *    .CTRL_UP_EVENT:		'ctrlUp',
 *    .SPACE_DOWN_EVENT:	'spaceDown',
 *    .SPACE_UP_EVENT:		'spaceUp',
 *    .init:				function,
 *    .addKeyDownEvent:		function,
 *    .keyEventDispatcher:	function,
 * 
 * ショートカットキーの監視とテキスト入力(input, textarea)、チェックボックスを管理する。
 * キー入力はdocumentで受けて、テキスト編集中(input, textarea)はそちらにキーイベント流す。
 * 
 */
var KeyEvent = ( function(){
	var EVENT_LIST_MAP = [],
		LOCK_UP        = [],
		LOCK_DOWN      = [],
		application    = null,
		currentList    = null;

	window.focus();
	
	var focusTicket    = null,
		keydownTicket  = null,
		keyupTicket    = null,
		keyPress       = null,
		keypressTicket = null;
	
	function unlock( lock, key ){
		lock.splice( Util.getIndex( lock, key ), 1 );
	};
	
	function onKeyChange( e ){
		var cancel         = false,
			type           = e.type,
			key            = e.keyCode, // || e.which,
			shift          = Type.isBoolean( e.shiftKey ) === true ? e.shiftKey : ( e.modifiers & Event.SHIFT_MASK ),
			ctrl           = Type.isBoolean( e.ctrlKey  ) === true ? e.ctrlKey  : ( e.modifiers & Event.CONTROL_MASK ),
			lock           = type === 'keyup' ? LOCK_UP : LOCK_DOWN;
			
		// block chattering
		if( Util.getIndex( lock, key ) !== -1 ) return;
		lock.push( key );
		AsyncCall.add( SUPER_USER_KEY, unlock, [ lock, key ] );
		
		if( key === 16 || shift === true ){
			KeyEvent.shiftEnabled = type !== 'keyup';
		};
		if( key === 17 || ctrl === true ){
			KeyEvent.ctrlEnabled  = type !== 'keyup';
		};
		for( var i=currentList.length, t; t = currentList[ --i ]; ){
			if( Type.isFunction( t[ type ] ) === true && t.keyCode === key && ( t.shift === undefined || t.shift === shift ) && ( t.ctrl === undefined || t.ctrl === ctrl )){
				AsyncCall.add( t.apiuser, t[ type ], e );
				cancel = true;
			};
		};
		if( cancel === true || key === 18 || key === 9 || key === 27 || e.altKey === true ){ // 13.enter 18.esc 9.tab 27.esc   || ( key === 13 && overlayEnabled === false)
			return false;
		};
	};
	
	if( UA.isIE === true && UA.ieRenderingVersion < 9 ){
		keyPress = function( e ){
			var key = e.keyCode;
			if( key === 13 || key === 27 ){
				e.type = 'keydown';
				return onKeyChange( e );
			};
		};
	};
	
	var KeyEventTicketClass = function( _apiuser, _type, _onKeydown, _onKeyup, _keyCode, _shift, _ctrl ){
		this.apiuser = _apiuser;
		this.type    = _type;
		this.keydown = _onKeydown;
		this.keyup   = _onKeyup;		
		this.keyCode = _keyCode;
		this.shift   = _shift;
		this.ctrl    = _ctrl;
		_apiuser = _onKeydown = _onKeyup = null;
	}
	KeyEventTicketClass.prototype = {
		match: function( _apiuser, _type, _handler, _keyCode, _shift, _ctrl ){
			if( _apiuser  && _apiuser  !== this.apiuser ) return false;
			if( _type     && _type     !== this.type )    return false;
			if( _handler ){
				if( this.type === 'keydown' ){
					if( _handler !== this.keydown ) return false;
				} else {
					if( _handler !== this.keyup )   return false;
				};
			};
			if( _keyCode  && _keyCode  !== this.keyCode ) return false;
			if( _shift    && _shift    !== this.shift )   return false;
			if( _ctrl     && _ctrl     !== this.ctrl )    return false;
			return true;
		},
		destroy: function( _apiuser, _type, _handler, _keyCode, _shift, _ctrl ){
			if( this.match( _apiuser, _type, _handler, _keyCode, _shift, _ctrl ) === false ) return false;
			
			delete this.apiuser;
			delete this.keydown;
			delete this.keyup;
			
			return true;
		}
	};
	
	function registerEvent( _apiuser, _type, _onKeydown, _onKeyup, _keyCode, _shift, _ctrl ){
		var _uid  = _apiuser.getUID(),
			_list = EVENT_LIST_MAP[ _uid ];
		if( Type.isArray( _list ) === false ){
			_list = EVENT_LIST_MAP[ _uid ] = [];
		}
		for( var i=0, l=_list.length; i<l; ++i ){
			if( _list[ i ].match( _apiuser, _type, _onKeydown || _onKeyup, _keyCode, _shift, _ctrl ) === true ) return;
		}
		_list.push( new KeyEventTicketClass( _apiuser, _type, _onKeydown, _onKeyup, _keyCode, _shift, _ctrl ));
		
		if( _apiuser === application ) KeyEvent.updateCurrentListener( _apiuser );
	};
	
	return {
		add: function( _apiuser, _type, _handler, _keyCode, _shift, _ctrl ){
			if( _type === 'keydown' ){
				registerEvent( _apiuser, _type, _handler, null, _keyCode, _shift, _ctrl );
			} else
			if( _type === 'keyup' ){
				registerEvent( _apiuser, _type, null, _handler, _keyCode, _shift, _ctrl );
			} else
			if( _type === 'keychange' ){
				registerEvent( _apiuser, _type, _handler, _handler, _keyCode, _shift, _ctrl );
			} else
			if( _type === 'cursol' ){
				
			};
		},
		remove: function( _apiuser, _type, _handler, _keyCode, _shift, _ctrl ){
			var _list = EVENT_LIST_MAP[ _apiuser.getUID() ],
				i = 0;
			if( Type.isArray( _list ) === true ){
				while( i < _list.length ){
					if( _list[ i ].destroy( _apiuser, _type, _handler, _keyCode, _shift, _ctrl ) === true ){
						_list.splice( i, 1 );
					} else {
						++i;
					}
				}
			}
			if( _apiuser === application ) KeyEvent.updateCurrentListener( _apiuser );
		},
		shiftEnabled: false,
		ctrlEnabled:  false,
		/*
		 * currentListener
		 *  currrentApplication ( overlay Application ) or
		 *  superuser ( UI )
		 */
		updateCurrentListener: function( _apiuser ){
			application = _apiuser;
			var _uid    = _apiuser.getUID();
			currentList = EVENT_LIST_MAP[ _uid ] || ( EVENT_LIST_MAP[ _uid ] = [] );
			
			var _ticket,
				_down = false,
				_up   = false;
			for( var i=currentList.length; _ticket = currentList[ --i ]; ){
				if( _down === false ) _down = !!_ticket.keydown;
				if( _up   === false ) _up   = !!_ticket.keyup;
				if( _down && _up ) break;
			}
			if( _down === true ){
				keydownTicket = new EventTicketClass( document, 'keydown', onKeyChange );
				keypressTicket = keyPress !== null ? new EventTicketClass( doc, 'keypress', keyPress ) : null;
			} else {
				keydownTicket && keydownTicket.destroy();
				keypressTicket && keypressTicket.destroy();
				keydownTicket = keypressTicket = null;
			}
			if( _up === true ){
				keyupTicket   = new EventTicketClass( document, 'keyup', onKeyChange );
			} else {
				keyupTicket && keyupTicket.destroy();
				keyupTicket = null;
			}
			
			if( _down === true || _up === true ){
				focusTicket   = new EventTicketClass( document, 'mouseenter', window.focus );
			} else {
				focusTicket && focusTicket.destroy();
				focusTicket = null;
			}
		},
		onApplicationShutdown: function( _apiuser ){
			KeyEvent.remove( _apiuser );
		},
		onSystemShutdown: function(){
			
		}
	}
})();

/**
 * 
 * http://thudjs.tumblr.com/post/637855087/stylesheet-onload-or-lack-thereof
 */

var Css = ( function(){
	var head = doc.getElementsByTagName( 'head' )[ 0 ];
	
	var TICKET_LIST = [];
	var STATE_LIST  = 'loaded,complete,uninitialized'.split( ',' );
	
	var cssRules, sheet;
	
	var FetchCssTicketClass = function( _apiuser, _url, _elm, _onload, _onerror ){
		this.apiusers = [ _apiuser ];
		this.url      = _url;
		this.elm      = _elm;
		this.onload   = [ _onload ];
		this.onerror  = [ _onerror ];
		this.time     = 0;
	};
	FetchCssTicketClass.prototype = {
		match: function( _apiuser, _url ){
			if( _apiuser  && Util.getIndex( this.apiusers, _apiuser ) === -1 ) return false;
			if( _url      && _url      !== this.url     ) return false;
			return true;
		},
		destroy: function( _apiuser, _url ){
			if( this.match( _apiuser, _url ) === false ) return false;
			
			var i = Util.getIndex( this.apiusers, _apiuser );
			
			this.apiusers.splice( i, 1 );
			this.onload.splice( i, 1 );
			this.onerror.splice( i, 1 );
			
			if( this.apiusers.length !== 0 ) return false;
			
			head.removeChild( this.elm );
			this.elm.onreadystatechange = new Function();
			this.elm.onload = null;
			
			delete this.apiusers;
			delete this.url;
			delete this.elm;
			delete this.onload;
			delete this.onerror;
			delete this.time;
			
			return true;
		},
		loaded: function(){
        	for( var i = this.onload.length, f; i; ){
        		f = this.onload[ --i ];
        		Type.isFunction( f ) === true && AsyncCall.add( this.apiusers[ i ], f, this.url );
        		this.onload[ i ] = this.onerror[ i ] = null;
        	};
		},
		error: function(){
        	for( var i = this.onerror.length, c; i; ){
        		c = this.onerror[ --i ];
        		Type.isFunction( c ) === true && AsyncCall.add( t.apiusers[ i ], c, t.url );
        		this.onload[ i ] = this.onerror[ i ] = null;
        	};
		},
		check: function(){
			var el = this.elm;
			try {
				return el[ sheet ] && el[ sheet ][ cssRules ].length > 0;
			} catch( e ){
				return false;
			};
		},
		done: false
	};
	
	function getTicket( elm ){
		for( var i = TICKET_LIST.length, t; i; ){
			t = TICKET_LIST[ --i ];
			if( t.elm === elm ) return t;
		};
	};
	
	function detect(){
		var t = getTicket( this ), rs = this.readyState, c;
		if( t && t.done === false && ( !rs || Util.getIndex( STATE_LIST, rs ) !== -1 ) ){
			t.done = true;
        	t.loaded();
        	this.onreadystatechange = new Function();
        	this.onload = null;
		};
	};
	
	function checkTimer(){
		var l = TICKET_LIST.length,
			n = 0;
		for( var i = 0; i < l; ++i ){
			t = TICKET_LIST[ i ];
			++t.time;
			if( t.check() === true ){
				t.loaded();
				++n;
			} else
			if( t.time > 99 ){
				t.error();
			} else {
				
			};
		};
		l === n && SystemTimer.remove( SUPER_USER_KEY, checkTimer );
	};
	
	return {
		load: function( _apiuser, _url, opt_onload, opt_onerror ){
			_url = Util.getAbsolutePath( _url );
			var t;
			for( var i=TICKET_LIST.length; i; ){
				t = TICKET_LIST[ --i ];
				if( t.match( null, _url ) === true ){
					if( t.match( _apiuser, _url ) === false ){
						t.apiusers.push( _apiuser );
						t.onload.push( opt_onload );
						t.onerror.push( opt_onerror );
					};
					SystemTimer.add( SUPER_USER_KEY, checkTimer, 333 );
					return;
				};
			};
			var elm = doc.createElement( 'link' );
			head.appendChild( elm );
			elm.rel  = 'stylesheet';
			elm.type = 'text\/css';
			elm.onreadystatechange = elm.onload = detect;
			elm.href = _url;
			
			if( !sheet ){ // only assign these once
				cssRules = 'cssRules';
				sheet    = 'sheet';
				if ( !( sheet in elm ) ) { // MSIE uses non-standard property names
					cssRules = 'rules';
					sheet    = 'styleSheet';
				};
			};
			
			TICKET_LIST.push( new FetchCssTicketClass( _apiuser, _url, elm, opt_onload, opt_onerror ) );
			
			SystemTimer.add( SUPER_USER_KEY, checkTimer, 333 );
		},
		unload: function( _apiuser, _url ){
			_url = _url ? Util.getAbsolutePath( _url ) : null;
			for( var i = 0; i < TICKET_LIST.length; ){
				t = TICKET_LIST[ i ];
				if( t.destroy( _apiuser, _url ) === true ){
					TICKET_LIST.splice( i, 1 );
				} else {
					++i;
				}
			};
			if( TICKET_LIST.length === 0 ){
				SystemTimer.remove( SUPER_USER_KEY, checkTimer );
			}
		}
	}
})();


/* ----------------------------------------
 * 
 */

var Overlay = ( function(){
	var elmContainer, elmShadow, elmCloseButton,
		application    = null,
		visible        = false,
		bodyOverflow   = '',
		windowW, windowH;

	function onCloseClick( e ){
		Overlay.hide();
		return false;
	};
	function asyncInit( /* arguments */ ){	
		
		//body.appendChild( application.rootElement );
		elmContainer.insertBefore( application.rootElement, elmCloseButton );
		application.rootElement.style.display = 'none';
		application.init();
	};
	function asyncOpen( /* arguments */ ){
		
		var _arg = Util.copyArray( arguments );
		_arg.unshift( windowW, windowH );
		application.open.apply( application, _arg );
		
		elmContainer.style.cssText = "top:" + body.scrollTop + 'px;display:none;';
		$( elmContainer ).stop().fadeIn( onFadeInComplete );		
	};
	function onFadeInComplete(){
		KeyEvent.add( application, Const.KEY.EVENT.KEY_DOWN, Overlay.hide, 27 ); // 27.esc
		MouseEvent.add( application, elmCloseButton, 'click', onCloseClick );
	};
	function onFadeOutComplete(){	
		Util.removeAllChildren( elmContainer );
		body.removeChild( elmContainer );
		elmContainer = elmShadow = elmCloseButton = null;
	};
	return {
		show: function( _application, _bootParams ){
			if( visible === true && application === _application ) return;
			if( Application.isApplicationInstance( _application ) === false ) return;
			
			elmContainer = document.createElement( 'div' );
			body.appendChild( elmContainer );
			
			elmContainer.id = 'overlay-container';
			
			bodyOverflow        = body.style.overflow;
			body.style.overflow = 'hidden';
			
			elmShadow = document.createElement( 'div' );
			elmContainer.appendChild( elmShadow );
			elmShadow.id = 'overlay-shadow';
			
			elmCloseButton  = document.createElement( 'div' );
			elmContainer.appendChild( elmCloseButton );
			elmCloseButton.id = 'overlay-close-button';
			elmCloseButton.appendChild( document.createTextNode( 'x' ) );
			
			elmContainer.style.display = 'none'; // hide for fadeIn
			
			_application.addAsyncCall( asyncInit );
			_application.addAsyncCall( asyncOpen, _bootParams );
			
			visible     = true;
			application = _application;
		},
		hide: function(){
			if( visible === false ) return;
			if( application.close() === false ) return false;
			
			body.style.overflow = bodyOverflow;
			
			$( elmContainer ).stop().css( {
				filter:		'',
				opacity:	''
			}).fadeOut( onFadeOutComplete );
			visible = false;
			
			KeyEvent.remove( application, Const.KEY.EVENT.KEY_DOWN, Overlay.hide ); // 27.esc
			MouseEvent.remove( application, elmCloseButton );
			
			application = null;
		},
		onWindowResize: function( _windowW, _windowH ){
			windowW = _windowW;
			windowH = _windowH;			
			
			if( application === null ) return;
			
			elmContainer.style.height = _windowH + 'px';
			elmContainer.style.top    = body.scrollTop + 'px';

			elmShadow.style.height = _windowH + 'px';

			AsyncCall.add( application, application.resize, [ _windowW, _windowH ] );
		}
	}
})();

/* ----------------------------------------
 * UI
 * 
 * keyEventRellay
 *  form -> overlay -> view
 * 
 */

var UI = ( function(){
	var UI_LIST     = [],
		currentUser = null,
		currentList = null,
		currentUi   = null,
		currentItem = null,
		windowW     = 0,
		windowH     = 0;

	var CLASSNAME_COMBOBOX_OPTION = 'combobox-option',
		CLASSNAME_COMBOBOX_OPTION_CURRENT = CLASSNAME_COMBOBOX_OPTION + ' combobox-option-current',
		ELM_A_ORIGIN = ( function(){
			var ret = document.createElement( 'a' );
			ret.href = '#';
			return ret;
		})(),
		ELM_INPUT_TEXT = ( function(){
			var ret = document.createElement( 'input' );
			ret.type = 'text';
			return ret;
		})(),
		ELM_COMBOBOX = ( function(){
			var ret       = document.createElement( 'a' ),
				elmToggle = document.createElement( 'span' ),
				elmValue  = document.createElement( 'span' );
			ret.href = '#';
			ret.appendChild( elmToggle );
			ret.appendChild( elmValue );
			elmToggle.className = 'combobox-toggle';
			elmValue.className  = 'combobox-value';
			
			elmToggle.appendChild( document.createTextNode( '▼' ));
			elmValue.appendChild( document.createTextNode( 'null' ));
			return ret;
		})();
		
	var InputTextClass = function( apiuser, uiGroup, elmWrapper, elmValue, onUpdate, validater ){
		var elmA     = ELM_A_ORIGIN.cloneNode( true ),
			instance = this,
			focus    = false,
			visible  = true,
			enabled  = true,
			value    = elmValue.innerHTML;
		elmValue.innerHTML = '';
		elmValue.className += ' editable-text';
		elmValue.appendChild( elmA );
		
		this.value = function( _value ){
			if( Type.isString( _value ) === true || Type.isNumber( _value ) === true ){
				value = '' + _value;
				elmA.innerHTML = '' + _value;
				if( focus === true ){
					ELM_INPUT_TEXT.value = '' + value;
				};
			};
			focus === true && instance.blur();
			return value;
		};
		this.focus = function( e ){
			focus = true;
			start( apiuser, uiGroup, instance );
			elmA.style.display = 'none';
			elmValue.appendChild( ELM_INPUT_TEXT );
			
			ELM_INPUT_TEXT.value = value;
			ELM_INPUT_TEXT.focus();
			ELM_INPUT_TEXT.select();

			return false;
		};
		this.blur = function( keep ){
			var _newValue = ELM_INPUT_TEXT.value,
				_validated = Type.isFunction( validater ) === true ? '' + validater( _newValue ) : _newValue;
			_newValue = keep !== 27 ? _validated : value; // 27:ESC

			elmValue.removeChild( ELM_INPUT_TEXT );
			
			elmA.innerHTML     = _newValue;
			elmA.style.display = 'block';
			
			onUpdate && _newValue !== value && AsyncCall.add( apiuser, onUpdate, [ _newValue, value ] );
			
			value = _newValue;
			focus = false;
			finish( apiuser, uiGroup, instance );
		};
		this.enabled = function(){
			return enabled;
		};
		this.visible = function( _visible ){
			if( Type.isBoolean( _visible ) === true ){
				elmWrapper.style.display = _visible ? '' : 'none';
				visible = _visible;
			}
			return visible;
		};
		this.destroy = function(){
			if( focus === true ){
				elmValue.removeChild( ELM_INPUT_TEXT );
			}
			MouseEvent.remove( apiuser, elmWrapper );
			
			apiuser = uiGroup = elmWrapper = elmValue = elmA = onUpdate = validater = instance = null;
		};
		instance.value( value );
		MouseEvent.add( apiuser, elmWrapper, 'click', instance.focus );
	};
	
	var ButtonClass = function( apiuser, uiGroup, elmWrapper, onUpdate ){
		var instance  = this,
			focus     = false,
			visible   = true,
			enabled   = true;
		MouseEvent.add( apiuser, elmWrapper, 'click', onClick );
		
		function onClick(){
			focus = true;
			// onUpdate();
			AsyncCall.add( apiuser, onUpdate );
			return false;
		};
		this.focus = function(){
			focus = true;
			Util.addClass( elmWrapper, 'button-has-focus' );
			start( apiuser, uiGroup, instance );
		};
		this.blur = function( keyCode ){
			keyCode === 13 && onClick();
			Util.removeClass( elmWrapper, 'button-has-focus' );
			focus = false;
			finish( apiuser, uiGroup, instance );
		};
		this.enabled = function( _enabled ){
			if( Type.isBoolean( _enabled ) === true && enabled !== _enabled ){
				_enabled === true ? Util.removeClass( elmWrapper, 'button-disabled' ) : Util.addClass( elmWrapper, 'button-disabled' );
				enabled = _enabled;
			};
			return enabled;
		};
		this.visible = function( _visible ){
			if( Type.isBoolean( _visible ) === true ){
				elmWrapper.style.display = _visible ? '' : 'none';
				visible = _visible;
			};
			return visible;
		};
		this.destroy = function(){
			MouseEvent.remove( apiuser, elmWrapper );
			apiuser = uiGroup = elmWrapper = onUpdate = instance = null;
		};
	};

	var FileInputClass = function( apiuser, uiGroup, elmWrapper, onUpdate, validater, elmFileInput ){
		var elmFilePath = Util.getElementsByClassName( elmWrapper, 'file-path' )[ 0 ],
			focus       = false,
			visible     = true,
			enabled     = true,
			index       = GROUP_ID ? itemList[ GROUP_ID ].length : -1,
			value,
			instance;
		elmFileInput = elmWrapper.getElementsByTagName('input')[0] || elmFileInput || document.createElement( 'input');
		elmFileInput.type = 'file';
		elmFileInput.style.visivility = 'hidden';

		elmWrapper.onclick = onClick;
		elmFileInput.onchange = onChange;
		
		function onChange(){
			elmFilePath.innerHTML = elmFileInput.value;
		}
		return {
			init: function(){
				instance = this;
				delete this.init;
			},
			focus: function( e ){
				focus = true;
				start( apiuser, uiGroup, instance );
				elmFileInput.click();
				return false;
			},
			blur: function( keep ){
				focus = false;
			},
			enabled: function(){
				return enabled;
			},
			visible: function( _visible ){
				if( Type.isBoolean( _visible ) === true ){
					elmWrapper.style.display = _visible ? '' : 'none';
					visible = _visible;
				};
				return visible;
			},
			index : index
		}
	}

	var ComboBoxClass = function( apiuser, uiGroup, elmWrapper, onUpdate ){
		var elmBox     = Util.getElementsByClassName( elmWrapper, 'combobox' )[ 0 ],
			elmA       = ELM_COMBOBOX.cloneNode( true ),
			elmToggle  = Util.getElementsByClassName( elmA, 'combobox-toggle' )[ 0 ],
			elmValue   = Util.getElementsByClassName( elmA, 'combobox-value' )[ 0 ].firstChild,
			index      = 0,
			optionList = [],
			instance   = this,
			focus      = false,
			visible    = true,
			enabled    = true,
			value;
		elmBox.appendChild( elmA );

		this.elm = elmBox;
		this.focus = function( e ){
			MouseEvent.remove( apiuser, elmWrapper, 'click', instance.focus );
			focus = true;
			elmA.className = 'combobox-has-focus';
			start( apiuser, uiGroup, instance );
			OptionControl.show( apiuser, instance, optionList );
			return false;
		};
		this.blur = function( keyCode ){
			OptionControl.hide( instance );
			focus = false;
			elmA.className = '';
			finish( apiuser, uiGroup, instance );
			MouseEvent.add( apiuser, elmWrapper, 'click', instance.focus );
		};
		this.enabled = function(){
			return enabled;
		};
		this.visible = function( _visible ){
			if( Type.isBoolean( _visible ) === true ){
				elmWrapper.style.display = _visible ? '' : 'none';
				visible = _visible;
			};
			return visible;
		};
		this.value = function( _value ){
			if( Type.isString( _value ) === true && value !== _value ){
				for( var i=0, l = optionList.length, _option; i<l; ++i ){
					_option = optionList[ i ];
					if( _value === _option.value ){
						value = _value;
						index = i;
						elmValue.data = _option.displayValue;
						if( focus === true ){
							OptionControl.update( instance, _value );
						};
						Type.isFunction( onUpdate ) === true && AsyncCall.add( apiuser, onUpdate, [ _value ] );
						break;
					};
				};
			};
			return value;
		};
		this.selectIndex = function(){
			return index;
		};
		this.createOption = function( _displayValue, _value, _isSelected ){
			var option = null,
				_option, i;
			_value      = _value || _displayValue;
			_isSelected = !!_isSelected;
			for( i = optionList.length; i; ){
				_option = optionList[ --i ];
				if( _value === _option.value ){
					option = _option;
					break;
				};
			};
			if( _isSelected === true ){
				index = optionList.length;
				elmValue.data = _displayValue;
			};			
			if( option === null ){
				option = new OptionClass( _displayValue, _value, _isSelected );
				optionList.push( option );
			};
		};
		this.destroy = function(){
			instance.blur();
			MouseEvent.remove( apiuser, elmWrapper );
			optionList.length = 0;
			apiuser = uiGroup = elmWrapper = onUpdate = elmBox = elmA = elmToggle = elmValue = optionList = instance = null;
		};
		MouseEvent.add( apiuser, elmWrapper, 'click', instance.focus );
	};
	var OptionClass = function( displayValue, value, isCurrent ){
		this.displayValue = displayValue;
		this.value        = value || displayValue;
		this.current      = isCurrent;
		displayValue = value = null;
	};

	var OptionControl = ( function(){
		var ELM_OPTION_WRAPPER = ( function(){
				var ret = document.createElement( 'div' );
				ret.className = 'option-container';
				return ret;
			})(),
			ELM_OPTION_ORIGIN = ( function(){
				var ret = document.createElement( 'a' );
				ret.appendChild( document.createTextNode( 'option' ) );
				ret.href = '#';
				return ret;
			})();

		var OptionClass = function( option ){
			this.elm     = ELM_OPTION_ORIGIN.cloneNode( true );
			this.data    = option;
			this.init();
		};
		OptionClass.prototype = {
			init: function(){
				ELM_OPTION_WRAPPER.appendChild( this.elm );
				this.elm.firstChild.data = this.data.displayValue;
				this.current( this.data.current );
				MouseEvent.add( SUPER_USER_KEY, this.elm, 'mousedown', onOptionSelect );// onclick では 選択ボックス 隠すように body に設定した onmouseup が先に動いてしまう！
			},
			current: function( _current ){
				this.elm.className = _current === true ? CLASSNAME_COMBOBOX_OPTION_CURRENT : CLASSNAME_COMBOBOX_OPTION;
				this.data.current  = _current;
				currentOption      = _current === true ? this : currentOption;
			},
			destroy: function(){
				MouseEvent.remove( SUPER_USER_KEY, this.elm );
				Util.removeAllChildren( this.elm );
				ELM_OPTION_WRAPPER.removeChild( this.elm );
				delete this.elm;
				delete this.data;
			},
		};
		
		function onOptionSelect( e ){
			for( var i = 0, l = OPTION_LIST.length, _option; i < l; ++i){
				_option = OPTION_LIST[ i ];
				if( this === _option.elm ){
					updateCurrrentOption(  _option.data.value, true );
					currentCombobox.blur();
					break;
				};
			};
			return false;			
		};
		
		var OPTION_LIST     = [],
			currentCombobox = null,
			apiuser,
			elm,
			currentOption,
			currentIndex;
		
		function updateCurrrentOption( _value, _updateCombobox ){
			var _option;
			for( var i = OPTION_LIST.length; i; ){
				_option = OPTION_LIST[ --i ];
				if( _value === _option.data.value ){
					currentOption && currentOption.current( false );
					_option.current( true );
					currentOption = _option;
					currentIndex  = i;
					_updateCombobox === true && currentCombobox.value( _value );
					
					break;
				};
			};
		};
		function bodyMouseupHandler(){
			currentCombobox.blur();
			OptionControl.hide( currentCombobox );
		};
		function updateWrapperPosition(){
			var position = Util.getAbsolutePosition( elm );

			ELM_OPTION_WRAPPER.style.cssText = [
					'width:', elm.offsetWidth - 2, 'px;',
					'left:', position.x, 'px;',
					'top:', position.y + elm.offsetHeight, 'px;'
				].join( '' );
		};
		function change( e ){
			var l   = OPTION_LIST.length,
				i   = currentIndex + ( e.keyCode === 40 ? 1 : -1 );
			if( currentCombobox === null || l < 2 ) return;
			i = i < 0 ?
					l - 1 :
					i < l ? i : 0;
			updateCurrrentOption( OPTION_LIST[ i ].data.value, true );
			return false;
		};
		return {
			show: function( _apiuser, _combobox, _optionList ){
				if( currentItem !== _combobox || currentCombobox === _combobox ) return;
				currentCombobox && currentCombobox.blur();
				
				apiuser         = _apiuser;
				currentCombobox = _combobox;
				elm             = _combobox.elm;
				
				for( var i = 0, l = _optionList.length; i<l; ++i ){
					OPTION_LIST.unshift( new OptionClass( _optionList[ i ] ) );
				};
				MouseEvent.add( SUPER_USER_KEY, document, 'mouseup', bodyMouseupHandler );
				KeyEvent.add( SUPER_USER_KEY, Const.KEY.EVENT.KEY_DOWN, change, 38 );
				KeyEvent.add( SUPER_USER_KEY, Const.KEY.EVENT.KEY_DOWN, change, 40 );
				//KeyEvent.add( SUPER_USER_KEY, Const.KEY.EVENT.KEY_DOWN, onEnter, 13 );
				//KeyEvent.updateCurrentListener( SUPER_USER_KEY );
				
				body.appendChild( ELM_OPTION_WRAPPER );
				
				updateCurrrentOption( _combobox.value(), false );
				updateWrapperPosition();
			},
			hide: function( _combobox ){
				if( currentCombobox !== _combobox || currentCombobox === null ) return;

				var _option;
				while( _option = OPTION_LIST.shift() ){
					_option.destroy();
				};
				
				body.removeChild( ELM_OPTION_WRAPPER );
				
				MouseEvent.remove( SUPER_USER_KEY, doc, 'mouseup', bodyMouseupHandler );
				KeyEvent.remove( SUPER_USER_KEY, Const.KEY.EVENT.KEY_DOWN, change );
				KeyEvent.remove( SUPER_USER_KEY, Const.KEY.EVENT.KEY_DOWN, change );
				//KeyEvent.remove( SUPER_USER_KEY, Const.KEY.EVENT.KEY_DOWN, onEnter );
				//KeyEvent.updateCurrentListener( apiuser );
				
				apiuser         = null;
				currentCombobox = null;
				currentOption   = null;
				currentIndex    = 0;				
			},
			onEnter: function(){
				currentCombobox.value( currentOption.data.value );
				//currentCombobox.blur();
				//OptionControl.hide( currentCombobox );
			},
			update: function( _combobox, _value ){
				if( currentCombobox !== _combobox || currentItem !== _combobox ) return;
				if( currentOption.data.value === _value ) return;
				updateCurrrentOption( _value, true );
			},
			onWindowResize: function( _w, _h ){
				currentCombobox && AsyncCall.add( apiuser, updateWrapperPosition );
			}
		}
	})();

	var UIGroupClass = function( apiuser ){
		var self        = this,
			itemList    = [],
			currentItem = null,
			windowW, windowH;

		this.focus = function( _value ){
			if( _value === true ){
				if( currentItem ){
					start( apiuser, self, currentItem );
				} else
				if( itemList.length > 0 ){
					start( apiuser, self, itemList[ 0 ] );
				};
			} else
			if( _value === false ){
				finish( apiuser, self, currentItem );
			} else
			if( _value && Util.getIndex( itemList, _value ) !== -1 ){
				currentItem = _value;
				currentList = itemList;
			};
			return currentUi === self; 
		};
		this.blur = function(){
			if( currentList === itemList ){
				currentList = null;
			};
		};
		this.createInputText = function( _elmWrapper, _onUpdate, _validater ){
			var _elmValue = Util.getElementsByClassName( _elmWrapper, 'editable-value' )[ 0 ];
			if( _elmValue ){
				var ret = new InputTextClass( apiuser, self, _elmWrapper, _elmValue, _onUpdate, _validater );
				itemList.push( ret );
				return ret;
			}
			alert( 'error createInputText' );
		};
		this.createButton = function( _elm, _onClick ){
			var ret = new ButtonClass( apiuser, self, _elm, _onClick );
			itemList.push( ret );
			return ret;
		};
		this.createFileInput = function( _elm, _onUpdate, _validater, _elmFileInput ){
			var ret = FileInputClass( apiuser, self, _elm, _onUpdate, _validater, _elmFileInput );
			itemList.push( ret );
			return ret;
		};
		this.createCombobox = function( _elm, _onUpdate, _optionList ){
			var ret = new ComboBoxClass( apiuser, self, _elm, _onUpdate, _optionList );
			itemList.push( ret );
			return ret;
		};
		this.createCheckBox = function(){
			
		};
		this.createRadio = function(){
			
		};
		this.createSlider = function(){
			
		};
		this.destroy = function(){
			var _item;
			while( _item = itemList.shift() ){
				_item.destroy();
				if( currentItem === _item ){
					currentItem = null;
				};
			};
		};
	};
	
	function start( _apiuser, _uigroup, _item ){
		if( currentItem !== _item ){
			currentItem !== null && currentItem.blur();
			
			// if( currentUser !== _apiuser ) {
				KeyEvent.add( SUPER_USER_KEY, Const.KEY.EVENT.KEY_DOWN, onKeyDown, 13 );
				KeyEvent.add( SUPER_USER_KEY, Const.KEY.EVENT.KEY_DOWN, onKeyDown, 27 );
				KeyEvent.add( SUPER_USER_KEY, Const.KEY.EVENT.KEY_DOWN, onKeyDown,  9 );
				KeyEvent.updateCurrentListener( SUPER_USER_KEY );
			// };
			if( currentUi !== _uigroup ){
				currentUi && currentUi.blur();
			};
			currentUser = _apiuser;
			currentUi   = _uigroup;
			currentItem = _item;
			
			_uigroup.focus( _item );
		};
	}
	function finish( _apiuser, _uigroup, _item ){
		if( currentItem === _item ){
			_uigroup.blur();
			
			currentUser = null;
			currentUi   = null;
			currentItem = null;
			
			KeyEvent.remove( SUPER_USER_KEY, Const.KEY.EVENT.KEY_DOWN, onKeyDown, 13 );
			KeyEvent.remove( SUPER_USER_KEY, Const.KEY.EVENT.KEY_DOWN, onKeyDown, 27 );
			KeyEvent.remove( SUPER_USER_KEY, Const.KEY.EVENT.KEY_DOWN, onKeyDown,  9 );
			KeyEvent.updateCurrentListener( _apiuser );
		};
	};

	function onKeyDown( e ){
		if( currentItem === null ) return true;
		var keyCode = e.keyCode,
			_index  = Util.getIndex( currentList, currentItem );
		if( keyCode === 13 || keyCode === 27 || keyCode === 9 || keyCode === 18 || e.altKey === true ){ // 13.return 27.esc 9.tab 18.alt
			keyCode === 9  && tabShift( _index, e.shiftKey === true ? -1 : 1 );
			keyCode === 13 && currentItem instanceof ComboBoxClass && OptionControl.onEnter();
			keyCode === 13 && tabShift( _index, 1 );			
			currentItem && currentItem.blur( keyCode );
		};
		return false;
	};

	function tabShift( _index, _way ){
		var l = currentList.length,
			i = _index + _way,
			_item;
		if( l < 2 ) return;
		while( i !== _index ){
			i = i < 0 ?
				l - 1 :
				i < l ? i : 0; // 0 < i < l
			_item = currentList[ i ];
			if( _item.enabled() === true && _item.visible() === true ){
				AsyncCall.add( currentUser, _item.focus );
				return;
			};
			i += _way;
		};
	};
	
	return {
		createUIGroup: function( _apiuser ){
			var _uid  = _apiuser.getUID(),
				_list = UI_LIST[ _uid ],
				_ui   = new UIGroupClass( _apiuser );
			if( Type.isArray( _list ) === false ){
				_list = UI_LIST[ _uid ] = [];
			}
			_list.push( _ui );
			return _ui;
		},
		onWindowResize: function( w, h ){
			windowW = w;
			windowH = h;
			currentItem instanceof ComboBoxClass && OptionControl.onWindowResize( w, h );
		},
		onCurrentApplicationChange: function( _apiuser ){
			currentList = UI_LIST[ _apiuser.getUID() ];
		},
		onApplicationShutdown: function( _apiuser ){
			KeyEvent.remove( _apiuser );
		},
		onSystemShutdown: function(){
			
		}
	}
})();


var Finder = ( function(){
	var HTML_FINDER_ICON = ( function(){
		return ( UA.isIE === true && UA.ieVersion < 8 ?
		[
			'<div class="finder-icon fnder-icon-ie7">',
				'<a href="#" class="finder-icon-main">',
					'<span class="finder-icon-handle"></span>',
					'<span class="finder-icon-thumbnail"></span>',
					'<span class="finder-icon-cell finder-icon-ie-filename">',
						'<span class="finder-icon-vertical-middle-outer">',
							'<span class="finder-icon-vertical-middle-inner">',
								'<span class="finder-icon-filename break-word">file name</span>',
							'</span>',
						'</span>',
					'</span>',
					'<span class="finder-icon-cell finder-icon-ie-summary">',
						'<span class="finder-icon-vertical-middle-outer">',
							'<span class="finder-icon-vertical-middle-inner">',
								'<span class="finder-icon-summary break-word">file descriptiion</span>',
							'</span>',
						'</span>',
					'</span>',
				'</a>',
				'<div class="finder-icon-console">',
					'<a href="#" class="finder-icon-console-action"></a>',
					'<a href="#" class="finder-icon-console-editor-apps"></a>',
					'<a href="#" class="finder-icon-console-viewer-apps"></a>',
				'</div>',
			'</div>'
		] :
		[
			'<div class="finder-icon fnder-icon-modern">',
				'<span class="finder-icon-handle"></span>',
				'<span class="finder-icon-thumbnail"></span>',
				'<span class="finder-icon-filename break-word">file name</span>',
				'<span class="finder-icon-summary break-word">file descriptiion</span>',
				'<div class="finder-icon-console">',
					'<a href="#" class="finder-icon-console-action"></a>',
					'<a href="#" class="finder-icon-console-editor-apps"></a>',
					'<a href="#" class="finder-icon-console-viewer-apps"></a>',
				'</div>',
			'</div>',
		] ).join( '' );
	})();

	
	var FINDER_ARRAY                    = [],
		ELM_ORIGIN_FINDER_LOCATION_ITEM = Util.pullHtmlAsTemplete( '<li id="templete-finder-location-item" class="finder-location-item"><a href="#"></a></li>'),
		ELM_ORIGIN_FINDER_ICON          = Util.pullHtmlAsTemplete( HTML_FINDER_ICON ),
		ELM_ORIGIN_CONTAINER            = Util.pullHtmlAsTemplete( [
			'<div id="templete-finder-container" class="finder-container">',
				'<div class="finder-header">',
					'<ul class="finder-location"></ul>',
					'<div class="finder-sidebar-switch button">side</div>',
					'<div class="finder-style-switch button">style</div>',
					'<div class="finder-action-switch button">action</div>',
				'</div>',
				'<div class="finder-body"></div>',
			'</div>',
		].join( '' ) ),
		ICON_HEIGHT                     = Util.getElementSize( ELM_ORIGIN_FINDER_ICON ).height,
		ICON_CLASSNAME                  = 'finder-icon-thumbnail',
		FINDER_ICON_POOL                = [],
		BREAD_OBJECT_POOL               = [];
	
	var FinderIconClass = function(){
		var elmContainer,
			ELM_WRAPPER       = ELM_ORIGIN_FINDER_ICON.cloneNode( true),
			ELM_THUMBNAIL     = Util.getElementsByClassName( ELM_WRAPPER, ICON_CLASSNAME )[ 0 ],
			ELM_FILENAME      = Util.getElementsByClassName( ELM_WRAPPER, 'finder-icon-filename' )[ 0 ],
			ELM_DESCRIPTION   = Util.getElementsByClassName( ELM_WRAPPER, 'finder-icon-summary' )[ 0 ],
			ELM_EDITOR_BUTTON = Util.getElementsByClassName( ELM_WRAPPER, 'finder-icon-console-editor-apps' )[ 0 ],
			ELM_VIEWER_BUTTON = Util.getElementsByClassName( ELM_WRAPPER, 'finder-icon-console-viewer-apps' )[ 0 ],
			ELM_ACTION_BUTTON = Util.getElementsByClassName( ELM_WRAPPER, 'finder-icon-console-action' )[ 0 ],
			instansce         = this,
			file, w, index, style,
			onDownCallback, onEditorCallback, onViewerCallback, onActionCallback,
			viewerList, editorList;
		
		MouseEvent.add( SUPER_USER_KEY, ELM_WRAPPER,       'click', onDownClick );
		MouseEvent.add( SUPER_USER_KEY, ELM_EDITOR_BUTTON, 'click', onEditorClick );
		MouseEvent.add( SUPER_USER_KEY, ELM_VIEWER_BUTTON, 'click', onViwerClick );
		MouseEvent.add( SUPER_USER_KEY, ELM_ACTION_BUTTON, 'click', onActionClick );
		function onDownClick(){
			onDownCallback( index );
			return false;
		}
		function onEditorClick(){
			onEditorCallback( file, editorList[ 0 ] );
			return false;
		}
		function onViwerClick(){
			onViewerCallback( file, viewerList[ 0 ] );
			return false;
		}
		function onActionClick(){
			onActionCallback( file );
			return false;
		}
		function draw(){
			var _thumb = file.getThumbnail();
			if( _thumb.image ){
				ELM_THUMBNAIL.className = ICON_CLASSNAME + ' has-thumbnail';
				ELM_THUMBNAIL.style.backgroundImage = [ 'url(', _thumb.image, ')'].join( '');
			} else {
				ELM_THUMBNAIL.className = ICON_CLASSNAME + ' ' + _thumb.className;
				ELM_THUMBNAIL.style.backgroundImage = '';
			}
			ELM_FILENAME.firstChild.data = file.getName();
			ELM_DESCRIPTION.firstChild.data = file.getSummary();
			
			if( Type.isArray( viewerList ) === true && viewerList.length > 0 ){
				ELM_VIEWER_BUTTON.style.display = '';
			} else {
				ELM_VIEWER_BUTTON.style.display = 'none';
			};
			if( Type.isArray( editorList ) === true && editorList.length > 0 ){
				ELM_EDITOR_BUTTON.style.display = '';
			} else {
				ELM_EDITOR_BUTTON.style.display = 'none';
			};
		}
		function resize(){
			// ELM_WRAPPER.style.top = (index * ICON_HEIGHT) +'px';
		}
		function onCollect(){
			elmContainer.removeChild( ELM_WRAPPER );
			elmContainer = null;
			FINDER_ICON_POOL.push( instansce );
		}
		
		this.init = function( _file, _elmContainer, _w, _index, _style, _onDownCallback, _onEditorCallback, _onViewerCallback, _onActionCallback ){
			if( elmContainer !== _elmContainer){
				_elmContainer.appendChild( ELM_WRAPPER );
				elmContainer = _elmContainer;
			}
			if( file !== _file ){
				file && file.destroy();
				file       = _file;
				viewerList = file.viewerApplicationList();
				editorList = file.editorApplicationList();
				draw();
			}
			if( index !== _index){
				index = _index;
				resize();
			}
			onDownCallback   = _onDownCallback;
			onEditorCallback = _onEditorCallback;
			onViewerCallback = _onViewerCallback;
			onActionCallback = _onActionCallback;
		};
		this.elm = ELM_WRAPPER,
		this.index = function( _index){	
			return index;
		};
		this.style = function( _style ){
			return style;
		};
		this.onResize = function( w ){
			
		};
		this.destroy = function(){
			//MouseEvent.remove( SUPER_USER_KEY, ELM_WRAPPER,       'click', onDownClick );
			//MouseEvent.remove( SUPER_USER_KEY, ELM_EDITOR_BUTTON, 'click', onEditorClick );
			//MouseEvent.remove( SUPER_USER_KEY, ELM_VIEWER_BUTTON, 'click', onViwerClick );
			//MouseEvent.remove( SUPER_USER_KEY, ELM_ACTION_BUTTON, 'click', onActionClick );
			elmContainer.removeChild( ELM_WRAPPER );
			file && file.destroy();
			file = elmContainer = onDownCallback = onEditorCallback = onViewerCallback = onActionCallback = viewerList = editorList = null;
			FINDER_ICON_POOL.push( instansce );
		};
	}
	function updateIconPosition( _style, _w, _index, _elm ){
		
	}
	var BreadcrumbClass = function(){
		var elmContainer,
			ELM_WRAPPER  = ELM_ORIGIN_FINDER_LOCATION_ITEM.cloneNode( true ),
			ELM_FILENAME = ELM_WRAPPER.getElementsByTagName( 'a' )[ 0 ],
			instansce    = this,
			file, w, index,
			callback;
		MouseEvent.add( SUPER_USER_KEY, ELM_WRAPPER, 'click', onClick );
		function draw(){
			ELM_FILENAME.className = 'file-icon-' + file.getType();
			ELM_FILENAME.innerHTML = file.getName();
		}
		function resize( index ){
			ELM_WRAPPER.style.left = ( index * 90 ) + 'px';
		}
		function onClick(){
			callback( index );
			return false;
		}
		this.init = function( _file, _elmContainer, _index, _callback ){
			instansce;
			if( elmContainer !== _elmContainer ){
				_elmContainer.appendChild( ELM_WRAPPER );
				elmContainer = _elmContainer;
			}
			if( file !== _file){
				file = _file;
				draw();
			}
			if( index !== _index){
				index = _index;
				resize( index );
			}
			callback = _callback;
		};
		this.elm = ELM_WRAPPER;
		this.index = function( _index ){
			return index;
		};
		this.onResize = function( w ){
			
		};
		this.destroy = function(){
			elmContainer.removeChild( ELM_WRAPPER );
			file = elmContainer = null;
			BREAD_OBJECT_POOL.push( this );
		};
	}
	
	var FinderClass = function( application, elmRoot, tree, header, footer, onSelect, viewerOption, editorOption ){
		var ICON_ARRAY       = [],
			BREAD_ARRAY      = [],
			elmContainer     = ELM_ORIGIN_CONTAINER.cloneNode( true ),
			elmLocation      = elmContainer.getElementsByTagName( 'ul' )[ 0 ],
			nodesDiv         = elmContainer.getElementsByTagName( 'div' ),
			elmSidebarButton = nodesDiv[ 1 ],
			elmStyleButton   = nodesDiv[ 2 ],
			elmActionButton  = nodesDiv[ 3 ],
			elmBody          = nodesDiv[ nodesDiv.length -1 ],
			headX,
			headY,
			headH            = Util.getElementSize( nodesDiv[ 0 ] ).height,
			bodyY,
			currentFile      = null,
			breadW           = 90,
			size             = Util.getElementSize( ELM_ORIGIN_FINDER_ICON ),
			iconW            = size.width,
			iconH            = size.height,
			style            = 0,
			w, h, bodyH,
			instance         = this;
			
		tree.addTreeEventListener( Const.TREE.EVENT.UPDATE, draw );
		elmRoot.appendChild( elmContainer );
		
		function draw( _w, _h ){
			w = Type.isFinite( _w ) === true ? _w : w;
			h = Type.isFinite( _h ) === true ? _h : h;
			bodyH = h - headH;
			var	l = tree.hierarchy() + 1,
				m = BREAD_ARRAY.length,
				_file, _bread;
			for( var i=0; i<l; ++i ){
				_file = i !== l-1 ? tree.getParentFileAt( i ) : tree.getCurrentFile();
				if( i < m ){
					BREAD_ARRAY[ i ].init( _file, elmLocation, i, onHeadClick );
				} else {
					BREAD_ARRAY.push( getBreadcrumb( _file, elmLocation, i, onHeadClick ));
				}
			};
			while( l < BREAD_ARRAY.length ){
				BREAD_ARRAY.pop().destroy();
			};
			
			l = _file.getChildFileLength();
			m = ICON_ARRAY.length;

			for( i=0; i<l; ++i ){
				if( i < m ){
					ICON_ARRAY[ i ].init( _file.getChildFileByIndex( i ), elmBody, w, i, style, onDown, onEditor, onViwer, onAction );
				} else {
					ICON_ARRAY.push( getFinderIcon( _file.getChildFileByIndex( i ), elmBody, _w, i, style, onDown, onEditor, onViwer, onAction ));
				}
			};
			if( _file.getState() === Const.FILE.STATE.LOADING ){
				elmBody.className = 'finder-body loading';
			} else {
				elmBody.className = 'finder-body';
			};
			
			elmBody.style.height = bodyH + 'px';
			
			while( l < ICON_ARRAY.length && ICON_ARRAY.length > 0 ){
				ICON_ARRAY.pop().destroy();
			};
		};
		
		function onHeadClick( i ){
			var l = BREAD_ARRAY.length - 1;
			if( i < l){
				var _file = tree.getParentFileAt( i);
				if( _file !== null){
					tree.up( i);
					draw( w, h );
				};
			};
		};
		function onDown( i ){
			if( i < ICON_ARRAY.length ){
				var _file = tree.getCurrentFile().getChildFileByIndex( i );
				if( _file !== null && ( _file.getChildFileLength() !== -1 || _file.getType() === Const.FILE.TYPE.FOLDER )){
					tree.down( i );
					draw( w, h );
				} else {
					Type.isFunction( onSelect ) === true && onSelect( _file );
				};
				_file.destroy();
			};
		};
		function onEditor( _file, _app, editorOption ){
			_app.boot( _file, editorOption );
		};
		function onViwer( _file, _app ){
			_app.boot( _file, viewerOption );
		};
		function onAction( _file ){

		};
		this.MIN_WIDTH     = 240;
		this.MIN_HEIGHT    = 240;
		this.onInit = function(){
			var position = Util.getAbsolutePosition( elmLocation );
			headX = position.x;
			headY = position.y;
			bodyY = Util.getAbsolutePosition( elmBody ).y;
		};
		this.onPaneResize = function( _w, _h ){
			draw( _w, _h );			
			
			w = _w;
			h = _h;
			elmBody.style.height = ( _h - headH ) + 'px';
			
			for( var i = ICON_ARRAY.length; i; ){
				ICON_ARRAY[ --i ].onResize( _w );
			};
		};
		this.destroy = function(){
			tree.removeTreeEventListener( Const.TREE.EVENT.UPDATE, draw );
			
			while( BREAD_ARRAY.length > 0 ){
				BREAD_ARRAY.shift().destroy();
			};
			while( ICON_ARRAY.length > 0 ){
				ICON_ARRAY.shift().destroy();
			};
			
			FINDER_ARRAY.splice( Util.getIndex( FINDER_ARRAY, instance ), 1 );
		};
	};
	FinderClass.prototype = new AbstractBasicPane();
	
	function getFinderIcon( _file, _elmContainer, w, index, style, onDown, onEditor, onViwer, onAction ){
		var _icon;
		if( FINDER_ICON_POOL.length > 0){
			_icon = FINDER_ICON_POOL.shift();
		} else {
			_icon = new FinderIconClass();
		};
		_icon.init( _file, _elmContainer, w, index, style, onDown, onEditor, onViwer, onAction );
		return _icon;
	};
	
	function getBreadcrumb( _file, _elmContainer, index, callback ){
		var _bread;
		if( BREAD_OBJECT_POOL.length > 0 ){
			_bread = BREAD_OBJECT_POOL.shift();
		} else {
			_bread = new BreadcrumbClass();
		};
		_bread.init( _file, _elmContainer, index, callback );
		return _bread;
	};

	return {
		init: function(){
			
		},
		create: function( _application, _elmTarget, _tree, _header, _footer, _onSelect, _viewerOption, _editorOption ){
			//if( Application.isApplicationInstance( _application ) === false ) return;
			
			var _finder = new FinderClass( _application, _elmTarget, _tree, _header, _footer, _onSelect, _viewerOption, _editorOption );
			_finder.init();
			FINDER_ARRAY.push( _finder );
			return _finder;
		},
		registerFinderHead: function(){
			
		},
		registerFinderPane: function( _finderPane ){
			
		},
		isFinderInstance: function( _finder ){
			return _finder instanceof FinderClass;
		},
		isFinderPaneInstance: function(){
			
		},
		isFinderHeadInstance: function(){
		}
	};
})();


/*
 * -- len, %
 * marginBottom, marginLeft, marginRight, marginTop, margin
 * padding, paddingBottom, paddingLeft, paddingRight, paddingTop
 * fontSize, textIndent
 * height, width
 * bottom, left, right, top			(len, %)
 *
 * -- len
 * borderBottomWidth, borderLeftWidth, borderRightWidth, borderTopWidth, borderWidth,
 * letterSpacing
 *
 * -- color
 * backgroundColor
 * borderBottomColor, borderLeftColor, borderRightColor, borderTopColor, borderColor
 * color
 *
 * -- special
 * clip			rect(0px, 40px, 40px, 0px);
 * backgroundPosition	(len, %)
 * opacity
 * lineHeight		(len, %, num)
 * zIndex			( order )
 */

var VisualEffect = ( function(){
	
	var ANIMATION_TICKET_ARRAY = [],
		fpms                   = 50,
		FILTER                 = 'filter',
		OPACITY                = 'opacity',
		REG_ALPHA              = /alpha\([^)]*\)/i,
		CORON                  = ':',
		EMPTY                  = '',
		SEMICORON              = ';';
		UNITS                  = 'px,cm,mm,in,pt,pc,em,%'.split( ',' );
		
		PARAMS = ( function(){
			ret = { size: {}, color: {}, special: {}, unit: {}, region: {} };
			register( ret.size, 'marginBottom,marginLeft,marginRight,marginTop,margin,padding,paddingBottom,paddingLeft,paddingRight,paddingTop,borderBottomWidth,borderLeftWidth,borderRightWidth,borderTopWidth,borderWidth,fontSize,textIndent,height,width,bottom,left,right,top' );
			register( ret.color, 'backgroundColor,borderBottomColor,borderLeftColor,borderRightColor,borderTopColor,borderColor,color' );
			register( ret.special, 'clip,backgroundPosition,opacity,lineHeight,zIndex,letterSpacing' );
			
			register( ret.unit, 'px,cm,mm,in,pt,pc,em,%' );
			register( ret.region, 'margin,padding,border,clip,backgroundPosition,borderColor' )
			
			function register( obj, params ){
				params = params.split( ',' );
				for( var i=params.length; i; ) obj[ params[ --i ] ] = true;
			};
			return ret;
		})();
		
	function startAnimation( _elm, _cssObject, _onComplete, _onEnterFrame, _numFrames ){
		var _ticket, i = ANIMATION_TICKET_ARRAY.length;
		for( ; i; ){
			_ticket = ANIMATION_TICKET_ARRAY[ --i ];
			if( _ticket.elm === _elm ){
				_ticket.destroy();
				break;
			};
		};
		
		var _currentParameters = [],
			_offsetParameters  = [],
			_endParameters     = [],
			_targetAttributes  = [],
			_units             = [];
		
		//  coputedStyle と _cssObject を比較して , 一致及び非数は削除しつつ コピー
		//  filter opacity の追加 REG_ALPHA.test( _value ) ? _value.replace( REG_ALPHA, opacity ) : _value + " " + opacity
		/*
		 * cssObject をまわす
		 * 非数は削除
		 * computtedStyle と一致も削除
		 * updateTraget として記録
		 * 初期値を cssObject としてセット
		 * cssTest にセット zoom もセット
		 */
		var x, v, _v, u,
			empty          = EMPTY,
			px             = 'px',
			$1             = '$1',
			isString       = Type.isString,
			isNumber       = Type.isNumber,
			inlineStyle    = Util.getInlineStyle( _elm ),
			computtedStyle = Util.getComputedStyle( _elm ),
			REG_UINIT      = /.*\d(\w{1,2})?/;
		for( var p in _cssObject ){
			x = _cssObject[ p ];
			if( PARAMS.size[ p ] === true ){
				/*
				 * size
				 */
				if( isString( x ) === true ){
					v = parseInt( x );
					if( ( empty + v ).length < x.length ){
						u = x.replace( REG_UINIT, $1 );
						if( PARAMS.unit[ u ] === false ) u = px;
					} else {
						u = px;
					};
				} else
				if( isNumber( x ) === true ){
					v = x;
					_unit = px; // 'px'
				} else {
					continue;
				};
			} else
			if( PARAMS.color[ p ] === true ){
				/*
				 * color
				 */
				if( isString( x ) === true ){
					/* #rrggbb
					 * "aqua, black, blue, fuchsia, gray, green, lime, maroon, navy, olive, orange, purple, red, silver, teal, white, yellow"
					 * #rgb
					 * rgb(r%, g%, b%)
					 * rgb(r, g, b)
					 */
				} else
				if( isNumber( x ) === true ){
					v = x;
					_unit = px; // 'px'
				} else {
					continue;
				};
			} else
			if( PARAMS.special[ p ] === true ){
				
			} else {
				continue;
			}
			
			/*
			 * ここまでで、新しい値 v と 単位 u が判明．
			 */
			_v = computtedStyle[ p ];
			/**
			 * _v を u を合わせて変換して v と比較．
			 * em の場合は comS.fontSize を元に
			 */
			if( v === _v ) continue;
			_currentParameters.push( _v );
			_offsetParameters.push( _v - v );
			_endParameters.push( v );
			_targetAttributes.push( p );
			_units.push( _unit );
			inlineStyle[ p ] = _v;
		};
		elm.style.cssText = Util.toCssText( inlineStyle );
		//
		/*
			var style = elem.style,
				currentStyle = elem.currentStyle,
				opacity = jQuery.isNumeric( value ) ? "alpha(opacity=" + value * 100 + ")" : "",
				filter = currentStyle && currentStyle.filter || style.filter || "";

			// IE has trouble with opacity if it does not have layout
			// Force it by setting the zoom level
			style.zoom = 1;
		 */
		
		ANIMATION_TICKET_ARRAY.push( new AnimationTaskClass(
			_elm,
			_currentParameters, _offsetParameters, _endParameters, _targetAttributes, _units,
			Type.isFunction( _onComplete ) === true   ? _onComplete   : null,
			Type.isFunction( _onEnterFrame ) === true ? _onEnterFrame : null,
			_numFrames
		) );
		
		SystemTimer.add( SUPER_USER_KEY, onEnterFrame, 1000 / fpms );
	};
	
	function onEnterFrame(){
		var _ticket,
			i = 0;
		while( i < ANIMATION_TICKET_ARRAY.length ){
			_ticket = ANIMATION_TICKET_ARRAY[ i ];
			if( _ticket.onEnterFrame() === true ){
				_ticket.destroy();
				ANIMATION_TICKET_ARRAY.splice( i, 1 );
			} else {
				++i;
			};
		};
		if( ANIMATION_TICKET_ARRAY.length === 0 ){
			SystemTimer.remove( SUPER_USER_KEY, onEnterFrame );
		};
	};
	function updateCss( _elm, _updateParameters, _targetAttributes, _numAttributes ){
		if( _numAttributes > 0 ){
			var _cssTexts = _elm.style.cssText.split( ';' ), _css,
				_separate, _target, _value, _number, _unit,
				i = _cssTexts.length, j;
			for( ; i; ){
				_css      = _cssTexts[ --i ];
				_separate = _css.indexOf( ':' );
				if( _separate >= 3 ){
					_target = _css.substr( 0, _separate + 1 );
					_value  = _css.substr( _separate + 1 );
					/*
					 * ie filter
					 */
					if( _target === FILTER ){
						for( j = _numAttributes; j; ){
							if( _targetAttributes[ --j ] === OPACITY ){
								_cssTexts[ i ] = [
									_target, ':',
									_value.replace( REG_ALPHA, 'alpha(opacity=' + _updateParameters[ j ] * 100 + ')' )
								].join( '');
								break;
							};
						};
					/*
					 * other
					 */
					} else {
						_number = '' + parseFloat( _value );
						_unit   = _value.indexOf( _number );
						_unit   = _unit > 0 ? _value.substr( _unit + _number.length ) : '';
						for( j = _numAttributes; j; ){
							if( _targetAttributes[ --j ] === _target ){
								_cssTexts[ i ] = [ _target, ':', _updateParameters[ j ], _unit ].join( '' );
								break;
							};
						};
					};
				};
			};
			_elm.style.cssText = _cssTexts.join( ';' );
			return;	
		};
		/*
		 * 一度に update する Attributes が少ない場合、cssText は使用しない.
		 */
	};
	
	var AnimationTaskClass = function( elm, currentParameters, offsetParameters, endParameters, targetAttributes, units, onComplete, onEnterFrame, numFrames ){
		var l = targetAttributes.length;
		this.elm          = elm;
		this.onEnterFrame = function(){
			var _updateCss = {}, i = l;
			if( numFrames === 1 ){
				for( ; i; ){
					--i;
					_updateCss[ targetAttributes[ i ] ] = endParameters[ i ];
				};
				updateCss( elm, currentParameters, targetAttributes, l );
				onComplete !== null && onComplete();
			} else {
				for( ; i; ){
					--i;
					_updateCss[ targetAttributes[ i ] ] = currentParameters[ i ] = Math.floor( currentParameters[ i ] + offsetParameters[ i ] );
				};
				updateCss( elm, currentParameters, targetAttributes, l );
				onEnterFrame !== null && onEnterFrame( _updateCss );
			}
			--numFrames;
			return numFrames === 0;
		};
		this.destroy = function(){
			elm = onComplete = onEnterFrame = null;
		};
	};
	
	var VisualEffectClass = function( elm ){
		var isHtmlElement;
		
		function registerAnime( _cssObject, _onComplete, _onEnterFrame, _time ){
			var _numFrames = Math.floor( _time / fpms);
			startAnimation( elm, _cssObject, _onComplete, _onEnterFrame, _numFrames );
		};
		function startFadeIn(){
			
		};
		function startFadeOut(){
			
		};
		function update( _x, _y, _w, _h){
			var _cssText = elm.style.cssText;
			
		};
		
		this.anime   = registerAnime;
		this.fadeIn  = startFadeIn;
		this.fadeOut = startFadeOut;
		this.update  = update;
	}
	
	return {
		createVisualEffect: function( _elm){
			return new VisualEffectClass( _elm);
		},
		isInstanceOfVisualEffect: function( _instance){
			return _instance instanceof VisualEffectClass;
		}
	}
})();


/* --------------------------------------------
 * 
 */

	Application.onCurrentApplicationChange( SUPER_USER_KEY );
	
	SERVICE_LIST.push( MouseEvent );
	
	new EventTicketClass( window, 'unload', function(){
		var _service;
		while( SERVICE_LIST.length > 0 ){
			_service = SERVICE_LIST.shift();
			Type.isFunction( _service.onSystemShutdown ) === true && _service.onSystemShutdown();
		}
	});
	// beforeunload


/* ---------------------------------------------
 * broadcast to global
 */
	window.gOS = {};
	
	gOS.registerApplication = Application.register;
	gOS.registerDriver      = File.registerDriver;
	
})( window, document );
