#原画
class OriginalPicturesController < ApplicationController
  layout 'test' if Pettanr::TestLayout
  before_filter :authenticate_user!, :only => [:index, :show, :history, :new, :edit, :create, :update, :destroy]
  before_filter :authenticate_admin!, :only => [:list, :browse]
  before_filter :authenticate_artist, :only => [:index, :show, :history, :new, :edit, :create, :update, :destroy]
  
  private
  
  def set_image(file)
    if file.respond_to?(:read)
      file.read
    else
      Base64.decode64(file.to_s.gsub(' ', '+')) #rubyのバグ？+でデコードされるべきキャラがスペースになる
    end
  end
  
  public
  
  # GET /original_pictures
  # GET /original_pictures.json
  def index
    @page = OriginalPicture.page params[:page]
    @page_size = OriginalPicture.page_size params[:page_size]
    @original_pictures = OriginalPicture.list(@artist.id, {}, @page, @page_size)

    respond_to do |format|
      format.html # index.html.erb
      format.json { render json: @original_pictures.to_json(OriginalPicture.list_json_opt) }
    end
  end

  # GET /original_pictures/1
  # GET /original_pictures/1.json
  def show
    @original_picture = OriginalPicture.show(params[:id], @artist)
    
    respond_to do |format|
      opt = {:type => @original_picture.mime_type, :disposition=>"inline"}
      format.png { send_data(@original_picture.restore, opt ) }
      format.gif { send_data(@original_picture.restore, opt ) }
      format.jpeg { send_data(@original_picture.restore, opt ) }
      format.html {
        #ライセンス付与のために原画ライセンスグループ作成準備
        @original_picture_license_group  = OriginalPictureLicenseGroup.new :original_picture_id => @original_picture.id
        render
      }
      format.json { render json: @original_picture.to_json(OriginalPicture.show_json_include_opt)}
    end
  end

  def history
    @original_picture = OriginalPicture.show(params[:id], @artist)
    @history = @original_picture.history
    
    respond_to do |format|
      format.html # index.html.erb
      format.json { render json: @history.to_json }
    end
  end
  
  def list
    @original_pictures = OriginalPicture.all

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @original_pictures }
    end
  end

  def browse
    @original_picture = OriginalPicture.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @original_picture }
    end
  end

  def refresh
    @original_picture = OriginalPicture.find(params[:id])
    img = Magick::Image.from_blob(@original_picture.restore).shift
    @original_picture.store img
    respond_to do |format|
      format.html { redirect_to original_pictures_url }
    end
  end
  
  # GET /original_pictures/new
  # GET /original_pictures/new.json
  def new
    @original_picture = OriginalPicture.new
    @original_picture.supply_default @artist

    respond_to do |format|
      format.html # new.html.erb
      format.js
      format.json { render json: @original_picture }
    end
  end

  # GET /original_pictures/1/edit
  def edit
    @original_picture = OriginalPicture.edit(params[:id], @author)
    respond_to do |format|
      format.html
      format.js
    end
  end

  # POST /original_pictures
  # POST /original_pictures.json
  def create
    @picture_data = set_image params[:original_picture][:file]
    @original_picture = OriginalPicture.new
    @original_picture.supply_default @artist

    respond_to do |format|
      if @original_picture.store(@picture_data, @artist)
        format.html { redirect_to @original_picture, notice: 'Original picture was successfully created.' }
        format.json { render json: @original_picture, status: :created, location: @original_picture }
      else
        format.html { render action: "new" }
        format.json { render json: @original_picture.errors, status: :unprocessable_entity }
      end
    end
  end

  # PUT /original_pictures/1
  # PUT /original_pictures/1.json
  def update
    @picture_data = set_image params[:original_picture][:file]
    @original_picture = OriginalPicture.edit(params[:id], @author)

    respond_to do |format|
      if @original_picture.store(@picture_data, @artist)
        format.html { redirect_to @original_picture, notice: 'Original picture was successfully created.' }
        format.json { render json: @original_picture, status: :created, location: @original_picture }
      else
        format.html { render action: "edit" }
        format.json { render json: @original_picture.errors, status: :unprocessable_entity }
      end
    end
  end

  # DELETE /original_pictures/1
  # DELETE /original_pictures/1.json
  def destroy
    @original_picture = OriginalPicture.edit(params[:id], @author)
    OriginalPicture.transaction do
      @original_picture.destroy
    end
    
    respond_to do |format|
      format.html { redirect_to original_pictures_url }
      format.json { head :ok }
    end
  end
end
