class PanelColor < ActiveRecord::Base
  belongs_to :panel
  
  validates :panel_id, :numericality => {:allow_blank => true}
  validates :z, :presence => true, :numericality => {:greater_than => 0}
  validates :code, :presence => true, :numericality => {:greater_than_or_equal_to => 0, :less_than => 0x1000000}
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.offset cnt, prm = nil
    offset = prm.to_i
    offset = cnt - 1 if offset >= cnt
    offset = cnt - offset.abs if offset < 0
    offset = 0 if offset < 0
    offset
  end
  
  def self.list opt = {}, page = 1, page_size = self.default_page_size
    opt.merge!(self.list_opt) unless opt[:include]
    opt.merge!({:conditions => 'panels.publish > 0', :order => 'panel_colors.updated_at desc', :limit => page_size, :offset => (page -1) * page_size})
    PanelColor.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {:panel => {:author => {}} }}
  end
  
  def self.list_json_opt
    {:include => {:panel => {:author => {}} }}
  end
  
  def self.mylist au, opt = {}, page = 1, panel_color_page_size = Author.default_panel_color_page_size
    opt.merge!(self.list_opt) unless opt[:include]
    opt.merge!({:conditions => ['panels.author_id = ?', au.id], :order => 'panel_colors.updated_at desc', :limit => page_size, :offset => (page -1) * panel_color_page_size})
    PanelColor.find(:all, opt)
  end
  
end
