# -*- encoding: utf-8 -*-
require 'spec_helper'
#コマ
describe PanelsController do
  before do
    FactoryGirl.create :admin
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create :user_yas
    @author = @user.author    #ユーザ作成時に連動して作成される
  end
  
  describe '一覧表示に於いて' do
    before do
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      Panel.stub(:list).and_return([@panel, @panel, @panel])
      sign_in @user
    end
    context '事前チェックする' do
      it '与えられたpageがセットされている' do
        get :index, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :index
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :index, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :index
        assigns(:page_size).should eq Panel.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 1500
        assigns(:page_size).should eq Panel.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 0
        assigns(:page_size).should eq Panel.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      it 'コマモデルに一覧を問い合わせている' do
        Panel.should_receive(:list).exactly(1)
        get :index
      end
      it '@panelsにリストを取得している' do
        get :index
        assigns(:panels).should have_at_least(3).items
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがリスト構造になっている' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいはコマっぽいものであって欲しい' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("border").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :index
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :index
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :index, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :index, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
  describe '単体表示に於いて' do
    before do
      @panel = FactoryGirl.create :panel, :author_id => @user.author.id
      Panel.stub(:show).and_return(@panel)
      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @panel.id
        response.should be_success
      end
      it 'コマモデルに単体取得を問い合わせている' do
        Panel.should_receive(:show).exactly(1)
        get :show
      end
      it '@panelにアレを取得している' do
        get :show, :id => @panel.id
        assigns(:panel).id.should eq(@panel.id)
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @panel.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @panel.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがアレになっている' do
          get :show, :id => @panel.id, :format => :json
          json = JSON.parse response.body
          json["width"].should eq 100
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :show, :id => @panel.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :show, :id => @panel.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :show, :id => @panel.id, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :show, :id => @panel.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
=begin
    context '対象コマがないとき' do
      context 'html形式' do
        it '例外404 not_foundを返す' do
          lambda{
            get :show, :id => 0
          }.should raise_error(ActiveRecord::RecordNotFound)
        end
      end
      context 'json形式' do
        it '例外404 not_foundを返す' do
          lambda{ 
            get :show, :id => 0, :format => :json
          }.should raise_error(ActiveRecord::RecordNotFound)
        end
      end
    end
    context '非公開コマを見ようとしたとき' do
      context 'html形式' do
        it '例外403 forbiddenを返す' do
          Panel.any_instance.stub(:visible?).with(any_args()).and_return(false)
          hidden = FactoryGirl.create :hidden_panel, :author_id => @author.id
          lambda{
            get :show, :id => hidden
          }.should raise_error(ActiveRecord::Forbidden)
        end
      end
      context 'json形式' do
        it '例外403 forbiddenを返す' do
          Panel.any_instance.stub(:visible?).with(any_args()).and_return(false)
          hidden = FactoryGirl.create :hidden_panel, :author_id => @author.id
          lambda{
            get :show, :id => hidden, :format => :json
          }.should raise_error(ActiveRecord::Forbidden)
        end
      end
    end
=end
  end
  
  describe 'コマ数取得に於いて' do
    before do
      Panel.should_receive(:visible_count).and_return(3)
#      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :count, :format => :json
        response.should be_success 
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :count, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがHash構造になっていてコマ数が1である' do
          get :count, :format => :json
          json = JSON.parse response.body
          json["count"].should == 3
        end
      end
    end
  end
  
  describe '新規作成フォーム表示に於いて' do
    before do
      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :new
        response.should be_success 
      end
      it '@panelに新規データを用意している' do
        get :new
        assigns(:panel).should be_a_new(Panel)
      end
      it 'コマモデルにデフォルト値補充を依頼している' do
        Panel.any_instance.should_receive(:supply_default).exactly(1)
        get :new
      end
      context 'html形式' do
        it 'newテンプレートを描画する' do
          get :new
          response.should render_template("new")
        end
      end
      context 'js形式' do
        it 'new.jsテンプレートを描画する' do
          get :new, :format => :js
          response.should render_template("new")
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :new
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :new
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'js形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :new, :format => :js
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :new, :format => :js
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
  describe '新規作成に於いて' do
    before do
      @panel = FactoryGirl.create :panel, :author_id => @user.author.id
      @attr = FactoryGirl.attributes_for(:panel, :author_id => @author.id)
      sign_in @user
    end
    context '事前チェックする' do
      before do
        controller
        Panel.stub(:count).and_return(10)
      end
      it 'panelがパラメータにあれば、展開する' do
        post :create, :panel => @attr
        assigns(:prm)['border'].to_i.should eq @panel.border
      end
      it 'jsonがパラメータにあれば、展開する' do
        post :create, :json => FactoryGirl.attributes_for(:panel, :border => 4).to_json
        assigns(:prm)['border'].to_i.should eq 4
      end
      it 'panel・json両パラメータがあれば、panelを優先して展開する' do
        post :create, {
          :panel => FactoryGirl.attributes_for(:panel), 
          :json => FactoryGirl.attributes_for(:panel, :border => 4).to_json
        }
        assigns(:prm)['border'].to_i.should eq @panel.border
      end
    end
    context 'つつがなく終わるとき' do
      it 'コマモデルにデフォルト値補充を依頼している' do
        Panel.any_instance.should_receive(:supply_default).exactly(1)
        post :create, :panel => @attr
      end
      it 'コマモデルに上書き補充を依頼している' do
        Panel.any_instance.should_receive(:overwrite).exactly(1)
        post :create, :panel => @attr
      end
      it 'モデルに保存依頼する' do
        Panel.any_instance.should_receive(:store).exactly(1)
        post :create, :panel => @attr
      end
      it "@panelに作成されたコマを保持していて、それがDBにある" do
        post :create, :panel => @attr
        assigns(:panel).should be_a(Panel)
        assigns(:panel).should be_persisted
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          Panel.any_instance.stub(:store).and_return(true)
          post :create, :panel => @attr
          response.status.should eq 302
        end
        it '作成されたコマの表示ページへ遷移する' do
#          Panel.any_instance.stub(:store).and_return(true)
          post :create, :panel => @attr
          response.should redirect_to(Panel.last)
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
#          Panel.any_instance.stub(:store).and_return(true)
          post :create, :panel => @attr, :format => :json
          response.should be_success 
        end
        it '作成されたコマをjsonデータで返す' do
          post :create, :panel => @attr, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがアレになっている' do
          post :create, :panel => @attr, :format => :json
          json = JSON.parse response.body
          json["width"].should eq @panel.width
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          post :create, :panel => @attr
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          post :create, :panel => @attr
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          post :create, :panel => @attr, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          post :create, :panel => @attr, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context '検証、保存に失敗した' do
      before do
        Panel.any_instance.stub(:store).and_return(false)
      end
      it "未保存のコマを保持している" do
        post :create, :panel => @attr
        assigns(:panel).should be_a_new(Panel)
      end
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          post :create, :panel => @attr
          response.status.should eq 200
        end
        it '新規ページを描画する' do
          post :create, :panel => @attr
          response.should render_template("new")
        end
      end
      context 'json形式' do
        it 'ステータスコード422 unprocessable_entity を返す' do
          post :create, :panel => @attr, :format => :json
          response.status.should eq 422
        end
        it '応答メッセージUnprocessable Entityを返す' do
          post :create, :panel => @attr, :format => :json
          response.message.should match(/Unprocessable/)
        end
      end
    end
  end

  describe '編集フォーム表示に於いて' do
    before do
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      sign_in @user
      Panel.stub(:show).and_return(@panel)
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :edit, :id => @panel.id
        response.should be_success 
      end
      it 'コマモデルに編集取得を問い合わせている' do
        Panel.should_receive(:edit).exactly(1)
        get :edit, :id => @panel.id
      end
      it '@panelにデータを用意している' do
        get :edit, :id => @panel.id
        assigns(:panel).should eq @panel
      end
      context 'html形式' do
        it 'editテンプレートを描画する' do
          get :edit, :id => @panel.id
          response.should render_template("edit")
        end
      end
      context 'js形式' do
        it 'edit.jsテンプレートを描画する' do
          get :edit, :id => @panel.id, :format => :js
          response.should render_template("edit")
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :edit, :id => @panel.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :edit, :id => @panel.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'js形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :edit, :id => @panel.id, :format => :js
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :edit, :id => @panel.id, :format => :js
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end

  describe '更新に於いて' do
    before do
      @panel = FactoryGirl.create :panel, :author_id => @user.author.id
      @attr = FactoryGirl.attributes_for(:panel, :author_id => @author.id)
      sign_in @user
    end
    context '事前チェックする' do
      before do
        controller
        Panel.stub(:count).and_return(10)
      end
      it 'panelがパラメータにあれば、展開する' do
        put :update, :id => @panel.id, :panel => @attr
        assigns(:prm)['border'].to_i.should eq @panel.border
      end
      it 'jsonがパラメータにあれば、展開する' do
        put :update, :id => @panel.id, :json => FactoryGirl.attributes_for(:panel, :border => 4).to_json
        assigns(:prm)['border'].to_i.should eq 4
      end
      it 'panel・json両パラメータがあれば、panelを優先して展開する' do
        put :update, {
          :id => @panel.id, 
          :panel => FactoryGirl.attributes_for(:panel), 
          :json => FactoryGirl.attributes_for(:panel, :border => 4).to_json
        }
        assigns(:prm)['border'].to_i.should eq @panel.border
      end
    end
    context 'つつがなく終わるとき' do
      it 'モデルに編集取得依頼する' do
        Panel.stub(:edit).with(any_args).and_return(@panel)
        Panel.should_receive(:edit).exactly(1)
        put :update, :id => @panel.id, :panel => @attr
      end
      it 'コマモデルに上書き補充を依頼している' do
        Panel.any_instance.should_receive(:overwrite).exactly(1)
        put :update, :id => @panel.id, :panel => @attr
      end
      it 'モデルに保存依頼する' do
        Panel.any_instance.should_receive(:store).exactly(1)
        put :update, :id => @panel.id, :panel => @attr
      end
      it "@panelに作成されたコマを保持していて、それがDBにある" do
        put :update, :id => @panel.id, :panel => @attr
        assigns(:panel).should be_a(Panel)
        assigns(:panel).should be_persisted
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          Panel.any_instance.stub(:store).and_return(true)
          put :update, :id => @panel.id, :panel => @attr
          response.status.should eq 302
        end
        it '作成されたコマの表示ページへ遷移する' do
#          Panel.any_instance.stub(:store).and_return(true)
          put :update, :id => @panel.id, :panel => @attr
          response.should redirect_to(Panel.last)
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
#          Panel.any_instance.stub(:store).and_return(true)
          put :update, :id => @panel.id, :panel => @attr, :format => :json
          response.should be_success 
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          put :update, :id => @panel.id, :panel => @attr
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          put :update, :id => @panel.id, :panel => @attr
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          put :update, :id => @panel.id, :panel => @attr, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          put :update, :id => @panel.id, :panel => @attr, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context '検証、保存に失敗した' do
      before do
        Panel.any_instance.stub(:store).and_return(false)
      end
      it "指定のコマを保持している" do
        put :update, :id => @panel.id, :panel => @attr
        assigns(:panel).should eq @panel
      end
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          put :update, :id => @panel.id, :panel => @attr
          response.status.should eq 200
        end
        it '編集ページを描画する' do
          put :update, :id => @panel.id, :panel => @attr
          response.should render_template("edit")
        end
      end
      context 'json形式' do
        it 'ステータスコード422 unprocessable_entity を返す' do
          put :update, :id => @panel.id, :panel => @attr, :format => :json
          response.status.should eq 422
        end
        it '応答メッセージUnprocessable Entityを返す' do
          put :update, :id => @panel.id, :panel => @attr, :format => :json
          response.message.should match(/Unprocessable/)
        end
      end
    end
  end

  describe '削除に於いて' do
    before do
      @panel = FactoryGirl.create :panel, :author_id => @user.author.id
      Panel.stub(:edit).and_return(@panel)
      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'コマモデルに編集取得を問い合わせている' do
        Panel.should_receive(:edit).exactly(1)
        delete :destroy, :id => @panel.id
      end
      it '@panelにアレを取得している' do
        delete :destroy, :id => @panel.id
        assigns(:panel).id.should eq(@panel.id)
      end
      it 'そのコマを一つのトランザクションで削除する' do
        lambda {
          delete :destroy, :id => @panel.id
        }.should change(Panel, :count)
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          delete :destroy, :id => @panel.id
          response.status.should eq 302
        end
        it 'コマ一覧ページへ遷移する' do
          delete :destroy, :id => @panel.id
          response.should redirect_to(panels_url)
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          delete :destroy, :id => @panel.id, :format => :json
          response.should be_success
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          delete :destroy, :id => @panel.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          delete :destroy, :id => @panel.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          delete :destroy, :id => @panel.id, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          delete :destroy, :id => @panel.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
=begin
    context '対象コマがないとき' do
    end
    context '他人のコマだったとき' do
    end
=end
  end
  
end

