# -*- encoding: utf-8 -*-
require 'spec_helper'
#セリフ
describe Balloon do
  before do
    FactoryGirl.create :admin
    @user = FactoryGirl.create( :user_yas)
    @author = @user.author
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
    
    @balloon = FactoryGirl.create :panel
    @speech_balloon_template = FactoryGirl.create :speech_balloon_template
    @speech_balloon = FactoryGirl.create :speech_balloon, :panel_id => @balloon.id, :speech_balloon_template_id => @speech_balloon_template.id
    @system_picture = FactoryGirl.create :system_picture
  end
  
  describe '検証に於いて' do
    before do
    end
    
    it 'オーソドックスなデータなら通る' do
      @balloon = FactoryGirl.build :balloon, :speech_balloon_id => @speech_balloon.id, :system_picture_id => @system_picture.id
      @balloon.should be_valid
    end
    
    context 'speech_balloon_idを検証するとき' do
      before do
        @balloon = FactoryGirl.build :balloon, :speech_balloon_id => @speech_balloon.id, :system_picture_id => @system_picture.id
      end
      #ネストの保存はnilを許可しなければならないので数値チェックだけ
      it 'テストデータの確認' do
        @balloon.speech_balloon_id = @speech_balloon.id
        @balloon.should be_valid
      end
      it '数値でなければ失敗する' do
        @balloon.speech_balloon_id = 'a'
        @balloon.should_not be_valid
      end
    end
    context 'system_picture_idを検証するとき' do
      before do
        @balloon = FactoryGirl.build :balloon, :speech_balloon_id => @speech_balloon.id, :system_picture_id => @system_picture.id
      end
      it 'テストデータの確認' do
        @balloon.system_picture_id = @system_picture.id
        @balloon.should be_valid
      end
      it 'nullなら失敗する' do
        @balloon.system_picture_id = nil
        @balloon.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @balloon.system_picture_id = 'a'
        @balloon.should_not be_valid
      end
      it '存在するシステム画像でなければ失敗する' do
        @balloon.system_picture_id = 0
        @balloon.should_not be_valid
      end
    end
    context 'xを検証するとき' do
      before do
        @balloon = FactoryGirl.build :balloon, :speech_balloon_id => @speech_balloon.id, :system_picture_id => @system_picture.id
      end
      it 'テストデータの確認' do
        @balloon.x = '1'
        @balloon.should be_valid
      end
      it 'nullなら失敗する' do
        @balloon.x = nil
        @balloon.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @balloon.x = 'a'
        @balloon.should_not be_valid
      end
      it '0なら通る' do
        @balloon.x = '0'
        @balloon.should be_valid
      end
      it '負でも通る' do
        @balloon.x = -1
        @balloon.should be_valid
      end
    end
    context 'yを検証するとき' do
      before do
        @balloon = FactoryGirl.build :balloon, :speech_balloon_id => @speech_balloon.id, :system_picture_id => @system_picture.id
      end
      it 'テストデータの確認' do
        @balloon.y = '1'
        @balloon.should be_valid
      end
      it 'nullなら失敗する' do
        @balloon.y = nil
        @balloon.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @balloon.y = 'a'
        @balloon.should_not be_valid
      end
      it '0なら通る' do
        @balloon.y = '0'
        @balloon.should be_valid
      end
      it '負でも通る' do
        @balloon.y = -1
        @balloon.should be_valid
      end
    end
    context 'widthを検証するとき' do
      before do
        @balloon = FactoryGirl.build :balloon, :speech_balloon_id => @speech_balloon.id, :system_picture_id => @system_picture.id
      end
      it 'テストデータの確認' do
        @balloon.width = 1
        @balloon.should be_valid
      end
      it 'nullなら失敗する' do
        @balloon.width = nil
        @balloon.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @balloon.width = 'a'
        @balloon.should_not be_valid
      end
      it '0なら失敗する' do
        @balloon.width = '0'
        @balloon.should_not be_valid
      end
      it '負でも失敗する' do
        @balloon.width = -1
        @balloon.should_not be_valid
      end
    end
    context 'heightを検証するとき' do
      before do
        @balloon = FactoryGirl.build :balloon, :speech_balloon_id => @speech_balloon.id, :system_picture_id => @system_picture.id
      end
      it 'テストデータの確認' do
        @balloon.height = '1'
        @balloon.should be_valid
      end
      it 'nullなら失敗する' do
        @balloon.height = nil
        @balloon.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @balloon.height = 'a'
        @balloon.should_not be_valid
      end
      it '0なら失敗する' do
        @balloon.height = '0'
        @balloon.should_not be_valid
      end
      it '負でも失敗する' do
        @balloon.height = -1
        @balloon.should_not be_valid
      end
    end
    context 'settingsを検証するとき' do
      before do
        @balloon = FactoryGirl.build :balloon, :speech_balloon_id => @speech_balloon.id, :system_picture_id => @system_picture.id
      end
      it 'テストデータの確認' do
        @balloon.settings = ''
        @balloon.should be_valid
      end
    end
  end
  
  describe 'データ補充に於いて' do
    before do
    end
    
  end
  
end
