# -*- encoding: utf-8 -*-
#実素材
require 'spec_helper'

describe Picture do
  before do
    FactoryGirl.create :admin
    @user = FactoryGirl.create( :user_yas)
    @author = @user.author
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
    @other_user = FactoryGirl.create( :user_yas)
    @other_author = @other_user.author
    @other_artist = FactoryGirl.create :artist_yas, :author_id => @other_author.id
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
  end
  
  describe '検証に於いて' do
    before do
    end
    
    it 'オーソドックスなデータなら通る' do
      @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      @p.should be_valid
    end
    
    context 'original_picture_idを検証するとき' do
      before do
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @p.original_picture_id = @op.id
        @p.should be_valid
      end
      it 'nullなら失敗する' do
        @p.original_picture_id = nil
        @p.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @p.original_picture_id = 'a'
        @p.should_not be_valid
      end
      it '存在する原画でなければ失敗する' do
        @p.original_picture_id = 0
        @p.should_not be_valid
      end
    end
    context 'revisionを検証するとき' do
      before do
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @p.revision = 1
        @p.should be_valid
      end
      it 'nullなら失敗する' do
        @p.revision = nil
        @p.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @p.revision = 'a'
        @p.should_not be_valid
      end
    end
    context 'extを検証するとき' do
      before do
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @p.ext = 'jpeg'
        @p.should be_valid
      end
      it 'nullなら失敗する' do
        @p.ext = ''
        @p.should_not be_valid
      end
      it '5文字以上なら失敗する' do
        @p.ext = 'a'*5
        @p.should_not be_valid
      end
      it 'png,gif,jpeg以外なら失敗する' do
        @p.ext = 'bmp'
        @p.should_not be_valid
      end
    end
    context 'widthを検証するとき' do
      before do
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @p.width = 1
        @p.should be_valid
      end
      it 'nullなら失敗する' do
        @p.width = nil
        @p.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @p.width = 'a'
        @p.should_not be_valid
      end
      it '0なら失敗する' do
        @p.width = '0'
        @p.should_not be_valid
      end
      it '負でも失敗する' do
        @p.width = -1
        @p.should_not be_valid
      end
    end
    context 'heightを検証するとき' do
      before do
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @p.height = 1
        @p.should be_valid
      end
      it 'nullなら失敗する' do
        @p.height = nil
        @p.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @p.height = 'a'
        @p.should_not be_valid
      end
      it '0なら失敗する' do
        @p.height = '0'
        @p.should_not be_valid
      end
      it '負でも失敗する' do
        @p.height = -1
        @p.should_not be_valid
      end
    end
    context 'filesizeを検証するとき' do
      before do
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @p.filesize = 1
        @p.should be_valid
      end
      it 'nullなら失敗する' do
        @p.filesize = nil
        @p.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @p.filesize = 'a'
        @p.should_not be_valid
      end
      it '負なら失敗する' do
        @p.filesize = '-1'
        @p.should_not be_valid
      end
      it '2MB以上なら失敗する' do
        @p.filesize = 2000000+1
        @p.should_not be_valid
      end
    end
    context 'artist_idを検証するとき' do
      before do
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @p.artist_id = @artist.id
        @p.should be_valid
      end
      it 'nullなら失敗する' do
        @p.artist_id = nil
        @p.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @p.artist_id = 'a'
        @p.should_not be_valid
      end
      it '存在する絵師でなければ失敗する' do
        @p.artist_id = 0
        @p.should_not be_valid
      end
    end
    context 'md5を検証するとき' do
      before do
        @p = FactoryGirl.build :picture
      end
      it 'テストデータの確認' do
        @p.md5 = 'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
        @p.should be_valid
      end
      it 'nullなら失敗する' do
        @p.md5 = ''
        @p.should_not be_valid
      end
      it '32文字以上なら失敗する' do
        @p.md5 = 'a'*33
        @p.should_not be_valid
      end
    end
    context 'license_idを検証するとき' do
      before do
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @p.license_id = @license.id
        @p.should be_valid
      end
      it 'nullなら失敗する' do
        @p.license_id = nil
        @p.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @p.license_id = 'a'
        @p.should_not be_valid
      end
      it '存在するライセンスグループでなければ失敗する' do
        @p.license_id = 0
        @p.should_not be_valid
      end
    end
    context 'artist_nameを検証するとき' do
      before do
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @p.artist_name = 'a'
        @p.should be_valid
      end
      it 'nullなら失敗する' do
        @p.artist_name = nil
        @p.should_not be_valid
      end
    end
    context 'classnameを検証するとき' do
      before do
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @p.classname = 'a'*50
        @p.should be_valid
      end
      it 'nullなら失敗する' do
        @p.classname = ''
        @p.should_not be_valid
      end
      it '51文字以上なら失敗する' do
        @p.classname = 'a'*51
        @p.should_not be_valid
      end
    end
    context 'creditを検証するとき' do
      before do
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @p.credit = 'a'
        @p.should be_valid
      end
    end
    context 'settingsを検証するとき' do
      before do
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'テストデータの確認' do
        @p.settings = 'a'
        @p.should be_valid
      end
    end
  end
  describe '補充に於いて' do
  end
  describe '最新Revision取得に於いて' do
    context '初めての原画を公開したとき' do
      before do
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'Revisionは1となる' do
        @p.new_revision.should eq 1
      end
    end
    context 'HEADが1のとき' do
      before do
        FactoryGirl.create :picture, :revision => 1, :original_picture_id => @op.id, :license_id => @license.id
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'Revisionは2となる' do
        @p.new_revision.should eq 2
      end
    end
    context 'HEADが5のとき' do
      before do
        FactoryGirl.create :picture, :revision => 1, :original_picture_id => @op.id, :license_id => @license.id
        FactoryGirl.create :picture, :revision => 5, :original_picture_id => @op.id, :license_id => @license.id
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'Revisionは6となる' do
        @p.new_revision.should eq 6
      end
    end
  end
  describe '素材からのコピーデータセットに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      attr = {:ext => 'jpeg', :width => 264, :height => 265, :filesize => 266, 
        :artist_id => @artist.id, :original_picture_id => @op.id, :license_id => @license.id, 
        :artist_name => 'tester', :classname => 'Tester', :credit => {:title => 'cap'}.to_json.to_s, :settings => {:set => 1}.to_json.to_s}
      @rp = FactoryGirl.build :resource_picture, attr
      @p = FactoryGirl.build :picture
    end
    it '素材オブジェクトから属性を取り出して対象実素材にセットしている' do
      res = @p.copy_data(@rp)
      @p.ext.should eq 'jpeg'
      @p.width.should eq 264
      @p.height.should eq 265
      @p.filesize.should eq 266
      @p.artist_id.should eq @artist.id
      @p.md5.should eq @rp.md5
      @p.license_id.should eq @license.id
      @p.original_picture_id.should eq @op.id
      @p.artist_name.should eq 'tester'
      @p.classname.should eq 'Tester'
      @p.credit.should match /title/
      @p.settings.should match /set/
    end
  end
  
  describe 'md5重複リストに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :artist_id => @artist.id, :license_id => @license.id, :revision => 0,
        :original_picture_id => @op.id, :md5 => 'a' * 32
      @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :original_picture_id => @op.id, :license_id => @license.id, :picture_id => @p.id
      @op2 = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p2 = FactoryGirl.create :picture, :artist_id => @artist.id, :license_id => @license.id, :revision => 0,
        :original_picture_id => @op2.id, :md5 => 'b' * 32
      @op3 = FactoryGirl.create :original_picture, :artist_id => @artist.id
    end
    it 'リストを返す' do
      res = Picture.list_by_md5(@p.md5, @p.original_picture_id)
      res.is_a?(Array).should be_true
    end
    it 'md5が違えば含まない' do
      res = Picture.list_by_md5(@p.md5, @p.original_picture_id)
      res.include?(@p2).should be_false
    end
    it '同一原画は含まない' do
      res = Picture.list_by_md5(@p.md5, @p.original_picture_id)
      res.empty?.should be_true
    end
    it '同一原画は旧版でも含まない' do
      @p3 = FactoryGirl.create :picture, :artist_id => @artist.id, :license_id => @license.id, :revision => 1,
        :original_picture_id => @op.id, :md5 => 'a' * 32
      res = Picture.list_by_md5(@p.md5, @p.original_picture_id)
      res.empty?.should be_true
    end
    it '他所の原画なら含む' do
      @p3 = FactoryGirl.create :picture, :artist_id => @artist.id, :license_id => @license.id, :revision => 1,
        :original_picture_id => @op2.id, :md5 => 'a' * 32
      res = Picture.list_by_md5(@p.md5, @p.original_picture_id)
      res.should eq [@p3]
    end
    it '他所の原画でもmd5が違えば含まない' do
      @p3 = FactoryGirl.create :picture, :artist_id => @artist.id, :license_id => @license.id, :revision => 1,
        :original_picture_id => @op2.id, :md5 => 'c' * 32
      res = Picture.list_by_md5(@p.md5, @p.original_picture_id)
      res.empty?.should be_true
    end
    it '更新日時順' do
      @p3 = FactoryGirl.create :picture, :artist_id => @artist.id, :license_id => @license.id, :revision => 1,
        :original_picture_id => @op2.id, :md5 => 'a' * 32
      @p4 = FactoryGirl.create :picture, :artist_id => @artist.id, :license_id => @license.id, :revision => 1,
        :original_picture_id => @op3.id, :md5 => 'a' * 32, :updated_at => Time.now + 100
      res = Picture.list_by_md5(@p.md5, @p.original_picture_id)
      res.should eq [@p4, @p3]
    end
  end
  
  describe 'md5重複判定に於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @op2 = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :artist_id => @artist.id, :original_picture_id => @op.id, :license_id => @license.id, :revision => 0
      @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :original_picture_id => @op.id, :license_id => @license.id, :picture_id => @p.id
    end
    context '同一原画以外に同じ値があるとき' do
      before do
        @p2 = FactoryGirl.create :picture, :artist_id => @artist.id, :original_picture_id => @op2.id, :license_id => @license.id
      end
      it 'trueを返す' do
        res = Picture.exist_by_md5(@p.md5, @p.original_picture_id)
        res.should be_true
      end
    end
    context '同一原画以外に同じ値がないとき' do
      it 'falseを返す' do
        res = Picture.exist_by_md5(@p.md5, @p.original_picture_id)
        res.should be_false
      end
    end
    context '同一原画に同じ値があるとき' do
      before do
        @p2 = FactoryGirl.create :picture, :artist_id => @artist.id, :original_picture_id => @op.id, :license_id => @license.id, :revision => 1
      end
      it 'falseを返す' do
        res = Picture.exist_by_md5(@p.md5, @p.original_picture_id)
        res.should be_false
      end
    end
  end
  
  describe 'head判定に於いて' do
    before do
    end
    context '自身とリンクした素材があるとき' do
      before do
        @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
        @p = FactoryGirl.create :picture, :artist_id => @artist.id, :original_picture_id => @op.id, :license_id => @license.id 
        @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :original_picture_id => @op.id, :license_id => @license.id, :picture_id => @p.id
      end
      it 'trueを返す' do
        res = @p.head?
        res.should be_true
      end
    end
    context '自身とリンクした素材がないとき' do
      before do
        @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
        @p = FactoryGirl.create :picture, :artist_id => @artist.id, :original_picture_id => @op.id, :license_id => @license.id, :revision => 0
        @p2 = FactoryGirl.create :picture, :artist_id => @artist.id, :original_picture_id => @op.id, :license_id => @license.id, :revision => 1
        @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :original_picture_id => @op.id, :license_id => @license.id, :picture_id => @p2.id
      end
      it 'falseを返す' do
        res = @p.head?
        res.should be_false
      end
    end
  end
  
  describe '作成に於いて' do
    before do
      #RMagickのスタブをおいておく
      class Mgk ; class Image ; end ; end
      @filesize = 76543
      Mgk::Image.stub(:from_blob).with(any_args).and_return([Mgk.new])
      Mgk.any_instance.stub(:format).with(any_args).and_return('png')
      Mgk.any_instance.stub(:rows).with(any_args).and_return(200)
      Mgk.any_instance.stub(:columns).with(any_args).and_return(100)
      Mgk.any_instance.stub(:filesize).with(any_args).and_return(@filesize)
      Mgk.any_instance.stub(:to_blob).with(any_args).and_return('data')
      #原画ファイル削除だけは必ず成功するものとしておく
      PictureIO::LocalPicture.any_instance.stub(:delete).with(any_args).and_return(true)
    end
    context '事前チェック' do
      before do
        #すべての処理が正常パターンで通過すれば、一番深い分岐まで通る。
        #それで外部のメソッド呼び出しだけに着目してテストする。
        Picture.any_instance.stub(:save).with(any_args).and_return(true)
        Picture.any_instance.stub(:store_picture).with(any_args).and_return(true)
        Picture.any_instance.stub(:to_gif?).with(any_args).and_return(true)
        class GifMgk < Mgk ; end  #store_pictureは二回呼び出される。区別をつけるために
        @gifmgk = GifMgk.new      #パラメータを二種類用意する。
        Picture.stub(:png_to_gif).with(any_args).and_return(@gifmgk)
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it '最新Revisionを取得している' do
        Picture.any_instance.should_receive(:new_revision).with(any_args).exactly(1)
        res = @p.store(Mgk.new)
      end
      it '自身を保存している' do
        Picture.any_instance.should_receive(:save).with(any_args).exactly(1)
        res = @p.store(Mgk.new)
      end
      it '画像ファイルの作成機能で画像を保存している' do
        mgk = Mgk.new   #一回目の本画像保存は与えたオブジェクトを使って保存する。
        Picture.any_instance.stub(:filename).with(any_args).and_return('1.png')
        Picture.any_instance.should_receive(:store_picture).with(mgk, '1.png').exactly(1)
        res = @p.store(mgk)
      end
      it '自身にgifフォーマット変換対象かを問い合わせている' do
        Picture.any_instance.should_receive(:to_gif?).with(any_args).exactly(1)
        res = @p.store(Mgk.new)
      end
      it '自身にGifフォーマット変換を依頼している' do
        Picture.should_receive(:png_to_gif).with(any_args).exactly(1)
        res = @p.store(Mgk.new)
      end
      it '画像ファイルの作成機能でgif画像を保存している' do
        #二回目の保存はgif変換の結果を渡す。
        Picture.any_instance.stub(:gifname).with(any_args).and_return('1.gif')
        Picture.any_instance.should_receive(:store_picture).with(@gifmgk, '1.gif').exactly(1)
        res = @p.store(Mgk.new)
      end
    end
    context 'つつがなく終わるとき' do
      before do
        #すべての処理を正常パターンで通過させ、保存機能をチェックする。
#        Picture.any_instance.stub(:save).with(any_args).and_return(true)
        Picture.any_instance.stub(:store_picture).with(any_args).and_return(true)
        Picture.any_instance.stub(:to_gif?).with(any_args).and_return(true)
        class GifMgk < Mgk ; end  #store_pictureは二回呼び出される。区別をつけるために
        @gifmgk = GifMgk.new      #パラメータを二種類用意する。
        Picture.stub(:png_to_gif).with(any_args).and_return(@gifmgk)
        @p = FactoryGirl.build :picture, :revision => nil, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'Trueを返す' do
        res = @p.store(Mgk.new)
        res.should be_true
      end
      it 'Revisionに最新Revisionセット' do
        res = @p.store(Mgk.new)
        @p.revision.should eq 1
      end
      it '自身が保存されている' do
        lambda {
          res = @p.store(Mgk.new)
        }.should change Picture, :count
      end
    end
    context 'gif変換なしで、つつがなく終わるとき' do
      before do
        #すべての処理を正常パターンで通過させ、保存機能をチェックする。
        Picture.any_instance.stub(:save).with(any_args).and_return(true)
        Picture.any_instance.stub(:store_picture).with(any_args).and_return(true)
        Picture.any_instance.stub(:to_gif?).with(any_args).and_return(false)
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'Trueを返す' do
        res = @p.store(Mgk.new)
        res.should be_true
      end
      it 'gif保存は呼ばれていない' do
        #二回目の画像作成が呼び出されないで1回こっきりならgif保存は呼ばれていないんだろう。
        Picture.any_instance.should_receive(:store_picture).with(any_args).exactly(1)
        res = @p.store(Mgk.new)
      end
    end
    #以下から例外ケース。処理先頭から失敗させていく
    context '自身の保存に失敗したとき' do
      before do
        Picture.any_instance.stub(:save).with(any_args).and_return(false)
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'Falseを返す' do
        res = @p.store(Mgk.new)
        res.should be_false
      end
      it '更新されていない' do
        @p.store(Mgk.new)
        @p.should be_a_new Picture
      end
    end
    context '画像の保存に失敗したとき' do
      before do
        Picture.any_instance.stub(:save).with(any_args).and_return(true)
        Picture.any_instance.stub(:store_picture).with(any_args).and_return(false)
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'Falseを返す' do
        res = @p.store(Mgk.new)
        res.should be_false
      end
      it 'gif変換判定は呼ばれていない' do
        Picture.any_instance.should_not_receive(:to_gif?).with(any_args)
        res = @p.store(Mgk.new)
      end
    end
    context 'gif変換に失敗したとき' do
      before do
        Picture.any_instance.stub(:save).with(any_args).and_return(true)
        Picture.any_instance.stub(:store_picture).with(any_args).and_return(true)
        Picture.any_instance.stub(:to_gif?).with(any_args).and_return(true)
        Picture.stub(:png_to_gif).with(any_args).and_return(false)
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'Falseを返す' do
        res = @p.store(Mgk.new)
        res.should be_false
      end
      it 'gif画像の保存は呼ばれていない' do
        #本画像の保存があるので、一度は呼ばれる
        Picture.any_instance.should_receive(:store_picture).with(any_args).exactly(1)
        res = @p.store(Mgk.new)
      end
    end
    context 'gif画像の保存に失敗したとき' do
      before do
        @mgk = Mgk.new
        Picture.any_instance.stub(:filename).with(any_args).and_return('1.png')
        Picture.any_instance.stub(:gifname).with(any_args).and_return('1.gif')
        Picture.any_instance.stub(:save).with(any_args).and_return(true)
        Picture.any_instance.stub(:store_picture).with(@mgk, '1.png').and_return(true)
        Picture.any_instance.stub(:to_gif?).with(any_args).and_return(true)
        class GifMgk < Mgk ; end  #store_pictureは二回呼び出される。区別をつけるために
        @gifmgk = GifMgk.new      #パラメータを二種類用意する。
        Picture.stub(:png_to_gif).with(any_args).and_return(@gifmgk)
        Picture.any_instance.stub(:store_picture).with(@gifmgk, '1.gif').and_return(false)
        @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
      end
      it 'Falseを返す' do
        res = @p.store(@mgk)
        res.should be_false
      end
    end
  end
  
  describe '反転画像の保存先に於いて' do
    before do
      @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
    end
    context '禁止のとき' do
      it 'ベースディレクトリだけを返す' do
        Picture.any_instance.should_receive(:flag_reverse).with(any_args).and_return(-1)
        r = @p.subdirs
        r.should have(1).item 
      end
      it 'ベースディレクトリだけを返す' do
        Picture.any_instance.should_receive(:flag_reverse).with(any_args).and_return(-1)
        r = @p.subdirs
        r.first.should eq ''
      end
    end
    context '許可のとき' do
      it 'ベースディレクトリと反転3種を返す' do
        Picture.any_instance.should_receive(:flag_reverse).with(any_args).and_return(1)
        r = @p.subdirs
        r.should have(4).item 
      end
      it 'ベースディレクトリと反転3種を返す' do
        Picture.any_instance.should_receive(:flag_reverse).with(any_args).and_return(1)
        r = @p.subdirs
        r.last.should eq 'vh'
      end
    end
  end
  
  describe '画像ファイルの作成に於いて' do
    #PictureIo経由で画像を保存するための機能。ファイル名に自身のidを使うので事前に自身の保存が必要。
    before do
      #RMagickのスタブをおいておく
      class Mgk ; class Image ; end ; end
      @filesize = 76543
      Mgk::Image.stub(:from_blob).with(any_args).and_return([Mgk.new])
      Mgk.any_instance.stub(:format).with(any_args).and_return('png')
      Mgk.any_instance.stub(:rows).with(any_args).and_return(200)
      Mgk.any_instance.stub(:columns).with(any_args).and_return(100)
      Mgk.any_instance.stub(:filesize).with(any_args).and_return(@filesize)
      Mgk.any_instance.stub(:to_blob).with(any_args).and_return('data')
      #原画ファイル削除だけは必ず成功するものとしておく
      PictureIO::LocalPicture.any_instance.stub(:delete).with(any_args).and_return(true)
      
      Picture.any_instance.stub(:subdirs).with(any_args).and_return(['', 'v', 'h', 'vh'])
      Picture.any_instance.stub(:h).with(any_args).and_return('data')
      Picture.any_instance.stub(:v).with(any_args).and_return('data')
      Picture.any_instance.stub(:vh).with(any_args).and_return('data')
      @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id
    end
    context '事前チェック' do
      before do
        #すべての処理を正常パターンで通過させ、保存機能をチェックする。
        PictureIO.picture_io.stub(:put).with(any_args).and_return(true)
      end
      it '画像ファイルの保存が4回呼ばれる' do
        PictureIO.picture_io.should_receive(:put).with(any_args).exactly(4)
        res = @p.store_picture(Mgk.new, '1.png')
      end
      it '画像ファイルのベースへの保存が1回呼ばれる' do
        PictureIO.picture_io.should_receive(:put).with('data', '1.png', '').exactly(1)
        res = @p.store_picture(Mgk.new, '1.png')
      end
      it '画像ファイルの垂直反転への保存が1回呼ばれる' do
        PictureIO.picture_io.should_receive(:put).with('data', '1.png', 'v').exactly(1)
        res = @p.store_picture(Mgk.new, '1.png')
      end
      it '画像ファイルの水平反転への保存が1回呼ばれる' do
        PictureIO.picture_io.should_receive(:put).with('data', '1.png', 'h').exactly(1)
        res = @p.store_picture(Mgk.new, '1.png')
      end
      it '画像ファイルの垂直水平反転への保存が1回呼ばれる' do
        PictureIO.picture_io.should_receive(:put).with('data', '1.png', 'vh').exactly(1)
        res = @p.store_picture(Mgk.new, '1.png')
      end
      it '垂直反転が1回呼ばれる' do
        Picture.any_instance.should_receive(:v).with(any_args).exactly(1)
        res = @p.store_picture(Mgk.new, '1.png')
      end
      it '水平反転が1回呼ばれる' do
        Picture.any_instance.should_receive(:h).with(any_args).exactly(1)
        res = @p.store_picture(Mgk.new, '1.png')
      end
      it '垂直水平反転が1回呼ばれる' do
        Picture.any_instance.should_receive(:vh).with(any_args).exactly(1)
        res = @p.store_picture(Mgk.new, '1.png')
      end
    end
    context 'つつがなく終わるとき' do
      before do
        #すべての処理を正常パターンで通過させ、保存機能をチェックする。
        PictureIO.picture_io.stub(:put).with(any_args).and_return(true)
      end
      it 'Trueを返す' do
        res = @p.store_picture(Mgk.new, '1.png')
        res.should be_true
      end
    end
    context '例外ケース' do
      before do
        PictureIO.picture_io.stub(:put).with(any_args).and_return(false)
      end
      it 'Falseを返す' do
        res = @p.store_picture(Mgk.new, '1.png')
        res.should be_false
      end
    end
    
  end
  
  describe 'フラグ展開に於いて' do
    before do
      @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id,
        :settings => "{\"open\": 1, \"commercial\": 2, \"official\": 3, \"attribution\": 4, \"derive\": 5, \"thumbnail\": 6, \"gif_convert\": 7, \"reverse\": 8, \"resize\": 9, \"sync_vh\": 10, \"overlap\": 11}"
    end
    context 'json展開チェック' do
      it '展開してなければ展開して@flagsに保管する' do
        @p.flags.should_not be_nil
      end
      it '展開できなければ{}' do
        @p.settings += '}'
        @p.flags.is_a?(Hash).should be_true
        @p.flags.empty?.should be_true
      end
    end
    context 'openについて' do
      it '@flag_openに保管する' do
        @p.flag_open.should_not be_nil
      end
      it '本当にフラグHashからじゃなく、変数から取得してる？' do
        @p.flag_open
        @p.flags = nil
        @p.flag_open.should_not be_nil
      end
      it '1を返す' do
        @p.flag_open.should eq 1
      end
    end
    context 'commercialについて' do
      it '@flag_commercialに保管する' do
        @p.flag_commercial.should_not be_nil
      end
      it '本当にフラグHashからじゃなく、変数から取得してる？' do
        @p.flag_commercial
        @p.flags = nil
        @p.flag_commercial.should_not be_nil
      end
      it '2を返す' do
        @p.flag_commercial.should eq 2
      end
    end
    context 'officialについて' do
      it '@flag_officialに保管する' do
        @p.flag_official.should_not be_nil
      end
      it '本当にフラグHashからじゃなく、変数から取得してる？' do
        @p.flag_official
        @p.flags = nil
        @p.flag_official.should_not be_nil
      end
      it 'を返す' do
        @p.flag_official.should eq 3
      end
    end
    context 'attributionについて' do
      it '@flag_attributionに保管する' do
        @p.flag_attribution.should_not be_nil
      end
      it '本当にフラグHashからじゃなく、変数から取得してる？' do
        @p.flag_attribution
        @p.flags = nil
        @p.flag_attribution.should_not be_nil
      end
      it '4を返す' do
        @p.flag_attribution.should eq 4
      end
    end
    context 'deriveについて' do
      it '@flag_deriveに保管する' do
        @p.flag_derive.should_not be_nil
      end
      it '本当にフラグHashからじゃなく、変数から取得してる？' do
        @p.flag_derive
        @p.flags = nil
        @p.flag_derive.should_not be_nil
      end
      it '5を返す' do
        @p.flag_derive.should eq 5
      end
    end
    context 'thumbnailについて' do
      it '@flag_thumbnailに保管する' do
        @p.flag_thumbnail.should_not be_nil
      end
      it '本当にフラグHashからじゃなく、変数から取得してる？' do
        @p.flag_thumbnail
        @p.flags = nil
        @p.flag_thumbnail.should_not be_nil
      end
      it '6を返す' do
        @p.flag_thumbnail.should eq 6
      end
    end
    context 'gif_convertについて' do
      it '@flag_gif_convertに保管する' do
        @p.flag_gif_convert.should_not be_nil
      end
      it '本当にフラグHashからじゃなく、変数から取得してる？' do
        @p.flag_gif_convert
        @p.flags = nil
        @p.flag_gif_convert.should_not be_nil
      end
      it '7を返す' do
        @p.flag_gif_convert.should eq 7
      end
    end
    context 'reverseについて' do
      it '@flag_reverseに保管する' do
        @p.flag_reverse.should_not be_nil
      end
      it '本当にフラグHashからじゃなく、変数から取得してる？' do
        @p.flag_reverse
        @p.flags = nil
        @p.flag_reverse.should_not be_nil
      end
      it '8を返す' do
        @p.flag_reverse.should eq 8
      end
    end
    context 'resizeについて' do
      it '@flag_resizeに保管する' do
        @p.flag_resize.should_not be_nil
      end
      it '本当にフラグHashからじゃなく、変数から取得してる？' do
        @p.flag_resize
        @p.flags = nil
        @p.flag_resize.should_not be_nil
      end
      it '9を返す' do
        @p.flag_resize.should eq 9
      end
    end
    context 'sync_vhについて' do
      it '@flag_sync_vhに保管する' do
        @p.flag_sync_vh.should_not be_nil
      end
      it '本当にフラグHashからじゃなく、変数から取得してる？' do
        @p.flag_sync_vh
        @p.flags = nil
        @p.flag_sync_vh.should_not be_nil
      end
      it '10を返す' do
        @p.flag_sync_vh.should eq 10
      end
    end
    context 'overlapについて' do
      it '@flag_overlapに保管する' do
        @p.flag_overlap.should_not be_nil
      end
      it '本当にフラグHashからじゃなく、変数から取得してる？' do
        @p.flag_overlap
        @p.flags = nil
        @p.flag_overlap.should_not be_nil
      end
      it '11を返す' do
        @p.flag_overlap.should eq 11
      end
    end
  end

  describe 'クレジットデータに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id, :credit => '{"system_picture_id": 2}'
    end
    it 'system_picture_idが入っている' do
      res = @p.credit_data
      res["system_picture_id"].should eq 2
    end
  end
  
end
